@@ -31,9 +31,14 @@
 
 #define DONT_DEFINE_VOID
 
-#if (MYSQLCLIENT_STATIC_LINKING)
-#if (MYSQL8)
+#define my_bool bool
+#define mysys_end my_end
+#define TRUE 1
+#define FALSE 0
+
 #define WIN32_LEAN_AND_MEAN
+
+#if (MYSQLCLIENT_STATIC_LINKING)
 #include <my_config.h>
 #include <my_sys.h>
 #include <mysql.h>
@@ -42,49 +47,28 @@
 #include <mysql/service_mysql_alloc.h>
 #include <m_ctype.h>
 #include <my_io.h>
+#include <thr_mutex.h>
 
-#define my_bool bool
-#define mysys_end my_end
+#else
 
-#ifndef TRUE
-#define TRUE 1
-#endif
+#include "include/mysql-8.0/my_config.h"
+#include "include/mysql-8.0/my_sys.h"
+#include <mysql.h>
+#include <mysqld_error.h>
+#include "include/mysql-8.0/my_alloc.h"
+#include "include/mysql-8.0/mysql/service_mysql_alloc.h"
+#include "include/mysql-8.0/m_ctype.h"
+#include "include/mysql-8.0/my_io.h"
+#include "include/mysql-8.0/thr_mutex.h"
 
-#ifndef FALSE
-#define FALSE 0
 #endif
 
 #ifdef _WIN32
 typedef DWORD thread_local_key_t;
 typedef CRITICAL_SECTION native_mutex_t;
 typedef int native_mutexattr_t;
-
 #else
 typedef pthread_key_t thread_local_key_t;
-//typedef pthread_mutex_t native_mutex_t;
-//typedef pthread_mutexattr_t native_mutexattr_t;
-#endif
-
-
-#else // MYSQL8
-
-#include <my_global.h>
-#include <mysql.h>
-#include <my_sys.h>
-#include <my_list.h>
-#include <m_string.h>
-#include <mysqld_error.h>
-#endif // MYSQL8
-
-#else
-
-#include "include/sys/my_global.h"
-#include "include/sys/my_thread.h"
-#include <mysql.h>
-#include "include/sys_main.h"
-#include <mysqld_error.h>
-#define myodbc_qsort my_qsort
-
 #endif
 
 #ifdef __cplusplus
@@ -99,22 +83,31 @@ extern "C"
 # error "Connector/ODBC requires v4.1 (or later) of the MySQL client library"
 #endif
 
-#ifdef MYSQLCLIENT_STATIC_LINKING
+#if !MYSQLCLIENT_STATIC_LINKING || MYSQL_VERSION_ID >= 80018
+
+  /*
+    Note: Things no longer defined in client library headers, but still used
+    by ODBC code.
+  */
+
+  #define set_if_bigger(a, b)   \
+    do {                        \
+      if ((a) < (b)) (a) = (b); \
+    } while (0)
+
+  #define set_if_smaller(a, b)  \
+    do {                        \
+      if ((a) > (b)) (a) = (b); \
+    } while (0)
+
+  static inline void reset_dynamic(DYNAMIC_ARRAY* array) { array->elements = 0; }
+
+#endif
+
 
 #define my_sys_init my_init
-#define myodbc_malloc(A,B) my_malloc(PSI_NOT_INSTRUMENTED,A,B)
-#ifndef x_free
 #define x_free(A) { void *tmp= (A); if (tmp) my_free((char *) tmp); }
-#endif
-
-#else
-
-#define myodbc_malloc(A,B) mysys_malloc(A,B)
-#ifndef x_free
-#define x_free(A) { void *tmp= (A); if (tmp) mysys_free((char *) tmp); }
-#endif
-
-#endif
+#define myodbc_malloc(A,B) my_malloc(PSI_NOT_INSTRUMENTED,A,B)
 
 #define myodbc_mutex_t native_mutex_t
 #define myodbc_key_t thread_local_key_t
@@ -130,7 +123,7 @@ extern "C"
 #define sort_dynamic(A,cmp) myodbc_qsort((A)->buffer, (A)->elements, (A)->size_of_element, (cmp))
 #define push_dynamic(A,B) insert_dynamic((A),(B))
 
-#define myodbc_snprintf my_snprintf
+#define myodbc_snprintf snprintf
 
   static my_bool inline myodbc_allocate_dynamic(DYNAMIC_ARRAY *array, uint max_elements)
   {
@@ -174,6 +167,10 @@ extern "C"
       (array->elements - idx)*array->size_of_element);
   }
 
+  static inline void *alloc_root(MEM_ROOT *root, size_t length) {
+    return root->Alloc(length);
+  }
+
 
 /* Get rid of defines from my_config.h that conflict with our myconf.h */
 #ifdef VERSION
