@@ -1,24 +1,30 @@
-/* Copyright (c) 2006, 2013, Oracle and/or its affiliates. All rights reserved.
-
-   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPLv2 as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-   
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
-   by the Free Software Foundation; version 2 of the License.
-   
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-   for more details.
-   
-   You should have received a copy of the GNU General Public License along
-   with this program; if not, write to the Free Software Foundation, Inc.,
-   51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+// Copyright (c) 2006, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of MySQL Connector/ODBC, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,7 +35,7 @@
     #include <dlfcn.h>
 #endif
 
-char *szSyntax =
+const char szSyntax[] =
 "\n" \
 "**********************************************\n" \
 "* dltest                                     *\n" \
@@ -152,9 +158,9 @@ static DLTestModule dltest_dlopen(const char *path)
                   GetLastError(),
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   (LPTSTR) &pszMsg,
-                  0, 
+                  0,
                   NULL);
-    printf("[%s][%d] ERROR LoadLibrary(): %s\n", __FILE__, __LINE__, pszMsg);
+    printf("[%s][%d] ERROR LoadLibrary(): %s\n", __FILE__, __LINE__, (char*)pszMsg);
     LocalFree(pszMsg);
     exit(1);
   }
@@ -195,9 +201,9 @@ static void dltest_dlsym(DLTestModule hModule, const char *sym)
                   GetLastError(),
                   MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                   (LPTSTR) &pszMsg,
-                  0, 
+                  0,
                   NULL);
-    printf("[%s][%d] ERROR: Could not find %s. %s\n",__FILE__,__LINE__,sym,pszMsg);
+    printf("[%s][%d] ERROR: Could not find %s. %s\n",__FILE__,__LINE__,sym, (char*)pszMsg);
     LocalFree(pszMsg);
     FreeLibrary(hModule);
     exit(1);
