@@ -1562,9 +1562,8 @@ primary_keys_no_i_s(SQLHSTMT hstmt,
       return handle_connection_error(stmt);
     }
 
-    stmt->lengths= (unsigned long*) myodbc_malloc( sizeof(long)*SQLPRIM_KEYS_FIELDS*
-                                            (ulong) stmt->result->row_count,
-                                            MYF(MY_ZEROFILL));
+    stmt->alloc_lengths(SQLPRIM_KEYS_FIELDS*(ulong) stmt->result->row_count);
+
     if (!stmt->lengths)
     {
       set_mem_error(&stmt->dbc->mysql);
@@ -1778,14 +1777,14 @@ procedure_columns_no_i_s(SQLHSTMT hstmt,
   int params_num= 0, return_params_num= 0;
   unsigned int i, j, total_params_num= 0;
 
-  if (init_dynamic_string(&dynQuery, "SELECT 1", 1024,1024))
+  if (myodbc_init_dynamic_string(&dynQuery, "SELECT 1", 1024,1024))
     return set_stmt_error(stmt, "HY001", "Not enough memory", 4001);
 
   params_r= params= (LIST *) myodbc_malloc(sizeof(LIST), MYF(MY_ZEROFILL));
 
   if (params_r == NULL)
   {
-    dynstr_free(&dynQuery);
+    myodbc_dynstr_free(&dynQuery);
     set_mem_error(&stmt->dbc->mysql);
     return handle_connection_error(stmt);
   }
@@ -1798,7 +1797,7 @@ procedure_columns_no_i_s(SQLHSTMT hstmt,
   {
     myodbc_mutex_unlock(&stmt->dbc->lock);
 
-    nReturn= set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+    nReturn= stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
                       mysql_errno(&stmt->dbc->mysql));
     goto clean_exit;
   }
@@ -1880,10 +1879,10 @@ procedure_columns_no_i_s(SQLHSTMT hstmt,
 
       if (cbColumnName)
       {
-        dynstr_append_mem(&dynQuery, ",", 1);
-        dynstr_append_os_quoted(&dynQuery, (char *)param_name, NullS);
-        dynstr_append_mem(&dynQuery, " LIKE ", 6);
-        dynstr_append_os_quoted(&dynQuery, (char *)szColumnName, NullS);
+        myodbc_append_mem(&dynQuery, ",", 1);
+        myodbc_append_os_quoted(&dynQuery, (char *)param_name, NullS);
+        myodbc_append_mem(&dynQuery, " LIKE ", 6);
+        myodbc_append_os_quoted(&dynQuery, (char *)szColumnName, NullS);
       }
 
       if (param_ordinal_position == 0)
@@ -1998,7 +1997,7 @@ procedure_columns_no_i_s(SQLHSTMT hstmt,
     {
       myodbc_mutex_unlock(&stmt->dbc->lock);
 
-      nReturn= set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+      nReturn= stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
                 mysql_errno(&stmt->dbc->mysql));
       goto exit_with_free;
     }
@@ -2010,7 +2009,7 @@ procedure_columns_no_i_s(SQLHSTMT hstmt,
 
     if (row == NULL)
     {
-      nReturn= set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+      nReturn= stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
                 mysql_errno(&stmt->dbc->mysql));
       goto exit_with_free;
     }
@@ -2100,7 +2099,7 @@ clean_exit:
     mysql_free_result(columns_res);
   }
 
-  dynstr_free(&dynQuery);
+  myodbc_dynstr_free(&dynQuery);
   x_free(params_r);
 
   return nReturn;
@@ -2235,7 +2234,7 @@ special_columns_no_i_s(SQLHSTMT hstmt, SQLUSMALLINT fColType,
 
     if ( fColType != SQL_BEST_ROWID )
     {
-        return set_error(stmt, MYERR_S1000,
+        return stmt->set_error( MYERR_S1000,
                          "Unsupported argument to SQLSpecialColumns", 4000);
     }
 
