@@ -1,30 +1,30 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Connector/ODBC, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of MySQL Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file  cursor.c
@@ -85,7 +85,7 @@ static const char *find_used_table(STMT *stmt)
                 table_name= field->org_table;
             if ( strcmp(field->org_table, table_name) )
             {
-                set_error(stmt,MYERR_S1000,
+                stmt->set_error(MYERR_S1000,
                           "Can't modify a row from a statement that uses more than one table",0);
                 return NULL;
             }
@@ -97,7 +97,7 @@ static const char *find_used_table(STMT *stmt)
                 table_name= field->table;
             if ( strcmp(field->table, table_name) )
             {
-                set_error(stmt,MYERR_S1000,
+                stmt->set_error(MYERR_S1000,
                           "Can't modify a row from a statement that uses more than one table",0);
                 return NULL;
             }
@@ -128,7 +128,7 @@ char *check_if_positioned_cursor_exists(STMT *pStmt, STMT **pStmtCursor)
 
   if (cursorName != NULL)
   {
-    
+
     LIST *list_element;
     DBC  *dbc= (DBC *)pStmt->dbc;
     char * wherePos= get_token(&pStmt->query, TOKEN_COUNT(&pStmt->query)- 4);
@@ -246,7 +246,7 @@ static my_bool check_if_usable_unique_key_exists(STMT *stmt)
   if (exec_stmt_query(stmt, buff, strlen(buff), FALSE) ||
       !(res= mysql_store_result(&stmt->dbc->mysql)))
   {
-    set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+    stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
               mysql_errno(&stmt->dbc->mysql));
     myodbc_mutex_unlock(&stmt->dbc->lock);
     return FALSE;
@@ -300,7 +300,7 @@ void set_current_cursor_data(STMT *stmt, SQLUINTEGER irow)
   long       nrow, row_pos;
   MYSQL_RES  *result= stmt->result;
 
-  
+
   /*
     If irow exists, then position the current row to point
     to the rowsetsize+irow, this is needed for positioned
@@ -357,10 +357,10 @@ static void set_dynamic_cursor_name(STMT *stmt)
 static SQLRETURN update_status(STMT *stmt, SQLUSMALLINT status)
 {
     if ( stmt->affected_rows == 0 )
-        return set_error(stmt,MYERR_01S03,NULL,0);
+        return stmt->set_error(MYERR_01S03,NULL,0);
 
     else if ( stmt->affected_rows > 1 )
-        return set_error(stmt,MYERR_01S04,NULL,0);
+        return stmt->set_error(MYERR_01S04,NULL,0);
 
     /*
       This only comes from SQLExecute(), not SQLSetPos() or
@@ -391,7 +391,7 @@ static SQLRETURN update_setpos_status(STMT *stmt, SQLINTEGER irow,
 
   if (irow && rows > 1)
   {
-    return set_error(stmt,MYERR_01S04,NULL,0);
+    return stmt->set_error(MYERR_01S04,NULL,0);
   }
 
   /*
@@ -434,7 +434,7 @@ static SQLRETURN copy_rowdata(STMT *stmt, DESCREC *aprec,
                          *aprec->octet_length_ptr + 1 : 7);
 
     if (stmt->extend_buffer(length) == NULL)
-        return set_error(stmt,MYERR_S1001,NULL,4001);
+        return stmt->set_error(MYERR_S1001,NULL,4001);
 
     rc= insert_param(stmt, NULL, stmt->apd, aprec, iprec, 0);
     if (!SQL_SUCCEEDED(rc))
@@ -444,7 +444,7 @@ static SQLRETURN copy_rowdata(STMT *stmt, DESCREC *aprec,
 
     /* insert "," */
     if (stmt->add_to_buffer(",", 1) == NULL)
-        return set_error(stmt,MYERR_S1001,NULL,4001);
+        return stmt->set_error(MYERR_S1001,NULL,4001);
 
     return(SQL_SUCCESS);
 }
@@ -496,16 +496,16 @@ static my_bool insert_field(STMT *stmt, MYSQL_RES *result,
       return 1;
     if (stmt->add_to_buffer(" AND ", 5) == NULL)
     {
-      return (my_bool)set_error(stmt, MYERR_S1001, NULL, 4001);
+      return (my_bool)stmt->set_error( MYERR_S1001, NULL, 4001);
     }
 
-    dynstr_append_mem(dynQuery, stmt->buf(), stmt->buf_pos());
+    myodbc_append_mem(dynQuery, stmt->buf(), stmt->buf_pos());
     stmt->buf_set_pos(0); // Buffer is used, can be reset
   }
   else
   {
     --dynQuery->length;
-    dynstr_append_mem(dynQuery, " IS NULL AND ",13);
+    myodbc_append_mem(dynQuery, " IS NULL AND ",13);
   }
   return 0;
 }
@@ -535,8 +535,8 @@ static SQLRETURN insert_pk_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
         if (!myodbc_strcasecmp(cursor->pkcol[index].name, field->org_name))
         {
           /* PK data exists...*/
-          dynstr_append_quoted_name(dynQuery, field->org_name);
-          dynstr_append_mem(dynQuery, "=", 1);
+          myodbc_append_quoted_name(dynQuery, field->org_name);
+          myodbc_append_mem(dynQuery, "=", 1);
           if (insert_field(stmt, result, dynQuery, ncol))
             return SQL_ERROR;
           cursor->pkcol[index].bind_done= TRUE;
@@ -590,7 +590,7 @@ static SQLRETURN append_all_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
   if (exec_stmt_query(stmt, select, strlen(select), FALSE) ||
       !(presultAllColumns= mysql_store_result(&stmt->dbc->mysql)))
   {
-    set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+    stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
               mysql_errno(&stmt->dbc->mysql));
     myodbc_mutex_unlock(&stmt->dbc->lock);
     return SQL_ERROR;
@@ -624,7 +624,7 @@ static SQLRETURN append_all_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
         table_field->type == MYSQL_TYPE_DOUBLE ||
         table_field->type == MYSQL_TYPE_DECIMAL)
     {
-      set_error(stmt,MYERR_S1000,
+      stmt->set_error(MYERR_S1000,
                 "Invalid use of floating point comparision in positioned operations",0);
       mysql_free_result(presultAllColumns);
       return SQL_ERROR;
@@ -637,8 +637,8 @@ static SQLRETURN append_all_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
       if (cursor_field->org_name &&
           !strcmp(cursor_field->org_name, table_field->name))
       {
-        dynstr_append_quoted_name(dynQuery, table_field->name);
-        dynstr_append_mem(dynQuery, "=", 1);
+        myodbc_append_quoted_name(dynQuery, table_field->name);
+        myodbc_append_mem(dynQuery, "=", 1);
         if (insert_field(stmt, result, dynQuery, j))
         {
           mysql_free_result(presultAllColumns);
@@ -668,7 +668,7 @@ static SQLRETURN append_all_fields(STMT *stmt, DYNAMIC_STRING *dynQuery)
   @purpose : build the where clause
 */
 
-static SQLRETURN build_where_clause( STMT * pStmt, 
+static SQLRETURN build_where_clause( STMT * pStmt,
                                      DYNAMIC_STRING * dynQuery,
                                      SQLUSMALLINT     irow )
 {
@@ -676,7 +676,7 @@ static SQLRETURN build_where_clause( STMT * pStmt,
     set_current_cursor_data( pStmt, irow );
 
     /* simply append WHERE to our statement */
-    dynstr_append_mem( dynQuery, " WHERE ", 7 );
+    myodbc_append_mem( dynQuery, " WHERE ", 7 );
 
     /*
       If a suitable key exists, then we'll use those columns, otherwise
@@ -704,11 +704,11 @@ static SQLRETURN build_where_clause( STMT * pStmt,
 
         sprintf( buff, " LIMIT %lu",
                  (unsigned long)pStmt->ard->array_size );
-        dynstr_append( dynQuery, buff );
+        myodbc_append_mem( dynQuery, buff , (uint)strlen(buff));
     }
     else
     {
-        dynstr_append_mem( dynQuery, " LIMIT 1", 8 );
+        myodbc_append_mem( dynQuery, " LIMIT 1", 8 );
     }
 
     return SQL_SUCCESS;
@@ -731,7 +731,7 @@ static SQLRETURN build_set_clause(STMT *stmt, SQLULEN irow,
     MYSQL_RES   *result= stmt->result;
     DESCREC *arrec, *irrec;
 
-    dynstr_append_mem(dynQuery," SET ",5);
+    myodbc_append_mem(dynQuery," SET ",5);
 
     desc_rec_init_apd(aprec);
     desc_rec_init_ipd(iprec);
@@ -749,7 +749,7 @@ static SQLRETURN build_set_clause(STMT *stmt, SQLULEN irow,
         field= mysql_fetch_field_direct(result,ncol);
         arrec= desc_get_rec(stmt->ard, ncol, FALSE);
         irrec= desc_get_rec(stmt->ird, ncol, FALSE);
-        
+
         if (!irrec)
         {
           return SQL_ERROR; // The error info is already set inside desc_get_rec()
@@ -795,8 +795,8 @@ static SQLRETURN build_set_clause(STMT *stmt, SQLULEN irow,
             }
         }
 
-        dynstr_append_quoted_name(dynQuery,field->org_name);
-        dynstr_append_mem(dynQuery,"=",1);
+        myodbc_append_quoted_name(dynQuery,field->org_name);
+        myodbc_append_mem(dynQuery,"=",1);
 
         iprec->concise_type= get_sql_data_type(stmt, field, NULL);
         aprec->concise_type= arrec->concise_type;
@@ -822,7 +822,7 @@ static SQLRETURN build_set_clause(STMT *stmt, SQLULEN irow,
         if ( copy_rowdata(stmt,aprec,iprec) != SQL_SUCCESS )
             return(SQL_ERROR);
 
-        dynstr_append_mem(dynQuery, stmt->buf(), stmt->buf_pos());
+        myodbc_append_mem(dynQuery, stmt->buf(), stmt->buf_pos());
         stmt->buf_set_pos(0); // Buffer is used, can be reset
     }
 
@@ -865,9 +865,9 @@ SQLRETURN my_pos_delete(STMT *stmt, STMT *stmtParam,
   @purpose : updates the positioned cursor row
 */
 
-SQLRETURN my_pos_update( STMT *             pStmtCursor, 
+SQLRETURN my_pos_update( STMT *             pStmtCursor,
                          STMT *             pStmt,
-                         SQLUSMALLINT       nRow, 
+                         SQLUSMALLINT       nRow,
                          DYNAMIC_STRING *   dynQuery )
 {
     SQLRETURN   rc;
@@ -890,7 +890,7 @@ SQLRETURN my_pos_update( STMT *             pStmtCursor,
     pStmtTemp = (STMT *)hStmtTemp;
 
     if (my_SQLPrepare(pStmtTemp, (SQLCHAR *)dynQuery->str, dynQuery->length,
-                      FALSE) != SQL_SUCCESS)
+                      false, true) != SQL_SUCCESS)
     {
         my_SQLFreeStmt( pStmtTemp, SQL_DROP );
         return set_stmt_error( pStmt, "HY000", "my_SQLPrepare() failed.", 0 );
@@ -919,7 +919,7 @@ SQLRETURN my_pos_update( STMT *             pStmtCursor,
         To check: do we really need that?
       */
       if (my_SQLPrepare(pStmt, (SQLCHAR *)dynQuery->str, dynQuery->length,
-                        FALSE) != SQL_SUCCESS)
+                        false, true) != SQL_SUCCESS)
         return SQL_ERROR;
       pStmt->dae_type= DAE_NORMAL;
     }
@@ -965,15 +965,15 @@ static SQLRETURN fetch_bookmark(STMT *stmt)
     data_seek(stmt, (my_ulonglong)rowset_pos);
     if (arrec->data_ptr)
     {
-      TargetValuePtr= ptr_offset_adjust(arrec->data_ptr, 
-                                        stmt->ard->bind_offset_ptr, 
-                                        stmt->ard->bind_type, 
+      TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
+                                        stmt->ard->bind_offset_ptr,
+                                        stmt->ard->bind_type,
                                         arrec->octet_length, rowset_pos - 1);
     }
 
     curr_bookmark_index= atol((const char*) TargetValuePtr);
 
-    nReturn= myodbc_single_fetch(stmt, SQL_FETCH_ABSOLUTE, 
+    nReturn= myodbc_single_fetch(stmt, SQL_FETCH_ABSOLUTE,
                                 curr_bookmark_index,
                                 stmt->ird->rows_processed_ptr,
                                 stmt->stmt_options.rowStatusPtr_ex ?
@@ -1005,9 +1005,9 @@ static SQLRETURN setpos_delete_bookmark(STMT *stmt, DYNAMIC_STRING *dynQuery)
   SQLPOINTER TargetValuePtr= NULL;
   long curr_bookmark_index= 0;
 
-  /* 
-     we want to work with base table name - 
-     we expect call to fail if more than one base table involved 
+  /*
+     we want to work with base table name -
+     we expect call to fail if more than one base table involved
   */
   if (!(table_name= find_used_table(stmt)))
   {
@@ -1015,7 +1015,7 @@ static SQLRETURN setpos_delete_bookmark(STMT *stmt, DYNAMIC_STRING *dynQuery)
   }
 
   /* appened our table name to our DELETE statement */
-  dynstr_append_quoted_name(dynQuery,table_name);
+  myodbc_append_quoted_name(dynQuery,table_name);
   query_length= dynQuery->length;
 
   IS_BOOKMARK_VARIABLE(stmt);
@@ -1037,9 +1037,9 @@ static SQLRETURN setpos_delete_bookmark(STMT *stmt, DYNAMIC_STRING *dynQuery)
   {
     if (arrec->data_ptr)
     {
-      TargetValuePtr= ptr_offset_adjust(arrec->data_ptr, 
-                                        stmt->ard->bind_offset_ptr, 
-                                        stmt->ard->bind_type, 
+      TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
+                                        stmt->ard->bind_offset_ptr,
+                                        stmt->ard->bind_type,
                                         arrec->octet_length, rowset_pos);
     }
 
@@ -1101,7 +1101,7 @@ static SQLRETURN setpos_delete(STMT *stmt, SQLUSMALLINT irow,
   }
 
   /* appened our table name to our DELETE statement */
-  dynstr_append_quoted_name(dynQuery,table_name);
+  myodbc_append_quoted_name(dynQuery,table_name);
   query_length= dynQuery->length;
 
   /* IF irow == 0 THEN delete all rows in the current rowset ELSE specific (as in one) row */
@@ -1170,7 +1170,7 @@ static SQLRETURN setpos_update_bookmark(STMT *stmt, DYNAMIC_STRING *dynQuery)
     return SQL_ERROR;
   }
 
-  dynstr_append_quoted_name(dynQuery,table_name);
+  myodbc_append_quoted_name(dynQuery,table_name);
   query_length= dynQuery->length;
 
   IS_BOOKMARK_VARIABLE(stmt);
@@ -1192,9 +1192,9 @@ static SQLRETURN setpos_update_bookmark(STMT *stmt, DYNAMIC_STRING *dynQuery)
   {
     if (arrec->data_ptr)
     {
-      TargetValuePtr= ptr_offset_adjust(arrec->data_ptr, 
-                                        stmt->ard->bind_offset_ptr, 
-                                        stmt->ard->bind_type, 
+      TargetValuePtr= ptr_offset_adjust(arrec->data_ptr,
+                                        stmt->ard->bind_offset_ptr,
+                                        stmt->ard->bind_type,
                                         arrec->octet_length, rowset_pos);
     }
 
@@ -1230,7 +1230,7 @@ static SQLRETURN setpos_update_bookmark(STMT *stmt, DYNAMIC_STRING *dynQuery)
       stmt->ird->array_status_ptr[curr_bookmark_index]= SQL_ROW_UPDATED;
     }
 
-    ++rowset_pos; 
+    ++rowset_pos;
   }
 
   global_set_affected_rows(stmt, affected);
@@ -1255,7 +1255,7 @@ static SQLRETURN setpos_update(STMT *stmt, SQLUSMALLINT irow,
   if ( !(table_name= find_used_table(stmt)) )
       return SQL_ERROR;
 
-  dynstr_append_quoted_name(dynQuery,table_name);
+  myodbc_append_quoted_name(dynQuery,table_name);
   query_length= dynQuery->length;
 
   if ( !irow )
@@ -1324,13 +1324,13 @@ static SQLRETURN setpos_update(STMT *stmt, SQLUSMALLINT irow,
             to handle this. see break_insert.
 
     \param  stmt            Viable statement.
-    \param  irow            Position of the row in the rowset on which to perform the operation. 
+    \param  irow            Position of the row in the rowset on which to perform the operation.
                             If RowNumber is 0, the operation applies to every row in the rowset.
-    \param  ext_query       The INSERT statement up to and including the VALUES. So something 
+    \param  ext_query       The INSERT statement up to and including the VALUES. So something
                             like; "INSERT .... VALUES"
-                            
+
     \return SQLRETURN
-    
+
     \retval SQLERROR        Something went wrong.
     \retval SQL_SUCCESS     Success!
 */
@@ -1375,7 +1375,7 @@ static SQLRETURN batch_insert( STMT *stmt, SQLULEN irow, DYNAMIC_STRING *ext_que
         while (count < insert_count)
         {
             /* Append values for each column. */
-            dynstr_append_mem(ext_query,"(", 1);
+            myodbc_append_mem(ext_query,"(", 1);
             for ( ncol= 0; ncol < result->field_count; ++ncol )
             {
                 MYSQL_FIELD *field= mysql_fetch_field_direct(result,ncol);
@@ -1388,7 +1388,7 @@ static SQLRETURN batch_insert( STMT *stmt, SQLULEN irow, DYNAMIC_STRING *ext_que
                   aprec= desc_get_rec(stmt->setpos_apd, ncol, FALSE);
                 else
                   desc_rec_init_apd(aprec);
-                
+
                 if (arrec)
                 {
                   if (aprec->par.is_dae)
@@ -1456,9 +1456,9 @@ static SQLRETURN batch_insert( STMT *stmt, SQLULEN irow, DYNAMIC_STRING *ext_que
             } /* END OF for (ncol= 0; ncol < result->field_count; ++ncol) */
 
             length = stmt->buf_pos();
-            dynstr_append_mem(ext_query, stmt->buf(), length - 1);
+            myodbc_append_mem(ext_query, stmt->buf(), length - 1);
             stmt->buf_set_pos(0); // Buffer is used, can be reset
-            dynstr_append_mem(ext_query, "),", 2);
+            myodbc_append_mem(ext_query, "),", 2);
             ++count;
 
             /*
@@ -1502,7 +1502,7 @@ static SQLRETURN batch_insert( STMT *stmt, SQLULEN irow, DYNAMIC_STRING *ext_que
           pcbValue= NULL;
           TargetValuePtr= NULL;
 
-          reset_getdata_position(stmt);
+          stmt->reset_getdata_position();
 
           if (arrec->data_ptr)
           {
@@ -1523,7 +1523,7 @@ static SQLRETURN batch_insert( STMT *stmt, SQLULEN irow, DYNAMIC_STRING *ext_que
 
           _len= sprintf(_value, "%ld", i + 1);
           res= sql_get_bookmark_data(stmt, arrec->concise_type, (uint)0,
-                                TargetValuePtr, arrec->octet_length, 
+                                TargetValuePtr, arrec->octet_length,
                                 pcbValue, _value, _len, arrec);
 
           if (!SQL_SUCCEEDED(res))
@@ -1564,7 +1564,7 @@ static SQLRETURN setpos_dae_check_and_init(STMT *stmt, SQLSETPOSIROW irow,
 {
   int dae_rec;
   SQLRETURN rc;
-  
+
   /*
     If this statement hasn't already had the dae params set,
     check if there are any and begin the SQLParamData() sequence.
@@ -1576,7 +1576,7 @@ static SQLRETURN setpos_dae_check_and_init(STMT *stmt, SQLSETPOSIROW irow,
       return set_stmt_error(stmt, "HYC00", "Multiple row insert "
                             "with data at execution not supported",
                             0);
-    
+
     /* create APD, and copy ARD to it */
     stmt->setpos_apd= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
                                  DESC_APP, DESC_PARAM);
@@ -1585,7 +1585,7 @@ static SQLRETURN setpos_dae_check_and_init(STMT *stmt, SQLSETPOSIROW irow,
                             4001);
     if(rc= stmt_SQLCopyDesc(stmt, stmt->ard, stmt->setpos_apd))
       return rc;
-                  
+
     stmt->current_param= dae_rec;
     stmt->dae_type= dae_type;
     stmt->setpos_row= irow;
@@ -1599,7 +1599,7 @@ static SQLRETURN setpos_dae_check_and_init(STMT *stmt, SQLSETPOSIROW irow,
 
 /*!
     \brief  Shadow function for SQLSetPos.
-    
+
             Sets the cursor position in a rowset and allows an application
             to refresh data in the rowset or to update or delete data in
             the result set.
@@ -1628,7 +1628,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
     CLEAR_STMT_ERROR(stmt);
 
     if ( !result )
-        return set_error(stmt,MYERR_S1010,NULL,0);
+        return stmt->set_error(MYERR_S1010,NULL,0);
 
     /* With mysql_use_reslt we cannot do anything but move cursor
        forward. additional connection?
@@ -1643,39 +1643,39 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
       if (fOption != SQL_POSITION)
       {
         /* HY109. Perhaps 24000 Invalid cursor state is a better fit*/
-        return set_error(stmt, MYERR_S1109,NULL, 0);
+        return stmt->set_error( MYERR_S1109,NULL, 0);
       }
       /* We can't go back with forwrd only cursor */
       else if (irow < stmt->current_row)
       {
         /* Same HY109 Invalid cursor position*/
-        return set_error(stmt, MYERR_S1109,NULL, 0); 
+        return stmt->set_error( MYERR_S1109,NULL, 0);
       }
     }
 
     /* If irow > maximum rows in the resultset. for forwrd only row_count is 0
      */
     if ( fOption != SQL_ADD && irow > num_rows(stmt))
-        return set_error(stmt,MYERR_S1107,NULL,0);
+        return stmt->set_error(MYERR_S1107,NULL,0);
 
     /* Not a valid lock type ..*/
     if ( fLock != SQL_LOCK_NO_CHANGE )
-        return set_error(stmt,MYERR_S1C00,NULL,0);
+        return stmt->set_error(MYERR_S1C00,NULL,0);
 
     switch ( fOption )
     {
         case SQL_POSITION:
             {
                 if ( irow == 0 )
-                    return set_error(stmt,MYERR_S1109,NULL,0);
+                    return stmt->set_error(MYERR_S1109,NULL,0);
 
                 if ( irow > stmt->rows_found_in_set )
-                    return set_error(stmt,MYERR_S1107,NULL,0);
+                    return stmt->set_error(MYERR_S1107,NULL,0);
 
                 /* If Dynamic cursor, fetch the latest resultset */
                 if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
                 {
-                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
+                    return stmt->set_error(MYERR_S1000, alloc_error, 0);
                 }
 
                 myodbc_mutex_lock(&stmt->dbc->lock);
@@ -1684,7 +1684,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                 stmt->cursor_row= (long)(stmt->current_row+irow);
                 data_seek(stmt, (my_ulonglong)stmt->cursor_row);
                 stmt->current_values= fetch_row(stmt);
-                reset_getdata_position(stmt);
+                stmt->reset_getdata_position();
                 if ( stmt->fix_fields )
                     stmt->current_values= (*stmt->fix_fields)(stmt,stmt->current_values);
                 /*
@@ -1702,18 +1702,18 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                 DYNAMIC_STRING dynQuery;
 
                 if ( irow > stmt->rows_found_in_set )
-                    return set_error(stmt,MYERR_S1107,NULL,0);
+                    return stmt->set_error(MYERR_S1107,NULL,0);
 
                 /* IF dynamic cursor THEN rerun query to refresh resultset */
                 if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-                    return set_error(stmt,MYERR_S1000, alloc_error, 0);
+                    return stmt->set_error(MYERR_S1000, alloc_error, 0);
 
                 /* start building our DELETE statement */
-                if ( init_dynamic_string(&dynQuery, "DELETE FROM ", 1024, 1024) )
-                    return set_error(stmt,MYERR_S1001,NULL,4001);
+                if ( myodbc_init_dynamic_string(&dynQuery, "DELETE FROM ", 1024, 1024) )
+                    return stmt->set_error(MYERR_S1001,NULL,4001);
 
                 sqlRet = setpos_delete( stmt, irow, &dynQuery );
-                dynstr_free(&dynQuery);
+                myodbc_dynstr_free(&dynQuery);
                 break;
             }
 
@@ -1722,22 +1722,22 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                 DYNAMIC_STRING dynQuery;
 
                 if ( irow > stmt->rows_found_in_set )
-                    return set_error(stmt,MYERR_S1107,NULL,0);
+                    return stmt->set_error(MYERR_S1107,NULL,0);
 
                 /* IF dynamic cursor THEN rerun query to refresh resultset */
                 if (!stmt->dae_type && if_dynamic_cursor(stmt) &&
                     set_dynamic_result(stmt))
-                  return set_error(stmt,MYERR_S1000, alloc_error, 0);
+                  return stmt->set_error(MYERR_S1000, alloc_error, 0);
 
                 if (rc= setpos_dae_check_and_init(stmt, irow, fLock,
                                                   DAE_SETPOS_UPDATE))
                   return rc;
 
-                if ( init_dynamic_string(&dynQuery, "UPDATE ", 1024, 1024) )
-                    return set_error(stmt,MYERR_S1001,NULL,4001);
+                if ( myodbc_init_dynamic_string(&dynQuery, "UPDATE ", 1024, 1024) )
+                    return stmt->set_error(MYERR_S1001,NULL,4001);
 
                 sqlRet= setpos_update(stmt,irow,&dynQuery);
-                dynstr_free(&dynQuery);
+                myodbc_dynstr_free(&dynQuery);
                 break;
             }
 
@@ -1749,7 +1749,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
 
                 if (!stmt->dae_type && if_dynamic_cursor(stmt) &&
                     set_dynamic_result(stmt))
-                  return set_error(stmt,MYERR_S1000, alloc_error, 0);
+                  return stmt->set_error(MYERR_S1000, alloc_error, 0);
                 result= stmt->result;
 
                 if ( !(table_name= find_used_table(stmt)) )
@@ -1759,34 +1759,34 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                                                   DAE_SETPOS_INSERT))
                   return rc;
 
-                if ( init_dynamic_string(&dynQuery, "INSERT INTO ", 1024,1024) )
+                if ( myodbc_init_dynamic_string(&dynQuery, "INSERT INTO ", 1024,1024) )
                     return set_stmt_error(stmt, "S1001", "Not enough memory",
                                           4001);
 
                 /* Append the table's DB name if exists */
                 if (result->fields && result->fields[0].db_length)
                 {
-                  dynstr_append_quoted_name(&dynQuery, result->fields[0].db);
-                  dynstr_append_mem(&dynQuery,".",1);
+                  myodbc_append_quoted_name(&dynQuery, result->fields[0].db);
+                  myodbc_append_mem(&dynQuery,".",1);
                 }
 
-                dynstr_append_quoted_name(&dynQuery,table_name);
-                dynstr_append_mem(&dynQuery,"(",1);
+                myodbc_append_quoted_name(&dynQuery,table_name);
+                myodbc_append_mem(&dynQuery,"(",1);
 
                 /* build list of column names */
                 for (nCol= 0; nCol < result->field_count; ++nCol)
                 {
                     MYSQL_FIELD *field= mysql_fetch_field_direct(result, nCol);
-                    dynstr_append_quoted_name(&dynQuery, field->org_name);
-                    dynstr_append_mem(&dynQuery, ",", 1);
+                    myodbc_append_quoted_name(&dynQuery, field->org_name);
+                    myodbc_append_mem(&dynQuery, ",", 1);
                 }
                 --dynQuery.length;        /* Remove last ',' */
-                dynstr_append_mem(&dynQuery,") VALUES ",9);
+                myodbc_append_mem(&dynQuery,") VALUES ",9);
 
                 /* process row(s) using our INSERT as base */
                 sqlRet= batch_insert(stmt, irow, &dynQuery);
 
-                dynstr_free(&dynQuery);
+                myodbc_dynstr_free(&dynQuery);
                 break;
             }
 
@@ -1806,7 +1806,7 @@ SQLRETURN SQL_API my_SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
             }
 
         default:
-            return set_error(stmt,MYERR_S1009,NULL,0);
+            return stmt->set_error(MYERR_S1009,NULL,0);
     }
     return sqlRet;
 }
@@ -1820,20 +1820,20 @@ MySQLSetCursorName(SQLHSTMT hstmt, SQLCHAR *name, SQLSMALLINT len)
   CLEAR_STMT_ERROR(stmt);
 
   if (!name)
-    return set_error(stmt, MYERR_S1009, NULL, 0);
+    return stmt->set_error( MYERR_S1009, NULL, 0);
 
   if (len == SQL_NTS)
     len= strlen((char *)name);
 
   if (len < 0)
-    return set_error(stmt, MYERR_S1009, NULL, 0);
+    return stmt->set_error( MYERR_S1009, NULL, 0);
 
   /** @todo use charset-aware casecmp */
   if (len == 0 ||
       len > MYSQL_MAX_CURSOR_LEN ||
       myodbc_casecmp((char *)name, "SQLCUR", 6) == 0 ||
       myodbc_casecmp((char *)name, "SQL_CUR", 7) == 0)
-    return set_error(stmt, MYERR_34000, NULL, 0);
+    return stmt->set_error( MYERR_34000, NULL, 0);
 
   x_free(stmt->cursor.name);
   stmt->cursor.name= dupp_str((char*)name, len);
@@ -1870,9 +1870,17 @@ SQLCHAR *MySQLGetCursorName(HSTMT hstmt)
 SQLRETURN SQL_API SQLSetPos(SQLHSTMT hstmt, SQLSETPOSIROW irow,
                             SQLUSMALLINT fOption, SQLUSMALLINT fLock)
 {
+    SQLRETURN rc = SQL_SUCCESS;
     CHECK_HANDLE(hstmt);
+    STMT *stmt = (STMT*)hstmt;
 
-    return my_SQLSetPos(hstmt,irow,fOption,fLock);
+    // This operation number has to prevent using SSPS when SQLSetPos()
+    // has to generate its own query different from the current statement.
+    stmt->setpos_op = fOption;
+
+    rc = my_SQLSetPos(hstmt, irow, fOption, fLock);
+    stmt->setpos_op = 0;
+    return rc;
 }
 
 /*
@@ -1894,7 +1902,7 @@ SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
   CLEAR_STMT_ERROR(stmt);
 
   if ( !result )
-      return set_error(stmt,MYERR_S1010,NULL,0);
+      return stmt->set_error(MYERR_S1010,NULL,0);
 
   stmt->stmt_options.bookmark_insert= FALSE;
 
@@ -1917,7 +1925,7 @@ SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
       if (!stmt->dae_type && if_dynamic_cursor(stmt) &&
           set_dynamic_result(stmt))
       {
-        return set_error(stmt,MYERR_S1000, alloc_error, 0);
+        return stmt->set_error(MYERR_S1000, alloc_error, 0);
       }
 
       if (rc= setpos_dae_check_and_init(stmt, irow, SQL_LOCK_NO_CHANGE,
@@ -1926,13 +1934,13 @@ SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
         return rc;
       }
 
-      if ( init_dynamic_string(&dynQuery, "UPDATE ", 1024, 1024) )
+      if ( myodbc_init_dynamic_string(&dynQuery, "UPDATE ", 1024, 1024) )
       {
-        return set_error(stmt,MYERR_S1001,NULL,4001);
+        return stmt->set_error(MYERR_S1001,NULL,4001);
       }
 
       sqlRet= setpos_update_bookmark(stmt, &dynQuery);
-      dynstr_free(&dynQuery);
+      myodbc_dynstr_free(&dynQuery);
       break;
     }
   case SQL_DELETE_BY_BOOKMARK:
@@ -1941,14 +1949,14 @@ SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
 
       /* IF dynamic cursor THEN rerun query to refresh resultset */
       if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-          return set_error(stmt,MYERR_S1000, alloc_error, 0);
+          return stmt->set_error(MYERR_S1000, alloc_error, 0);
 
       /* start building our DELETE statement */
-      if ( init_dynamic_string(&dynQuery, "DELETE FROM ", 1024, 1024) )
-          return set_error(stmt,MYERR_S1001,NULL,4001);
+      if ( myodbc_init_dynamic_string(&dynQuery, "DELETE FROM ", 1024, 1024) )
+          return stmt->set_error(MYERR_S1001,NULL,4001);
 
       sqlRet = setpos_delete_bookmark(stmt, &dynQuery);
-      dynstr_free(&dynQuery);
+      myodbc_dynstr_free(&dynQuery);
       break;
     }
   case SQL_FETCH_BY_BOOKMARK:
@@ -1957,7 +1965,7 @@ SQLRETURN SQL_API SQLBulkOperations(SQLHSTMT  Handle, SQLSMALLINT Operation)
       break;
     }
   default:
-    return set_error((STMT*)Handle,MYERR_S1C00,NULL,0);
+    return ((STMT*)(Handle))->set_error(MYERR_S1C00,NULL,0);
   }
 
   return sqlRet;
