@@ -209,6 +209,63 @@ typedef enum { DESC_HDR, DESC_REC } fld_loc;
 /* data-at-exec handling done for current SQLSetPos() call */
 #define DAE_SETPOS_DONE 10
 
+#define DONT_USE_LOCALE_CHECK(STMT) if (!STMT->dbc->ds->dont_use_set_locale)
+
+#if defined _WIN32
+
+  #define DECLARE_LOCALE_HANDLE int loc;
+
+    #define __LOCALE_SET(LOC) \
+    { \
+      loc = _configthreadlocale(0); \
+      _configthreadlocale(_ENABLE_PER_THREAD_LOCALE); \
+      setlocale(LC_NUMERIC, LOC);  /* force use of '.' as decimal point */ \
+    }
+
+  #define __LOCALE_RESTORE() \
+    { \
+      setlocale(LC_NUMERIC, default_locale); \
+      _configthreadlocale(loc); \
+    }
+
+#elif defined LC_GLOBAL_LOCALE
+  #define DECLARE_LOCALE_HANDLE locale_t nloc;
+
+  #define __LOCALE_SET(LOC) \
+    { \
+      nloc = newlocale(LC_CTYPE_MASK, LOC, (locale_t)0); \
+      uselocale(nloc); \
+    }
+
+  #define __LOCALE_RESTORE() \
+    { \
+      uselocale(LC_GLOBAL_LOCALE); \
+      freelocale(nloc); \
+    }
+#else
+
+  #define DECLARE_LOCALE_HANDLE
+
+  #define __LOCALE_SET(LOC) \
+      { \
+        setlocale(LC_NUMERIC, LOC);  /* force use of '.' as decimal point */ \
+      }
+
+  #define __LOCALE_RESTORE() \
+      { \
+        setlocale(LC_NUMERIC, default_locale); \
+      }
+#endif
+
+#define C_LOCALE_SET(STMT) \
+    DONT_USE_LOCALE_CHECK(STMT) \
+    __LOCALE_SET("C")
+
+#define DEFAULT_LOCALE_SET(STMT) \
+    DONT_USE_LOCALE_CHECK(STMT) \
+    __LOCALE_RESTORE()
+
+
 typedef struct {
   int perms;
   SQLSMALLINT data_type; /* SQL_IS_SMALLINT, etc */
@@ -473,7 +530,9 @@ struct tempBuf
   char* extend_buffer(char *to, size_t len);
   char* extend_buffer(size_t len);
 
+  // Append data to the current buffer
   char* add_to_buffer(const char *from, size_t len);
+
   char* add_to_buffer(char *to, const char *from, size_t len);
   void remove_trail_zeroes();
 
@@ -519,6 +578,7 @@ struct STMT
   MY_PARSED_QUERY	query, orig_query;
   DYNAMIC_ARRAY     *param_bind;
 
+  my_bool           lengths_allocated;
   unsigned long     *lengths; /* used to set lengths if we shuffle field values
                          of the resultset of auxiliary query or if we fix_fields. */
   /*
@@ -549,6 +609,7 @@ struct STMT
   DESC *setpos_apd;
   SQLSETPOSIROW setpos_row;
   SQLUSMALLINT setpos_lock;
+  SQLUSMALLINT setpos_op;
 
   MYSQL_STMT *ssps;
   MYSQL_BIND *result_bind;
@@ -564,6 +625,7 @@ struct STMT
   char* extend_buffer(size_t len);
 
   char* add_to_buffer(const char *from, size_t len);
+  char* place_to_buffer(const char *from, size_t len);
   char* buf() { return tempbuf.buf; }
   char* endbuf() { return tempbuf.buf + tempbuf.cur_pos; }
   size_t buf_pos() { return tempbuf.cur_pos; }
@@ -571,11 +633,16 @@ struct STMT
   void buf_set_pos(size_t pos) { tempbuf.cur_pos = pos; }
   void buf_add_pos(size_t pos) { tempbuf.cur_pos += pos; }
   void buf_remove_trail_zeroes() { tempbuf.remove_trail_zeroes(); }
+  void alloc_lengths(size_t num);
+  void free_lengths();
+  void reset_getdata_position();
+  long compute_cur_row(unsigned fFetchType, SQLLEN irow);
+  SQLRETURN set_error(myodbc_errid errid, const char *errtext, SQLINTEGER errcode);
 
 
   STMT() : dbc(NULL), result(NULL), array(NULL), result_array(NULL),
     current_values(NULL), fields(NULL), end_of_set(NULL), table_name(NULL),
-    param_bind(NULL), lengths(NULL), affected_rows(0),
+    param_bind(NULL), lengths_allocated(FALSE), lengths(NULL), affected_rows(0),
     current_row(0), cursor_row(0), dae_type(0),
     order(NULL), order_count(0), param_count(0), current_param(0),
     rows_found_in_set(0),
@@ -586,6 +653,8 @@ struct STMT
     ssps(NULL), result_bind(NULL), param_place_bind(NULL),
     out_params_state(OPS_UNKNOWN)
   { }
+
+  ~STMT();
 };
 
 
@@ -685,8 +754,7 @@ SQLRETURN SQL_API MySQLGetStmtAttr(SQLHSTMT hstmt, SQLINTEGER Attribute,
                                      __attribute__((unused)),
                                   SQLINTEGER *StringLengthPtr);
 SQLRETURN SQL_API MySQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType);
-SQLRETURN SQL_API MySQLPrepare(SQLHSTMT hstmt, SQLCHAR *query, SQLINTEGER len,
-                               my_bool dupe);
+SQLRETURN SQL_API MySQLPrepare(SQLHSTMT hstmt, SQLCHAR *query, SQLINTEGER len, bool dupe, bool reset_select_limit);
 SQLRETURN SQL_API MySQLPrimaryKeys(SQLHSTMT hstmt,
                                    SQLCHAR *catalog, SQLSMALLINT catalog_len,
                                    SQLCHAR *schema, SQLSMALLINT schema_len,
