@@ -176,7 +176,7 @@ SQLRETURN set_dbc_error(DBC *dbc, char *state,
   conversion
 */
 
-SQLRETURN set_stmt_error( STMT       *  stmt,
+SQLRETURN myodbc_set_stmt_error( STMT       *  stmt,
                           const char *  state,
                           const char *  message,
                           uint          errcode )
@@ -322,19 +322,22 @@ SQLRETURN set_conn_error(DBC *dbc, myodbc_errid errid, const char *errtext,
 }
 
 
-/*
-  @type    : myodbc3 internal
-  @purpose : sets the error information to statement handle.
-*/
-
-SQLRETURN set_error(STMT *stmt, myodbc_errid errid, const char *errtext,
-                    SQLINTEGER errcode)
+MYERROR::MYERROR(myodbc_errid errid, const char *errtext, SQLINTEGER errcode,
+                 char *prefix)
 {
-    return copy_error(&stmt->error, errid, errtext, errcode,
-                      stmt->dbc->st_error_prefix);
+  SQLCHAR     *errmsg;
+
+  errmsg = (errtext ? (SQLCHAR *)errtext :
+    (SQLCHAR *)myodbc3_errors[errid].message);
+  native_error = errcode ? (myodbc_errid)errcode : errid + MYODBC_ERROR_CODE_START;
+
+  retcode = myodbc3_errors[errid].retcode;  /* RETCODE */
+  myodbc_stpmov(sqlstate, myodbc3_errors[errid].sqlstate);   /* SQLSTATE */
+  strxmov(message, prefix, errmsg, NullS);           /* MESSAGE */
 }
 
 
+
 /*
   @type    : myodbc3 internal
   @purpose : sets a myodbc_malloc() failure on a MYSQL* connection
