@@ -1,30 +1,30 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file  execute.c
@@ -160,7 +160,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
       /* Query was supposed to return result, but result is NULL*/
       if (returned_result(stmt))
       {
-        set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+        stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
                 mysql_errno(&stmt->dbc->mysql));
         goto exit;
       }
@@ -193,7 +193,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
     {
       if (bind_result(stmt) || get_result(stmt))
       {
-          set_error(stmt, MYERR_S1000, mysql_error(&stmt->dbc->mysql),
+          stmt->set_error( MYERR_S1000, mysql_error(&stmt->dbc->mysql),
                   mysql_errno(&stmt->dbc->mysql));
           goto exit;
       }
@@ -269,7 +269,7 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
     if (stmt->dummy_state != ST_DUMMY_PREPARED &&
         (!aprec || !aprec->par.real_param_done))
     {
-      rc= set_error(stmt, MYERR_07001,
+      rc= stmt->set_error( MYERR_07001,
           "The number of parameter markers is not equal "
           "to he number of parameters provided",0);
       goto error;
@@ -336,6 +336,7 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
     if (finalquery!=NULL)
     {
       char *dupquery = NULL;
+
       if (!(dupquery = (char*)myodbc_memdup(stmt->buf(),
         stmt->buf_pos(), MYF(0))))
       {
@@ -360,7 +361,7 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
   return rc;
 
 memerror:      /* Too much data */
-  rc= set_error(stmt,MYERR_S1001,NULL,4001);
+  rc= stmt->set_error(MYERR_S1001,NULL,4001);
 error:
   /* ! was _already_ locked, when we tried to lock */
   if (!mutex_was_locked)
@@ -447,7 +448,7 @@ BOOL bind_param(MYSQL_BIND *bind, const char *value, unsigned long length,
   {
     return FALSE;
   }
-  
+
   if (allocate_param_buffer(bind, length))
   {
     return TRUE;
@@ -1187,7 +1188,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
         }
     }
 
-    if (bind != NULL && ssps_used(stmt))
+    if (bind != NULL && ssps_used(stmt) && stmt->setpos_op == 0)
     {
       bind_param(bind, data, length, MYSQL_TYPE_STRING);
     }
@@ -1236,7 +1237,7 @@ memerror:
     {
       x_free(data);
     }
-    return set_error(stmt, MYERR_S1001, NULL, 4001);
+    return stmt->set_error( MYERR_S1001, NULL, 4001);
 }
 
 
@@ -1259,8 +1260,8 @@ SQLRETURN do_my_pos_cursor( STMT *pStmt, STMT *pStmtCursor )
   while ( isspace( *pszQuery ) )
       ++pszQuery;
 
-  if ( init_dynamic_string( &dynQuery, pszQuery, 1024, 1024 ) )
-      return set_error( pStmt, MYERR_S1001, NULL, 4001 );
+  if ( myodbc_init_dynamic_string( &dynQuery, pszQuery, 1024, 1024 ) )
+      return pStmt->set_error(MYERR_S1001, NULL, 4001 );
 
   if ( !myodbc_casecmp( pszQuery, "delete", 6 ) )
   {
@@ -1272,13 +1273,13 @@ SQLRETURN do_my_pos_cursor( STMT *pStmt, STMT *pStmtCursor )
   }
   else
   {
-      nReturn = set_error( pStmt, MYERR_S1000, "Specified SQL syntax is not supported", 0 );
+      nReturn = pStmt->set_error(MYERR_S1000, "Specified SQL syntax is not supported", 0 );
   }
 
   if ( SQL_SUCCEEDED( nReturn ) )
       pStmt->state = ST_EXECUTED;
 
-  dynstr_free( &dynQuery );
+  myodbc_dynstr_free( &dynQuery );
 
   return( nReturn );
 }
@@ -1351,12 +1352,12 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
   CLEAR_STMT_ERROR( pStmt );
 
   if (!GET_QUERY(&pStmt->query))
-      return set_error(pStmt, MYERR_S1010,
+      return pStmt->set_error( MYERR_S1010,
                        "No previous SQLPrepare done", 0);
 
   if (is_set_names_statement(GET_QUERY(&pStmt->query)))
   {
-    return set_error(pStmt, MYERR_42000,
+    return pStmt->set_error( MYERR_42000,
                      "SET NAMES not allowed by driver", 0);
   }
 
@@ -1365,14 +1366,14 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
     /* Save a copy of the query, because we're about to modify it. */
     if (copy_parsed_query(&pStmt->query, &pStmt->orig_query))
     {
-      return set_error(pStmt,MYERR_S1001,NULL,4001);
+      return pStmt->set_error(MYERR_S1001,NULL,4001);
     }
 
     /* Cursor statement use mysql_use_result - thus any operation
        will couse commands out of sync */
     if (if_forward_cache(pStmtCursor))
     {
-      return set_error(pStmt,MYERR_S1010,NULL,0);
+      return pStmt->set_error(MYERR_S1010,NULL,0);
     }
 
     /* Chop off the 'WHERE CURRENT OF ...' - doing it a hard way...*/
@@ -1397,7 +1398,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
 
   if ( pStmt->ipd->rows_processed_ptr )
   {
-    *pStmt->ipd->rows_processed_ptr= 0;
+    *pStmt->ipd->rows_processed_ptr= (SQLULEN)0;
   }
 
   /* Locking if we have params array for "SELECT" statemnt */
@@ -1698,14 +1699,15 @@ static SQLRETURN execute_dae(STMT *stmt)
 {
   SQLRETURN rc;
   char *query;
+  SQLULEN query_len = 0;
 
   switch (stmt->dae_type)
   {
   case DAE_NORMAL:
     query= GET_QUERY(&stmt->query);
-    if (!SQL_SUCCEEDED(rc= insert_params(stmt, 0, &query, 0)))
+    if (!SQL_SUCCEEDED(rc= insert_params(stmt, 0, &query, &query_len)))
       break;
-    rc= do_query(stmt, query, 0);
+    rc= do_query(stmt, query, query_len);
     break;
   case DAE_SETPOS_INSERT:
     stmt->dae_type= DAE_SETPOS_DONE;
@@ -1734,7 +1736,7 @@ static SQLRETURN find_next_out_stream(STMT *stmt, SQLPOINTER *token)
   if (rec != NULL)
   {
     uint cur_column_number= stmt->getdata.column;
-    reset_getdata_position(stmt);
+    stmt->reset_getdata_position();
     stmt->getdata.column= cur_column_number;
     stmt->getdata.src_offset= 0;
 
