@@ -490,12 +490,9 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
   stmt->stmt_options= dbc->stmt_options;
   stmt->state= ST_UNKNOWN;
   stmt->dummy_state= ST_DUMMY_UNKNOWN;
+  stmt->ssps= NULL;
+  stmt->setpos_op = 0;
   myodbc_stpmov(stmt->error.sqlstate, "00000");
-
-#if MYSQL_VERSION_ID >= 50723 && MYSQL_VERSION_ID < 80000
-  init_alloc_root(PSI_NOT_INSTRUMENTED, &stmt->alloc_root, 32, 32);
-#endif
-
   init_parsed_query(&stmt->query);
   init_parsed_query(&stmt->orig_query);
 
@@ -645,35 +642,25 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     }
     else
     {
-#if MYSQL_VERSION_ID >= 50722 && MYSQL_VERSION_ID < 80000
-      if (stmt->result != NULL
-#else
-      if((stmt->result != NULL) && (stmt->result->field_alloc != NULL)
-#if (!MYSQLCLIENT_STATIC_LINKING || !MYSQL8)
-         && (stmt->result->field_alloc.pre_alloc != NULL )
-#endif
-
+      if(stmt->result && stmt->result->field_alloc
+#if (!MYSQL8)
+         && stmt->result->field_alloc->pre_alloc
 #endif
          )
       {
-#if (MYSQL8)
         free_root(stmt->result->field_alloc, MYF(0));
-#else
-        free_root(&stmt->result->field_alloc, MYF(0));
-#endif
       }
 
-      x_free(stmt->result);
+      stmt_result_free(stmt);
     }
 
-    x_free(stmt->fields);
+    x_free(stmt->fields);   // TODO: Looks like STMT::fields is not used anywhere
     x_free(stmt->result_array);
-    x_free(stmt->lengths);
     stmt->result= 0;
     stmt->fake_result= 0;
     stmt->fields= 0;
     stmt->result_array= 0;
-    stmt->lengths= 0;
+    stmt->free_lengths();
     stmt->current_values= 0;   /* For SQLGetData */
     stmt->fix_fields= 0;
     stmt->affected_rows= 0;
@@ -714,6 +701,10 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
       x_free(stmt->array);
       stmt->array= 0;
       ssps_close(stmt);
+      if (stmt->ssps != NULL)
+      {
+        free_result_bind(stmt);
+      }
     }
 
     if (fOption == SQL_CLOSE)
@@ -752,7 +743,11 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     desc_free(stmt->ird);
 
     x_free(stmt->cursor.name);
-
+    if (stmt->ssps != NULL)
+    {
+      mysql_stmt_close(stmt->ssps);
+      stmt->ssps = NULL;
+    }
     delete_parsed_query(&stmt->query);
     delete_parsed_query(&stmt->orig_query);
     delete_param_bind(stmt->param_bind);
