@@ -255,15 +255,15 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_DRIVER_NAME:
 #ifdef MYODBC_UNICODEDRIVER
 # ifdef WIN32
-    MYINFO_SET_STR("myodbc5w.dll");
+    MYINFO_SET_STR("myodbc8w.dll");
 # else
-    MYINFO_SET_STR("libmyodbc5w.so");
+    MYINFO_SET_STR("libmyodbc8w.so");
 # endif
 #else
 # ifdef WIN32
-    MYINFO_SET_STR("myodbc5a.dll");
+    MYINFO_SET_STR("myodbc8a.dll");
 # else
-    MYINFO_SET_STR("libmyodbc5a.so");
+    MYINFO_SET_STR("libmyodbc8a.so");
 # endif
 #endif
   case SQL_DRIVER_ODBC_VER:
@@ -588,7 +588,10 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_STR("Y");
 
   case SQL_MAX_SCHEMA_NAME_LEN:
-    MYINFO_SET_USHORT(0);
+    if (!dbc->ds->no_catalog)
+      MYINFO_SET_USHORT(64);
+    else
+      MYINFO_SET_USHORT(0);
 
   case SQL_MAX_STATEMENT_LEN:
     MYINFO_SET_ULONG(dbc->net_buffer_len);
@@ -684,7 +687,12 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     MYINFO_SET_STR("");
 
   case SQL_SCHEMA_USAGE:
-    MYINFO_SET_ULONG(0);
+    if (!dbc->ds->no_catalog)
+      MYINFO_SET_ULONG(SQL_SU_DML_STATEMENTS | SQL_SU_PROCEDURE_INVOCATION |
+                       SQL_SU_TABLE_DEFINITION | SQL_SU_INDEX_DEFINITION |
+                       SQL_SU_PRIVILEGE_DEFINITION);
+    else
+      MYINFO_SET_ULONG(0);
 
   case SQL_SCROLL_OPTIONS:
     MYINFO_SET_ULONG(SQL_SO_FORWARD_ONLY |
@@ -1056,6 +1064,12 @@ SQLRETURN SQL_API MySQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   x_free(stmt->result_array);
   stmt->result_array = (char **)myodbc_malloc(sizeof(SQL_GET_TYPE_INFO_values),
                                               MYF(MY_FAE | MY_ZEROFILL));
+  if (!stmt->result || !stmt->result_array)
+  {
+    stmt_result_free(stmt);
+    x_free(stmt->result_array);
+    return set_stmt_error(stmt, "S1001", "Not enough memory", 4001);
+  }
 
   if (fSqlType == SQL_ALL_TYPES)
   {
@@ -1066,6 +1080,7 @@ SQLRETURN SQL_API MySQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   }
   else
   {
+    stmt->result->row_count= 0;
     for (i = 0; i < MYSQL_DATA_TYPES; ++i)
     {
       if (atoi(SQL_GET_TYPE_INFO_values[i][1]) == fSqlType ||
@@ -1100,21 +1115,21 @@ void init_getfunctions(void)
   my_int2str(SQL_SMALLINT, sql_smallint, -10, 0);
   my_int2str(SQL_INTEGER, sql_integer, -10, 0);
   my_int2str(SQL_BIGINT, sql_bigint, -10, 0);
-  my_int2str(SQL_DECIMAL, sql_decimal, -10, 0);
-  my_int2str(SQL_NUMERIC, sql_numeric, -10, 0);
-  my_int2str(SQL_REAL, sql_real, -10, 0);
   my_int2str(SQL_FLOAT, sql_float, -10, 0);
+  my_int2str(SQL_REAL, sql_real, -10, 0);
   my_int2str(SQL_DOUBLE, sql_double, -10, 0);
   my_int2str(SQL_CHAR, sql_char, -10, 0);
   my_int2str(SQL_VARCHAR, sql_varchar, -10, 0);
   my_int2str(SQL_LONGVARCHAR, sql_longvarchar, -10, 0);
-  my_int2str(SQL_LONGVARBINARY, sql_longvarbinary, -10, 0);
+  my_int2str(SQL_TYPE_TIMESTAMP, sql_timestamp, -10, 0);
+  my_int2str(SQL_DECIMAL, sql_decimal, -10, 0);
+  my_int2str(SQL_NUMERIC, sql_numeric, -10, 0);
   my_int2str(SQL_VARBINARY, sql_varbinary, -10, 0);
+  my_int2str(SQL_TYPE_TIME, sql_time, -10, 0);
+  my_int2str(SQL_TYPE_DATE, sql_date, -10, 0);
+  my_int2str(SQL_LONGVARBINARY, sql_longvarbinary, -10, 0);
   my_int2str(SQL_BINARY, sql_binary, -10, 0);
   my_int2str(SQL_DATETIME, sql_datetime, -10, 0);
-  my_int2str(SQL_TYPE_TIMESTAMP, sql_timestamp, -10, 0);
-  my_int2str(SQL_TYPE_DATE, sql_date, -10, 0);
-  my_int2str(SQL_TYPE_TIME, sql_time, -10, 0);
 # if (ODBCVER < 0x0300)
   myodbc_sqlstate2_init();
   myodbc_ov2_inited = 1;
