@@ -119,7 +119,7 @@ BOOL ssps_get_out_params(STMT *stmt)
       {
         stmt->out_params_state= OPS_STREAMS_PENDING;
         stmt->current_param= ~0L;
-        reset_getdata_position(stmt);
+        stmt->reset_getdata_position();
       }
       else//(out_params & GOT_OUT_PARAMETERS)
       {
@@ -199,7 +199,7 @@ BOOL ssps_get_out_params(STMT *stmt)
               target= (char*)ptr_offset_adjust(aprec->data_ptr, stmt->apd->bind_offset_ptr,
                                     stmt->apd->bind_type, default_size, 0);
 
-              reset_getdata_position(stmt);
+              stmt->reset_getdata_position();
 
               if (iprec->parameter_type == SQL_PARAM_INPUT_OUTPUT
                || iprec->parameter_type == SQL_PARAM_OUTPUT)
@@ -308,6 +308,7 @@ void ssps_close(STMT *stmt)
     mysql_stmt_close(stmt->ssps);
     stmt->ssps= NULL;
   }
+  stmt->buf_set_pos(0);
 }
 
 
@@ -365,20 +366,17 @@ bool is_varlen_type(enum enum_field_types type)
   return (type == MYSQL_TYPE_BLOB ||
           type == MYSQL_TYPE_TINY_BLOB ||
           type == MYSQL_TYPE_MEDIUM_BLOB ||
-          type == MYSQL_TYPE_LONG_BLOB);
+          type == MYSQL_TYPE_LONG_BLOB ||
+          type == MYSQL_TYPE_VAR_STRING);
 }
 
 /* The structure and following allocation function are borrowed from c/c++ and adopted */
 typedef struct tagBST
 {
-  char * buffer;
-  size_t size;
+  char * buffer = NULL;
+  size_t size = 0;
   enum enum_field_types type;
 
-  tagBST() : buffer(NULL), size(0)
-  {
-  }
-
   tagBST(char *b, size_t s, enum enum_field_types t) :
     buffer(b), size(s), type(t)
   {}
@@ -501,17 +499,22 @@ allocate_buffer_for_field(const MYSQL_FIELD * const field, BOOL outparams)
 /* }}} */
 
 
-static MYSQL_ROW fetch_varlength_columns(STMT *stmt, MYSQL_ROW columns)
+static MYSQL_ROW fetch_varlength_columns(STMT *stmt, MYSQL_ROW values)
 {
   const unsigned int  num_fields= field_count(stmt);
   unsigned int i;
   uint desc_index= ~0L, stream_column= ~0L;
 
+  // Only do something if values have not been fetched yet
+  if (values)
+    return values;
+
   if (stmt->out_params_state == OPS_STREAMS_PENDING)
   {
     desc_find_outstream_rec(stmt, &desc_index, &stream_column);
   }
 
+  bool reallocated_buffers = false;
   for (i= 0; i < num_fields; ++i)
   {
 
@@ -529,15 +532,28 @@ static MYSQL_ROW fetch_varlength_columns(STMT *stmt, MYSQL_ROW columns)
         stmt->array[i]= (char*)myodbc_realloc(stmt->array[i], *stmt->result_bind[i].length,
           MYF(MY_ALLOW_ZERO_PTR));
         stmt->lengths[i]= *stmt->result_bind[i].length;
+        stmt->result_bind[i].buffer_length = *stmt->result_bind[i].length;
+        reallocated_buffers = true;
       }
 
-      stmt->result_bind[i].buffer= stmt->array[i];
-      stmt->result_bind[i].buffer_length= stmt->lengths[i];
+      stmt->result_bind[i].buffer = stmt->array[i];
+
+      if (stmt->lengths[i] > 0 /* || is_varlen_type(stmt->result_bind[i].buffer_type)*/)
+      {
+       // For fixed-length types we should not set zero length
+        stmt->result_bind[i].buffer_length = stmt->lengths[i];
+      }
+
+      if (reallocated_buffers)
+        mysql_stmt_fetch_column(stmt->ssps, &stmt->result_bind[i], i, 0);
 
-      mysql_stmt_fetch_column(stmt->ssps, &stmt->result_bind[i], i, 0);
     }
   }
 
+  // Result buffers must be set again after reallocating
+  if (reallocated_buffers)
+    mysql_stmt_bind_result(stmt->ssps, stmt->result_bind);
+
   fill_ird_data_lengths(stmt->ird, stmt->result_bind[0].length,
                                   stmt->result->field_count);
 
@@ -560,6 +576,156 @@ char *STMT::add_to_buffer(const char *from, size_t len)
   return tempbuf.add_to_buffer(from, len);
 }
 
+void STMT::free_lengths()
+{
+  if (lengths && lengths_allocated)
+  {
+    x_free(lengths);
+    lengths = nullptr;
+    lengths_allocated = false;
+  }
+}
+
+void STMT::alloc_lengths(size_t num)
+{
+  free_lengths();
+
+  lengths = (unsigned long*)myodbc_malloc(sizeof(unsigned long)*num,
+                                          MYF(MY_ZEROFILL));
+  lengths_allocated = true;
+}
+
+STMT::~STMT()
+{
+  free_lengths();
+}
+
+void STMT::reset_getdata_position()
+{
+  getdata.column = (uint)~0L;
+  getdata.source = NULL;
+  getdata.dst_bytes = (ulong)~0L;
+  getdata.dst_offset = (ulong)~0L;
+  getdata.src_offset = (ulong)~0L;
+  getdata.latest_bytes = getdata.latest_used = 0;
+}
+
+SQLRETURN STMT::set_error(myodbc_errid errid, const char *errtext,
+                         SQLINTEGER errcode)
+{
+  error = MYERROR(errid, errtext, errcode, dbc->st_error_prefix);
+  return error.retcode;
+}
+
+long STMT::compute_cur_row(unsigned fFetchType, SQLLEN irow)
+{
+  long cur_row = 0;
+  long max_row = (long)num_rows(this);
+
+  switch (fFetchType)
+  {
+  case SQL_FETCH_NEXT:
+    cur_row = (current_row < 0 ? 0 : current_row + rows_found_in_set);
+    break;
+  case SQL_FETCH_PRIOR:
+    cur_row = (current_row <= 0 ? -1 :
+      (long)(current_row - ard->array_size));
+    break;
+  case SQL_FETCH_FIRST:
+    cur_row = 0L;
+    break;
+  case SQL_FETCH_LAST:
+    cur_row = max_row - ard->array_size;
+    break;
+  case SQL_FETCH_ABSOLUTE:
+    if (irow < 0)
+    {
+      /* Fetch from end of result set */
+      if (max_row + irow < 0 && -irow <= (long)ard->array_size)
+      {
+        /*
+          | FetchOffset | > LastResultRow AND
+          | FetchOffset | <= RowsetSize
+        */
+        cur_row = 0;     /* Return from beginning */
+      }
+      else
+        cur_row = max_row + irow;     /* Ok if max_row <= -irow */
+    }
+    else
+      cur_row = (long)irow - 1;
+    break;
+
+  case SQL_FETCH_RELATIVE:
+    cur_row = current_row + irow;
+    if (current_row > 0 && cur_row < 0 &&
+      (long)-irow <= (long)ard->array_size)
+    {
+      cur_row = 0;
+    }
+    break;
+
+  case SQL_FETCH_BOOKMARK:
+  {
+    cur_row = irow;
+    if (cur_row < 0 && (long)-irow <= (long)ard->array_size)
+    {
+      cur_row = 0;
+    }
+  }
+  break;
+
+  default:
+    set_error(MYERR_S1106, "Fetch type out of range", 0);
+    throw error;
+  }
+
+  if (cur_row < 0)
+  {
+    current_row = -1;  /* Before first row */
+    rows_found_in_set = 0;
+    data_seek(this, 0L);
+    throw MYERROR(SQL_NO_DATA_FOUND);
+  }
+  if (cur_row > max_row)
+  {
+    if (scroller_exists(this))
+    {
+      while (cur_row > (max_row = (long)scroller_move(this)));
+
+      switch (scroller_prefetch(this))
+      {
+      case SQL_NO_DATA:
+        throw MYERROR(SQL_NO_DATA_FOUND);
+      case SQL_ERROR:
+        set_error(MYERR_S1000, mysql_error(&dbc->mysql), 0);
+        throw error;
+      }
+    }
+    else
+    {
+      cur_row = max_row;
+    }
+  }
+
+  if (!result_array && !if_forward_cache(this))
+  {
+    /*
+      If Dynamic, it loses the stmt->end_of_set, so
+      seek to desired row, might have new data or
+      might be deleted
+    */
+    if (stmt_options.cursor_type != SQL_CURSOR_DYNAMIC &&
+      cur_row && cur_row == (long)(current_row + rows_found_in_set))
+      row_seek(this, this->end_of_set);
+    else
+      data_seek(this, cur_row);
+  }
+  current_row = cur_row;
+  return current_row;
+
+}
+
 int STMT::ssps_bind_result()
 {
   const unsigned int  num_fields= field_count(this);
@@ -572,27 +738,27 @@ int STMT::ssps_bind_result()
 
   if (result_bind)
   {
-    /* We have fields requiring to read real length first */
-    if (fix_fields != NULL)
-    {
-      for (i=0; i < num_fields; ++i)
-      {
-        /* length marks such fields */
-        if (lengths[i] > 0)
-        {
-          if (result_bind[i].buffer == array[i])
-          {
-            /* make sure we do not free it twice */
-            array[i]= NULL;
-            lengths[i]= 0;
-          }
-          /* Resetting buffer and buffer_length for those fields */
-          x_free(result_bind[i].buffer);
-          result_bind[i].buffer       = 0;
-          result_bind[i].buffer_length= 0;
-        }
-      }
-    }
+    ///* We have fields requiring to read real length first */
+    //if (fix_fields != NULL)
+    //{
+    //  for (i=0; i < num_fields; ++i)
+    //  {
+    //    /* length marks such fields */
+    //    if (lengths[i] > 0)
+    //    {
+    //      if (result_bind[i].buffer == array[i])
+    //      {
+    //        /* make sure we do not free it twice */
+    //        array[i]= NULL;
+    //        lengths[i]= 0;
+    //      }
+    //      /* Resetting buffer and buffer_length for those fields */
+    //      x_free(result_bind[i].buffer);
+    //      result_bind[i].buffer       = 0;
+    //      result_bind[i].buffer_length= 0;
+    //    }
+    //  }
+    //}
   }
   else
   {
@@ -634,10 +800,7 @@ int STMT::ssps_bind_result()
 
         /* Need to alloc it only once*/
         if (lengths == NULL)
-        {
-          lengths= (unsigned long*)myodbc_malloc(sizeof(unsigned long)*num_fields, MYF(MY_ZEROFILL));
-        }
-        /* Buffer of initial length? */
+          alloc_lengths(num_fields);
       }
     }
 
@@ -647,15 +810,17 @@ int STMT::ssps_bind_result()
   return 0;
 }
 
-
-BOOL ssps_0buffers_truncated_only(STMT *stmt)
+/*
+  Function determines if the buffers need to be extended
+*/
+BOOL ssps_buffers_need_extending(STMT *stmt)
 {
-  if (stmt->fix_fields == NULL)
-  {
-    /* That is enough to tell that not */
-    return FALSE;
-  }
-  else
+  //if (stmt->fix_fields == NULL)
+  //{
+  //  /* That is enough to tell that not */
+  //  return FALSE;
+  //}
+  //else
   {
     const unsigned int  num_fields= field_count(stmt);
     unsigned int i;
@@ -663,14 +828,14 @@ BOOL ssps_0buffers_truncated_only(STMT *stmt)
     for (i= 0; i < num_fields; ++i)
     {
       if (*stmt->result_bind[i].error != 0
-        && stmt->result_bind[i].buffer_length >= (*stmt->result_bind[i].length))
+        && stmt->result_bind[i].buffer_length < (*stmt->result_bind[i].length))
       {
-        return FALSE;
+        return TRUE;
       }
     }
   }
 
-  return TRUE;
+  return FALSE;
 }
 
 
