@@ -38,7 +38,7 @@
 
 BOOL ssps_used(STMT *stmt)
 {
-  return stmt->ssps != NULL;
+  return (stmt->ssps != NULL);
 }
 
 
@@ -78,13 +78,7 @@ my_bool free_current_result(STMT *stmt)
     }
     free_internal_result_buffers(stmt);
     /* We need to always free stmt->result because SSPS keep metadata there */
-    if (stmt->fake_result)
-    {
-      x_free(stmt->result);
-    }
-    else
-      mysql_free_result(stmt->result);
-
+    stmt_result_free(stmt);
     stmt->result= NULL;
   }
   return res;
@@ -218,11 +212,16 @@ MYSQL_ROW fetch_row(STMT *stmt)
       return NULL;
     }
 
-    if ((error= mysql_stmt_fetch(stmt->ssps)))
+    error= mysql_stmt_fetch(stmt->ssps);
+
     {
-      if (error != MYSQL_DATA_TRUNCATED || !ssps_0buffers_truncated_only(stmt))
+
+      if (error == MYSQL_NO_DATA)
+        return nullptr;
+
+      if (stmt->fix_fields)
       {
-        return NULL;
+        return stmt->fix_fields(stmt, nullptr); // it returns stmt->array
       }
     }
 
@@ -372,7 +371,8 @@ BOOL is_null(STMT *stmt, ulong column_number, char *value)
 /* Prepares statement depending on connection option either on a client or
    on a server. Returns SQLRETURN result code since preparing on client or
    server can produce errors, memory allocation to name one.  */
-SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length)
+SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
+          bool reset_sql_limit)
 {
   /* TODO: I guess we always have to have query length here */
   if (query_length <= 0)
@@ -387,7 +387,7 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length)
      If that changes we will need to make "parse" to set error and return rc */
   if (parse(&stmt->query))
   {
-    return set_error(stmt, MYERR_S1001, NULL, 4001);
+    return stmt->set_error( MYERR_S1001, NULL, 4001);
   }
 
   ssps_close(stmt);
@@ -404,7 +404,15 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length)
        it at the moment */
     if (!get_cursor_name(&stmt->query))
     {
-      if (mysql_stmt_prepare(stmt->ssps, query, query_length))
+     myodbc_mutex_lock(&stmt->dbc->lock);
+
+     if (reset_sql_limit)
+        set_sql_select_limit(stmt->dbc, 0, false);
+
+     int prep_res = mysql_stmt_prepare(stmt->ssps, query, query_length);
+     myodbc_mutex_unlock(&stmt->dbc->lock);
+
+     if (prep_res)
       {
         MYLOG_QUERY(stmt, mysql_error(&stmt->dbc->mysql));
 
@@ -427,6 +435,7 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length)
       }
 
       /* Getting result metadata */
+      stmt->fake_result = false;  // reset in case it was set before
       if ((stmt->result= mysql_stmt_result_metadata(stmt->ssps)))
       {
         /*stmt->state= ST_SS_PREPARED;*/
@@ -465,7 +474,7 @@ SQLRETURN append2param_value(STMT *stmt, DESCREC * aprec, const char *chunk, uns
                                         aprec->par.value_length + length + 1,
                                         MYF(0))) )
     {
-      return set_error(stmt,MYERR_S1001,NULL,4001);
+      return stmt->set_error(MYERR_S1001,NULL,4001);
     }
 
     memcpy(aprec->par.value+aprec->par.value_length,chunk,length);
@@ -478,7 +487,7 @@ SQLRETURN append2param_value(STMT *stmt, DESCREC * aprec, const char *chunk, uns
     /* New value */
     if ( !(aprec->par.value= (char*)myodbc_malloc(length+1,MYF(0))) )
     {
-      return set_error(stmt,MYERR_S1001,NULL,4001);
+      return stmt->set_error(MYERR_S1001,NULL,4001);
     }
     memcpy(aprec->par.value,chunk,length);
     aprec->par.value_length= length;
