@@ -45,7 +45,7 @@
 
 #define if_dynamic_cursor(st) ((st)->stmt_options.cursor_type == SQL_CURSOR_DYNAMIC)
 #define if_forward_cache(st) ((st)->stmt_options.cursor_type == SQL_CURSOR_FORWARD_ONLY && \
-			     (st)->dbc->ds->dont_cache_result)
+           (st)->dbc->ds->dont_cache_result)
 #define is_connected(dbc)    ((dbc)->mysql.net.vio)
 #define trans_supported(db) ((db)->mysql.server_capabilities & CLIENT_TRANSACTIONS)
 #define autocommit_on(db) ((db)->mysql.server_status & SERVER_STATUS_AUTOCOMMIT)
@@ -76,14 +76,8 @@
 #define MAX32_BUFF_SIZE 11
 #define MAX64_BUFF_SIZE 21
 
-/* Wrappers to hide differences in client library versions. */
-#if MYSQL_VERSION_ID >= 40100
-# define my_int2str(val, dst, radix, upcase) \
-    int2str((val), (dst), (radix), (upcase))
-#else
-# define my_int2str(val, dst, radix, upcase) \
-    int2str((val), (dst), (radix))
-#endif
+#define my_int2str(val, dst, radix, upcase) \
+    myodbc_int10_to_str((val), (dst), (radix))
 
 #if MYSQL_VERSION_ID >= 50100
 typedef unsigned char * DYNAMIC_ELEMENT;
@@ -131,9 +125,9 @@ typedef char * DYNAMIC_ELEMENT;
   Utility function prototypes that share among files
 */
 
-CHARSET_INFO *myodbc_get_charset(uint cs_number, myf flags);
-SQLRETURN         my_SQLPrepare (SQLHSTMT hstmt, SQLCHAR *szSqlStr, SQLINTEGER cbSqlStr,
-                                my_bool dupe);
+SQLRETURN         my_SQLPrepare (SQLHSTMT hstmt, SQLCHAR *szSqlStr,
+                                 SQLINTEGER cbSqlStr, bool dupe,
+                                 bool reset_select_limit);
 SQLRETURN         my_SQLExecute         (STMT * stmt);
 SQLRETURN SQL_API my_SQLFreeStmt        (SQLHSTMT hstmt,SQLUSMALLINT fOption);
 SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,
@@ -150,9 +144,9 @@ void      fix_result_types  (STMT *stmt);
 char *    fix_str           (char *to,const char *from,int length);
 char *    dupp_str          (char *from,int length);
 SQLRETURN my_pos_delete (STMT *stmt,STMT *stmtParam,
-			                  SQLUSMALLINT irow,DYNAMIC_STRING *dynStr);
+                        SQLUSMALLINT irow,DYNAMIC_STRING *dynStr);
 SQLRETURN my_pos_update (STMT *stmt,STMT *stmtParam,
-			                  SQLUSMALLINT irow,DYNAMIC_STRING *dynStr);
+                        SQLUSMALLINT irow,DYNAMIC_STRING *dynStr);
 char *    check_if_positioned_cursor_exists (STMT *stmt, STMT **stmtNew);
 SQLRETURN insert_param  (STMT *stmt, MYSQL_BIND *bind, DESC *apd,
                         DESCREC *aprec, DESCREC *iprec, SQLULEN row);
@@ -176,9 +170,15 @@ copy_binary_result(STMT *stmt,
                    SQLCHAR *result, SQLLEN result_bytes, SQLLEN *used_bytes,
                    MYSQL_FIELD *field, char *src, unsigned long src_bytes);
 SQLRETURN copy_binhex_result(STMT *stmt,
-			     SQLCHAR *rgbValue, SQLINTEGER cbValueMax,
-			     SQLLEN *pcbValue, MYSQL_FIELD *field, char *src,
-			     ulong src_length);
+           SQLCHAR *rgbValue, SQLINTEGER cbValueMax,
+           SQLLEN *pcbValue, MYSQL_FIELD *field, char *src,
+           ulong src_length);
+SQLRETURN copy_bit_result(STMT *stmt,
+                          SQLCHAR *result, SQLLEN result_bytes, SQLLEN *used_bytes,
+                          MYSQL_FIELD *field, char *src, unsigned long src_bytes);
+SQLRETURN wcopy_bit_result(STMT *stmt,
+                          SQLWCHAR *result, SQLLEN result_bytes, SQLLEN *used_bytes,
+                          MYSQL_FIELD *field, char *src, unsigned long src_bytes);
 SQLRETURN copy_wchar_result(STMT *stmt,
                             SQLWCHAR *rgbValue, SQLINTEGER cbValueMax,
                             SQLLEN *pcbValue, MYSQL_FIELD *field, char *src,
@@ -186,7 +186,7 @@ SQLRETURN copy_wchar_result(STMT *stmt,
 
 SQLRETURN set_dbc_error   (DBC *dbc, char *state,const char *message,uint errcode);
 #define   set_stmt_error  myodbc_set_stmt_error
-SQLRETURN set_stmt_error  (STMT *stmt, const char *state, const char *message, uint errcode);
+SQLRETURN myodbc_set_stmt_error  (STMT *stmt, const char *state, const char *message, uint errcode);
 SQLRETURN set_desc_error  (DESC *desc, char *state,
                           const char *message, uint errcode);
 SQLRETURN handle_connection_error (STMT *stmt);
@@ -223,16 +223,16 @@ SQLLEN      get_bookmark_value                  (SQLSMALLINT fCType, SQLPOINTER
    (field)->type == MYSQL_TYPE_NEWDECIMAL)
 
 SQLRETURN SQL_API my_SQLBindParameter(SQLHSTMT hstmt,SQLUSMALLINT ipar,
-				      SQLSMALLINT fParamType,
-				      SQLSMALLINT fCType, SQLSMALLINT fSqlType,
-				      SQLULEN cbColDef,
-				      SQLSMALLINT ibScale,
-				      SQLPOINTER  rgbValue,
-				      SQLLEN cbValueMax,
-				      SQLLEN *pcbValue);
+              SQLSMALLINT fParamType,
+              SQLSMALLINT fCType, SQLSMALLINT fSqlType,
+              SQLULEN cbColDef,
+              SQLSMALLINT ibScale,
+              SQLPOINTER  rgbValue,
+              SQLLEN cbValueMax,
+              SQLLEN *pcbValue);
 SQLRETURN SQL_API my_SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType,
-				      SQLLEN irow, SQLULEN *pcrow,
-				      SQLUSMALLINT *rgfRowStatus, my_bool upd_status);
+              SQLLEN irow, SQLULEN *pcrow,
+              SQLUSMALLINT *rgfRowStatus, my_bool upd_status);
 SQLRETURN SQL_API myodbc_single_fetch( SQLHSTMT hstmt, SQLUSMALLINT fFetchType,
               SQLLEN irow, SQLULEN *pcrow,
               SQLUSMALLINT *rgfRowStatus, my_bool upd_status);
@@ -262,18 +262,16 @@ void  myodbc_sqlstate2_init     (void);
 void  myodbc_sqlstate3_init     (void);
 int   check_if_server_is_alive  (DBC *dbc);
 
-my_bool   dynstr_append_quoted_name (DYNAMIC_STRING *str, const char *name);
+my_bool   myodbc_append_quoted_name (DYNAMIC_STRING *str, const char *name);
 SQLRETURN set_handle_error          (SQLSMALLINT HandleType, SQLHANDLE handle,
-			                              myodbc_errid errid, const char *errtext, SQLINTEGER errcode);
-SQLRETURN set_error     (STMT *stmt,myodbc_errid errid, const char *errtext,
-		                    SQLINTEGER errcode);
+                                    myodbc_errid errid, const char *errtext, SQLINTEGER errcode);
 SQLRETURN set_conn_error(DBC *dbc,myodbc_errid errid, const char *errtext,
-			                  SQLINTEGER errcode);
+                        SQLINTEGER errcode);
 SQLRETURN set_env_error (ENV * env,myodbc_errid errid, const char *errtext,
-			                  SQLINTEGER errcode);
+                        SQLINTEGER errcode);
 SQLRETURN copy_str_data (SQLSMALLINT HandleType, SQLHANDLE Handle,
-			                  SQLCHAR *rgbValue, SQLSMALLINT cbValueMax,
-			                  SQLSMALLINT *pcbValue,char *src);
+                        SQLCHAR *rgbValue, SQLSMALLINT cbValueMax,
+                        SQLSMALLINT *pcbValue,char *src);
 SQLRETURN SQL_API my_SQLAllocEnv      (SQLHENV * phenv);
 SQLRETURN SQL_API my_SQLAllocConnect  (SQLHENV henv, SQLHDBC *phdbc);
 SQLRETURN SQL_API my_SQLFreeConnect   (SQLHDBC hdbc);
@@ -382,15 +380,7 @@ long long     binary2numeric        (long long *dst, char *src, uint srcLen);
 void          fill_ird_data_lengths (DESC *ird, ulong *lengths, uint fields);
 
 /* Functions to work with prepared and regular statements  */
-
-#ifdef SERVER_PS_OUT_PARAMS
-# define IS_PS_OUT_PARAMS(_stmt) ((_stmt)->dbc->mysql.server_status & SERVER_PS_OUT_PARAMS)
-#else
-/* In case if driver is built against old libmysl. In fact is not quite
-   correct */
-# define IS_PS_OUT_PARAMS(_stmt) (ssps_used(_stmt) && is_call_procedure(&_stmt->query) && !mysql_more_results(&(_stmt)->dbc->mysql))
-#endif
-
+#define IS_PS_OUT_PARAMS(_stmt) ((_stmt)->dbc->mysql.server_status & SERVER_PS_OUT_PARAMS)
 /* my_stmt.c */
 BOOL              ssps_used           (STMT *stmt);
 BOOL              returned_result     (STMT *stmt);
@@ -420,7 +410,26 @@ char *        get_string  (STMT *stmt, ulong column_number, char *value,
 long double   get_double  (STMT *stmt, ulong column_number, char *value,
                           ulong length);
 BOOL          is_null     (STMT *stmt, ulong column_number, char *value);
-SQLRETURN     prepare     (STMT *stmt, char * query, SQLINTEGER query_length);
+SQLRETURN     prepare     (STMT *stmt, char * query, SQLINTEGER query_length,
+                           bool reset_sql_limit);
+
+
+inline
+void stmt_result_free(STMT * stmt)
+{
+  if (!stmt->result)
+    return;
+
+  if (stmt->fake_result)
+  {
+    x_free(stmt->result);
+  }
+  else
+    mysql_free_result(stmt->result);
+
+  stmt->result = NULL;
+}
+
 
 /* scroller-related functions */
 void          scroller_reset      (STMT *stmt);
@@ -442,7 +451,7 @@ void        ssps_close            (STMT *stmt);
 SQLRETURN   ssps_fetch_chunk      (STMT *stmt, char *dest, unsigned long dest_bytes,
                                   unsigned long *avail_bytes);
 void        free_result_bind      (STMT *stmt);
-BOOL        ssps_0buffers_truncated_only(STMT *stmt);
+BOOL        ssps_buffers_need_extending(STMT *stmt);
 long long   ssps_get_int64        (STMT *stmt, ulong column_number, char *value,
                                   ulong length);
 long double ssps_get_double       (STMT *stmt, ulong column_number, char *value,
@@ -453,6 +462,8 @@ SQLRETURN   ssps_send_long_data   (STMT *stmt, unsigned int param_num, const cha
                                   unsigned long length);
 MYSQL_BIND * get_param_bind       (STMT *stmt, unsigned int param_number, int reset);
 
+bool is_varlen_type(enum enum_field_types type);
+
 /* connect.c */
 void free_connection_stmts(DBC *dbc);
 
@@ -527,7 +538,7 @@ void free_connection_stmts(DBC *dbc);
 
 #define CHECK_DESC_OUTPUT(d, s) CHECK_STMT_OUTPUT(d, s)
 
-#define CHECK_DATA_OUTPUT(s, d) if(d == NULL) return set_error((STMT *)s, MYERR_S1000, "Invalid output buffer", 0)
+#define CHECK_DATA_OUTPUT(s, d) if(d == NULL) return ((STMT *)s)->set_error(MYERR_S1000, "Invalid output buffer", 0)
 
 #define IF_NOT_NULL(v, x) if (v != NULL) x
 
