@@ -39,17 +39,6 @@
 #define SQL_MY_PRIMARY_KEY 1212
 
 
-void reset_getdata_position(STMT *stmt)
-{
-  stmt->getdata.column= (uint) ~0L;
-  stmt->getdata.source= NULL;
-  stmt->getdata.dst_bytes= (ulong) ~0L;
-  stmt->getdata.dst_offset= (ulong) ~0L;
-  stmt->getdata.src_offset= (ulong) ~0L;
-  stmt->getdata.latest_bytes= stmt->getdata.latest_used= 0;
-}
-
-
 /* Verifies if C type is suitable for copying SQL_BINARY data
    http://msdn.microsoft.com/en-us/library/ms713559%28VS.85%29.aspx */
 my_bool is_binary_ctype( SQLSMALLINT cType)
@@ -357,7 +346,7 @@ sql_get_bookmark_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
     break;
 
   default:
-    return set_error(stmt,MYERR_07006,
+    return stmt->set_error(MYERR_07006,
                      "Restricted data type attribute violation",0);
     break;
   }
@@ -463,13 +452,21 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
 
     if (field->type == MYSQL_TYPE_BIT)
     {
-      if (is_binary_ctype(fCType))
+      switch(fCType)
       {
+      case SQL_C_BIT:
         return copy_binary_result(stmt, (SQLCHAR *)rgbValue, cbValueMax,
                                   pcbValue, field, value, length);
-      }
-      else
-      {
+
+      //https://docs.microsoft.com/en-us/sql/odbc/reference/appendixes/sql-to-c-bit?view=sql-server-ver15
+      //When bit SQL data is converted to character C data, the possible values are "0" and "1".
+      case SQL_C_CHAR:
+        return copy_bit_result(stmt, (SQLCHAR *)rgbValue, cbValueMax,
+                               pcbValue, field, value, length);
+      case SQL_C_WCHAR:
+        return wcopy_bit_result(stmt, (SQLWCHAR *)rgbValue, cbValueMax,
+                                pcbValue, field, value, length);
+      default:
         binary2numeric(&numericValue, value, length);
         convert= 0;
       }
@@ -490,9 +487,10 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
         Handle BLOB -> CHAR conversion
         Conversion only for field which is having binary character set (63)
       */
-      if (((field->flags & (BLOB_FLAG|BINARY_FLAG))
-                == (BLOB_FLAG|BINARY_FLAG) )
-                && field->charsetnr == BINARY_CHARSET_NUMBER )
+      if ((field->flags & BINARY_FLAG) &&
+        field->charsetnr == BINARY_CHARSET_NUMBER &&
+        IS_LONGDATA(field->type) &&
+        !field->decimals)
       {
         return copy_binhex_result(stmt,
                                   (SQLCHAR *)rgbValue, cbValueMax, pcbValue,
@@ -539,6 +537,34 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
       {
         char *tmp= get_string(stmt, column_number, value, &length, as_string);
 
+        if ((field->flags & BINARY_FLAG) &&
+          field->charsetnr == BINARY_CHARSET_NUMBER &&
+          IS_LONGDATA(field->type) &&
+          !field->decimals)
+        {
+          /*
+            1. convert binary data to hex into a temporary buffer
+            2. convert hex in the temporary buffer to WCHAR
+          */
+
+          SQLCHAR *c_buf = (SQLCHAR*)myodbc_malloc(length*2 + 1, MYF(0));
+          SQLRETURN rc2 = SQL_SUCCESS, rc3 = SQL_SUCCESS;
+
+          rc2 = copy_binhex_result(stmt, c_buf, cbValueMax, pcbValue,
+                                   field, tmp, length);
+
+          rc3 = copy_wchar_result(stmt, (SQLWCHAR *)rgbValue,
+                  (SQLINTEGER)(cbValueMax / sizeof(SQLWCHAR)), pcbValue,
+                  field, (char*)c_buf, length*2);
+
+          x_free(c_buf);
+
+          if (rc3 == SQL_SUCCESS)
+            return rc2;
+
+          return rc3;
+        }
+
         return copy_wchar_result(stmt, (SQLWCHAR *)rgbValue,
                         (SQLINTEGER)(cbValueMax / sizeof(SQLWCHAR)), pcbValue,
                         field, tmp, length);
@@ -705,7 +731,7 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
         }
         else
         {
-          return set_error(stmt,MYERR_07006,
+          return stmt->set_error(MYERR_07006,
                        "Restricted data type attribute violation",0);
         }
 
@@ -886,7 +912,10 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
         if (rgbValue)
         {
           if (convert)
+          {
             sqlnum_from_str(get_string(stmt, column_number, value, &length, as_string), sqlnum, &overflow);
+            *pcbValue = sizeof(SQL_NUMERIC_STRUCT);
+          }
           else /* bit field */
           {
             /* Lazy way - converting number we have to a string.
@@ -896,19 +925,24 @@ sql_get_data(STMT *stmt, SQLSMALLINT fCType, uint column_number,
             sprintf(_value, "%llu", numericValue);
 
             sqlnum_from_str(_value, sqlnum, &overflow);
+            *pcbValue = sizeof(ulonglong);
           }
 
         }
-        *pcbValue= sizeof(ulonglong);
 
-        if (overflow)
+        if (overflow == 1)
           return set_stmt_error(stmt, "22003",
                                 "Numeric value out of range", 0);
+        else if (overflow == 2)
+        {
+          set_stmt_error(stmt, "01S07", "Fractional truncation", 0);
+          result = SQL_SUCCESS_WITH_INFO;
+        }
       }
       break;
 
     default:
-      return set_error(stmt,MYERR_07006,
+      return stmt->set_error(MYERR_07006,
                        "Restricted data type attribute violation",0);
       break;
     }
@@ -1165,7 +1199,7 @@ MySQLColAttribute(SQLHSTMT hstmt, SQLUSMALLINT column,
   }
 
   if (column == 0 || column > stmt->ird->count)
-    return set_error((STMT*)hstmt,  MYERR_07009, NULL, 0);
+    return ((STMT*)hstmt)->set_error(MYERR_07009, NULL, 0);
 
   if (!num_attr)
     num_attr= &nparam;
@@ -1431,6 +1465,7 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT      StatementHandle,
     SQLRETURN result;
     ulong length= 0;
     DESCREC *irrec, *arrec;
+    DECLARE_LOCALE_HANDLE
     /*
       Signed column number required for bookmark column 0,
       which will become -1 when decremented later.
@@ -1489,16 +1524,14 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT      StatementHandle,
     if (sColNum != stmt->getdata.column)
     {
       /* New column. Reset old offset */
-      reset_getdata_position(stmt);
+      stmt->reset_getdata_position();
       stmt->getdata.column= sColNum;
     }
     irrec= desc_get_rec(stmt->ird, sColNum, FALSE);
 
     assert(irrec);
 
-    if (!stmt->dbc->ds->dont_use_set_locale)
-      setlocale(LC_NUMERIC, "C");
-
+    C_LOCALE_SET(stmt)
 
     if ((sColNum == -1 && stmt->stmt_options.bookmarks == SQL_UB_VARIABLE))
     {
@@ -1526,8 +1559,7 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT      StatementHandle,
                           arrec);
     }
 
-    if (!stmt->dbc->ds->dont_use_set_locale)
-        setlocale(LC_NUMERIC,default_locale);
+    DEFAULT_LOCALE_SET(stmt)
 
     return result;
 }
@@ -1580,8 +1612,12 @@ SQLRETURN SQL_API SQLMoreResults( SQLHSTMT hStmt )
       case CR_SERVER_LOST:
         nReturn = set_stmt_error( pStmt, "08S01", mysql_error( &pStmt->dbc->mysql ), nRetVal );
         goto exitSQLMoreResults;
+      case CR_COMMANDS_OUT_OF_SYNC:
+      case CR_UNKNOWN_ERROR:
+        nReturn = set_stmt_error( pStmt, "HY000", mysql_error( &pStmt->dbc->mysql ), nRetVal );
+        goto exitSQLMoreResults;
       default:
-        nReturn = set_stmt_error(pStmt, "HY000", mysql_error(&pStmt->dbc->mysql), nRetVal);
+        nReturn = set_stmt_error( pStmt, "HY000", "unhandled error from mysql_next_result()", nRetVal );
         goto exitSQLMoreResults;
     }
   }
@@ -1737,7 +1773,7 @@ fill_fetch_bookmark_buffers(STMT *stmt, ulong value, uint rownum)
     SQLPOINTER TargetValuePtr= NULL;
     ulong copy_bytes= 0;
 
-    reset_getdata_position(stmt);
+    stmt->reset_getdata_position();
 
     if (arrec->data_ptr)
     {
@@ -1803,7 +1839,7 @@ fill_fetch_buffers(STMT *stmt, MYSQL_ROW values, uint rownum)
       SQLLEN *pcbValue= NULL;
       SQLPOINTER TargetValuePtr= NULL;
 
-      reset_getdata_position(stmt);
+      stmt->reset_getdata_position();
 
       if (arrec->data_ptr)
       {
@@ -1876,6 +1912,7 @@ SQLRETURN SQL_API myodbc_single_fetch( SQLHSTMT             hstmt,
     SQLULEN           dummy_pcrow;
     BOOL              disconnected= FALSE;
     long              brow= 0;
+    DECLARE_LOCALE_HANDLE
 
     if ( !stmt->result )
       return set_stmt_error(stmt, "24000", "Fetch without a SELECT", 0);
@@ -1888,7 +1925,7 @@ SQLRETURN SQL_API myodbc_single_fetch( SQLHSTMT             hstmt,
     /* for scrollable cursor("scroller") max_row is max row for currently
        fetched part of resultset */
     max_row= (long) num_rows(stmt);
-    reset_getdata_position(stmt);
+    stmt->reset_getdata_position();
     stmt->current_values= 0;          /* For SQLGetData */
 
     switch ( fFetchType )
@@ -1962,7 +1999,7 @@ SQLRETURN SQL_API myodbc_single_fetch( SQLHSTMT             hstmt,
         break;
 
       default:
-          return set_error(stmt, MYERR_S1106, "Fetch type out of range", 0);
+          return stmt->set_error( MYERR_S1106, "Fetch type out of range", 0);
     }
 
     if ( cur_row < 0 )
@@ -1984,7 +2021,7 @@ SQLRETURN SQL_API myodbc_single_fetch( SQLHSTMT             hstmt,
           case SQL_NO_DATA:
             set_stmt_error(stmt, "01S07", "One or more row has error.", 0);
             return SQL_SUCCESS_WITH_INFO; //SQL_NO_DATA_FOUND
-          case SQL_ERROR:   return set_error(stmt,MYERR_S1000,
+          case SQL_ERROR:   return stmt->set_error(MYERR_S1000,
                                             mysql_error(&stmt->dbc->mysql), 0);
         }
       }
@@ -2043,10 +2080,7 @@ SQLRETURN SQL_API myodbc_single_fetch( SQLHSTMT             hstmt,
       }
     }
 
-    if (!stmt->dbc->ds->dont_use_set_locale)
-    {
-      setlocale(LC_NUMERIC, "C");
-    }
+    C_LOCALE_SET(stmt)
 
     res= SQL_SUCCESS;
     {
@@ -2163,10 +2197,7 @@ exitSQLSingleFetch:
       stmt->end_of_set= row_seek(stmt, save_position);
     }
 
-    if (!stmt->dbc->ds->dont_use_set_locale)
-    {
-      setlocale(LC_NUMERIC,default_locale);
-    }
+    DEFAULT_LOCALE_SET(stmt)
 
     if (SQL_SUCCEEDED(res)
       && stmt->rows_found_in_set < stmt->ard->array_size)
@@ -2211,6 +2242,7 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
     SQLULEN           dummy_pcrow;
     BOOL              disconnected= FALSE;
     long              brow= 0;
+    DECLARE_LOCALE_HANDLE
 
     if ( !stmt->result )
       return set_stmt_error(stmt, "24000", "Fetch without a SELECT", 0);
@@ -2237,12 +2269,12 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
     if ( stmt->stmt_options.cursor_type == SQL_CURSOR_FORWARD_ONLY )
     {
       if ( fFetchType != SQL_FETCH_NEXT && !stmt->dbc->ds->safe )
-        return  set_error(stmt,MYERR_S1106,
+        return  stmt->set_error(MYERR_S1106,
                           "Wrong fetchtype with FORWARD ONLY cursor", 0);
     }
 
     if ( if_dynamic_cursor(stmt) && set_dynamic_result(stmt) )
-      return set_error(stmt,MYERR_S1000,
+      return stmt->set_error(MYERR_S1000,
                        "Driver Failed to set the internal dynamic result", 0);
 
     if ( !pcrow )
@@ -2251,108 +2283,18 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
     /* for scrollable cursor("scroller") max_row is max row for currently
        fetched part of resultset */
     max_row= (long) num_rows(stmt);
-    reset_getdata_position(stmt);
+
+    stmt->reset_getdata_position();
     stmt->current_values= 0;          /* For SQLGetData */
 
-    switch ( fFetchType )
+    try
     {
-      case SQL_FETCH_NEXT:
-        cur_row= (stmt->current_row < 0 ? 0 :
-                  stmt->current_row+stmt->rows_found_in_set);
-        break;
-      case SQL_FETCH_PRIOR:
-        cur_row= (stmt->current_row <= 0 ? -1 :
-                  (long)(stmt->current_row - stmt->ard->array_size));
-        break;
-      case SQL_FETCH_FIRST:
-        cur_row= 0L;
-        break;
-      case SQL_FETCH_LAST:
-        cur_row= max_row - stmt->ard->array_size;
-        break;
-      case SQL_FETCH_ABSOLUTE:
-        if (irow < 0)
-        {
-          /* Fetch from end of result set */
-          if ( max_row+irow < 0 && -irow <= (long) stmt->ard->array_size )
-          {
-            /*
-              | FetchOffset | > LastResultRow AND
-              | FetchOffset | <= RowsetSize
-            */
-            cur_row= 0;     /* Return from beginning */
-          }
-          else
-            cur_row= max_row+irow;     /* Ok if max_row <= -irow */
-        }
-        else
-          cur_row= (long) irow-1;
-        break;
-
-      case SQL_FETCH_RELATIVE:
-        cur_row= stmt->current_row + irow;
-        if (stmt->current_row > 0 && cur_row < 0 &&
-           (long)-irow <= (long)stmt->ard->array_size)
-        {
-          cur_row= 0;
-        }
-        break;
-
-      case SQL_FETCH_BOOKMARK:
-        {
-          cur_row= irow;
-          if (cur_row < 0 && (long)-irow <= (long)stmt->ard->array_size)
-          {
-            cur_row= 0;
-          }
-        }
-        break;
-
-      default:
-          return set_error(stmt, MYERR_S1106, "Fetch type out of range", 0);
+      cur_row = stmt->compute_cur_row(fFetchType, irow);
     }
-
-    if ( cur_row < 0 )
+    catch (MYERROR &e)
     {
-      stmt->current_row= -1;  /* Before first row */
-      stmt->rows_found_in_set= 0;
-      data_seek(stmt, 0L);
-      return SQL_NO_DATA_FOUND;
+      return e.retcode;
     }
-    if ( cur_row > max_row )
-    {
-      if (scroller_exists(stmt))
-      {
-        while (cur_row > (max_row= (long)scroller_move(stmt)));
-
-        switch (scroller_prefetch(stmt))
-        {
-          case SQL_NO_DATA: return SQL_NO_DATA_FOUND;
-          case SQL_ERROR:   return set_error(stmt,MYERR_S1000,
-                                            mysql_error(&stmt->dbc->mysql), 0);
-        }
-      }
-      else
-      {
-        cur_row= max_row;
-      }
-    }
-
-    if ( !stmt->result_array && !if_forward_cache(stmt) )
-    {
-        /*
-          If Dynamic, it loses the stmt->end_of_set, so
-          seek to desired row, might have new data or
-          might be deleted
-        */
-        if ( stmt->stmt_options.cursor_type != SQL_CURSOR_DYNAMIC &&
-             cur_row && cur_row == (long)(stmt->current_row +
-                                          stmt->rows_found_in_set) )
-            row_seek(stmt, stmt->end_of_set);
-        else
-            data_seek(stmt, cur_row);
-    }
-    stmt->current_row= cur_row;
 
     if (scroller_exists(stmt)
       || (if_forward_cache(stmt) && !stmt->result_array)
@@ -2385,14 +2327,13 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
       }
     }
 
-    if (!stmt->dbc->ds->dont_use_set_locale)
-    {
-      setlocale(LC_NUMERIC, "C");
-    }
+    C_LOCALE_SET(stmt)
 
     res= SQL_SUCCESS;
     for (i= 0 ; i < rows_to_fetch ; ++i)
     {
+      values = nullptr;
+
       if ( stmt->result_array )
       {
         values= stmt->result_array + cur_row*stmt->result->field_count;
@@ -2442,9 +2383,9 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
           values= stmt->array;
         }
 
-        if ( stmt->fix_fields )
+        if (stmt->fix_fields)
         {
-            values= (*stmt->fix_fields)(stmt,values);
+          values= (*stmt->fix_fields)(stmt,values);
         }
 
         stmt->current_values= values;
@@ -2553,10 +2494,7 @@ SQLRETURN SQL_API my_SQLExtendedFetch( SQLHSTMT             hstmt,
       stmt->end_of_set= row_seek(stmt, save_position);
     }
 
-    if (!stmt->dbc->ds->dont_use_set_locale)
-    {
-      setlocale(LC_NUMERIC,default_locale);
-    }
+    DEFAULT_LOCALE_SET(stmt)
 
     if (SQL_SUCCEEDED(res)
       && stmt->rows_found_in_set < stmt->ard->array_size)
