@@ -1,40 +1,35 @@
 #ifndef SQL_STRING_INCLUDED
 #define SQL_STRING_INCLUDED
 
-// Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Server, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
-/* This file is originally from the mysql distribution. Coded by monty */
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License, version 2.0, for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /**
   @file include/sql_string.h
+  Our own string classes, used pervasively throughout the executor.
+  See in particular the comment on String before you use anything from here.
 */
 
+#include <stdint.h>
 #include <string.h>
 #include <sys/types.h>
 #include <new>
@@ -45,7 +40,6 @@
 #include "m_string.h"  // LEX_CSTRING
 #include "memory_debugging.h"
 #include "my_alloc.h"
-#include "my_byteorder.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -96,6 +90,7 @@ class Simple_cstring {
     set(str_arg, length_arg);
   }
   Simple_cstring(const LEX_STRING arg) { set(arg.str, arg.length); }
+  Simple_cstring(const LEX_CSTRING arg) { set(arg.str, arg.length); }
   void reset() { set(NULL, 0); }
   /**
     Set to a null-terminated string.
@@ -134,6 +129,8 @@ class String;
 struct CHARSET_INFO;
 struct IO_CACHE;
 
+bool validate_string(const CHARSET_INFO *cs, const char *str, size_t length,
+                     size_t *valid_length, bool *length_error);
 int sortcmp(const String *a, const String *b, const CHARSET_INFO *cs);
 String *copy_if_not_alloced(String *to, String *from, size_t from_length);
 inline size_t copy_and_convert(char *to, size_t to_length,
@@ -153,7 +150,8 @@ size_t convert_to_printable(char *to, size_t to_len, const char *from,
                             size_t from_len, const CHARSET_INFO *from_cs,
                             size_t nbytes = 0);
 
-size_t bin_to_hex_str(char *to, size_t to_len, char *from, size_t from_len);
+size_t bin_to_hex_str(char *to, size_t to_len, const char *from,
+                      size_t from_len);
 
 /**
   Using this class is fraught with peril, and you need to be very careful
@@ -178,7 +176,7 @@ class String {
         m_charset(&my_charset_bin),
         m_alloced_length(0),
         m_is_alloced(false) {}
-  String(size_t length_arg)
+  explicit String(size_t length_arg)
       : m_ptr(NULL),
         m_length(0),
         m_charset(&my_charset_bin),
@@ -218,10 +216,10 @@ class String {
         m_is_alloced(str.m_is_alloced) {
     str.m_is_alloced = false;
   }
-  static void *operator new(
-      size_t size, MEM_ROOT *mem_root,
-      const std::nothrow_t &arg MY_ATTRIBUTE((unused)) = std::nothrow) throw() {
-    return alloc_root(mem_root, size);
+  static void *operator new(size_t size, MEM_ROOT *mem_root,
+                            const std::nothrow_t &arg MY_ATTRIBUTE((unused)) =
+                                std::nothrow) noexcept {
+    return mem_root->Alloc(size);
   }
   static void operator delete(void *ptr_arg, size_t size) {
     (void)ptr_arg;
@@ -230,7 +228,7 @@ class String {
   }
 
   static void operator delete(
-      void *, MEM_ROOT *, const std::nothrow_t &)throw() { /* never called */
+      void *, MEM_ROOT *, const std::nothrow_t &)noexcept { /* never called */
   }
 
   ~String() { mem_free(); }
@@ -239,12 +237,14 @@ class String {
   const CHARSET_INFO *charset() const { return m_charset; }
   size_t length() const { return m_length; }
   size_t alloced_length() const { return m_alloced_length; }
-  char &operator[](size_t i) const { return m_ptr[i]; }
+  const char &operator[](size_t i) const { return m_ptr[i]; }
+  char &operator[](size_t i) { return m_ptr[i]; }
   void length(size_t len) { m_length = len; }
   bool is_empty() const { return (m_length == 0); }
   void mark_as_const() { m_alloced_length = 0; }
   /* Returns a pointer to data, may not include NULL terminating character. */
   const char *ptr() const { return m_ptr; }
+  char *ptr() { return m_ptr; }
   char *c_ptr() {
     DBUG_ASSERT(!m_is_alloced || !m_ptr || !m_alloced_length ||
                 (m_alloced_length >= (m_length + 1)));
@@ -267,10 +267,7 @@ class String {
       (void)mem_realloc(m_length);
     return m_ptr;
   }
-  LEX_STRING lex_string() const {
-    LEX_STRING lex_string = {(char *)ptr(), length()};
-    return lex_string;
-  }
+  LEX_STRING lex_string() { return {m_ptr, length()}; }
 
   LEX_CSTRING lex_cstring() const {
     LEX_CSTRING lex_cstring = {ptr(), length()};
@@ -280,7 +277,7 @@ class String {
   void set(String &str, size_t offset, size_t arg_length) {
     DBUG_ASSERT(&str != this);
     mem_free();
-    m_ptr = const_cast<char *>(str.ptr()) + offset;
+    m_ptr = str.ptr() + offset;
     m_length = arg_length;
     m_is_alloced = false;
     if (str.m_alloced_length)
@@ -329,6 +326,16 @@ class String {
   bool set(ulonglong num, const CHARSET_INFO *cs) {
     return set_int((longlong)num, true, cs);
   }
+
+  /**
+    Sets the contents of this string to the string representation of the given
+    double value.
+
+    @param num the double value
+    @param decimals the number of decimals
+    @param cs the character set of the string
+    @return false on success, true on error
+  */
   bool set_real(double num, uint decimals, const CHARSET_INFO *cs);
 
   /*
@@ -467,6 +474,16 @@ class String {
   bool copy(const char *s, size_t arg_length, const CHARSET_INFO *cs);
   static bool needs_conversion(size_t arg_length, const CHARSET_INFO *cs_from,
                                const CHARSET_INFO *cs_to, size_t *offset);
+  bool needs_conversion(const CHARSET_INFO *cs_to) const {
+    size_t offset;
+    return needs_conversion(length(), charset(), cs_to, &offset);
+  }
+  bool is_valid_string(const CHARSET_INFO *cs_to) const {
+    size_t valid_length;
+    bool length_error;
+    return !validate_string(cs_to, ptr(), length(), &valid_length,
+                            &length_error);
+  }
   static bool needs_conversion_on_storage(size_t arg_length,
                                           const CHARSET_INFO *cs_from,
                                           const CHARSET_INFO *cs_to);
@@ -484,10 +501,9 @@ class String {
   bool append(const char *s, size_t arg_length, const CHARSET_INFO *cs);
   bool append_ulonglong(ulonglong val);
   bool append_longlong(longlong val);
-  bool append(IO_CACHE *file, size_t arg_length);
   bool append_with_prefill(const char *s, size_t arg_length, size_t full_length,
                            char fill_char);
-  bool append_parenthesized(long nr, int radix = 10);
+  bool append_parenthesized(int64_t nr);
   /**
     Search for a substring.
 
@@ -521,13 +537,12 @@ class String {
     if (m_length < m_alloced_length) {
       m_ptr[m_length++] = chr;
     } else {
-      if (mem_realloc_exp(m_length + 1)) return 1;
+      if (mem_realloc_exp(m_length + 1)) return true;
       m_ptr[m_length++] = chr;
     }
-    return 0;
+    return false;
   }
   bool fill(size_t max_length, char fill);
-  void strip_sp();
   friend int sortcmp(const String *a, const String *b, const CHARSET_INFO *cs);
   friend int stringcmp(const String *a, const String *b);
   friend String *copy_if_not_alloced(String *to, String *from,
@@ -535,45 +550,10 @@ class String {
   size_t numchars() const;
   size_t charpos(size_t i, size_t offset = 0) const;
 
-  int reserve(size_t space_needed) {
+  bool reserve(size_t space_needed) {
     return mem_realloc(m_length + space_needed);
   }
-  int reserve(size_t space_needed, size_t grow_by);
-  /*
-    The following append operations do NOT check alloced memory
-    q_*** methods writes values of parameters itself
-    qs_*** methods writes string representation of value
-  */
-  void q_append(const char c) { m_ptr[m_length++] = c; }
-  void q_append(const uint32 n) {
-    int4store(m_ptr + m_length, n);
-    m_length += 4;
-  }
-  void q_append(double d) {
-    float8store(m_ptr + m_length, d);
-    m_length += 8;
-  }
-  void q_append(double *d) {
-    float8store(m_ptr + m_length, *d);
-    m_length += 8;
-  }
-  void q_append(const char *data, size_t data_len) {
-    memcpy(m_ptr + m_length, data, data_len);
-    m_length += data_len;
-  }
-
-  void write_at_position(int position, uint32 value) {
-    int4store(m_ptr + position, value);
-  }
-
-  void qs_append(const char *str, size_t len);
-  void qs_append(double d, size_t len);
-  void qs_append(const char c) {
-    m_ptr[m_length] = c;
-    m_length++;
-  }
-  void qs_append(int i);
-  void qs_append(uint i);
+  bool reserve(size_t space_needed, size_t grow_by);
 
   /* Inline (general) functions used by the protocol functions */
 
@@ -596,7 +576,7 @@ class String {
     m_length += arg_length;
     return false;
   }
-  void print(String *print);
+  void print(String *print) const;
 
   /* Swap two string objects. Efficient way to exchange data without memcpy. */
   void swap(String &s) noexcept;
@@ -675,9 +655,6 @@ inline LEX_CSTRING to_lex_cstring(const char *s) {
   return cstr;
 }
 
-bool validate_string(const CHARSET_INFO *cs, const char *str, uint32 length,
-                     size_t *valid_length, bool *length_error);
-
 bool append_escaped(String *to_str, const String *from_str);
 
 #endif /* SQL_STRING_INCLUDED */
