@@ -1,30 +1,30 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /*!
   @brief  This program will aid installers when installing/uninstalling
@@ -68,7 +68,7 @@ const char usage[] =
 "|    various Unix-like systems.                                        \n"
 "|                                                                      \n"
 #ifndef _WIN32
-#ifdef USE_IODBC
+#ifndef USE_IODBC
 "|    The program requires that unixODBC version 2.2.14 or newer        \n"
 "|    has already been installed on the system.                         \n"
 "|    UnixODBC can be downloaded from here:                             \n"
@@ -119,27 +119,27 @@ const char usage[] =
 #ifndef _WIN32
 "|    Register a Unicode driver (UNIX example)                          \n"
 "|    shell> myodbc-installer -d -a -n \"MySQL ODBC " MYODBC_STRSERIES " Unicode Driver\" \\ \n"
-"|              -t \"DRIVER=/path/to/driver/libmyodbc5w.so;SETUP=/path/to/gui/myodbc5S.so\"\n"
+"|              -t \"DRIVER=/path/to/driver/libmyodbc8w.so;SETUP=/path/to/gui/myodbc8S.so\"\n"
 "|                                                                      \n"
 "|      Note                                                            \n"
 "|         * The /path/to/driver is /usr/lib for 32-bit systems and     \n"
 "|           some 64-bit systems, and /usr/lib64 for most 64-bit systems\n"
 "|                                                                      \n"
-"|         * driver_name is libmyodbc5a.so for the ANSI version and     \n"
-"|           libmyodbc5w.so for the Unicode version of MySQL ODBC Driver\n"
+"|         * driver_name is libmyodbc8a.so for the ANSI version and     \n"
+"|           libmyodbc8w.so for the Unicode version of MySQL ODBC Driver\n"
 "|                                                                      \n"
 "|         * The SETUP parameter is optional; it provides location of   \n"
-"|           the GUI module (libmyodbc5S.so) for DSN setup, which       \n"
+"|           the GUI module (libmyodbc8S.so) for DSN setup, which       \n"
 "|           is not supported on Solaris and Mac OSX systems            \n"
 "|                                                                      \n"
 #else
 "|    Register a Unicode driver (Windows example)                       \n"
 "|    shell> myodbc-installer -d -a -n \"MySQL ODBC " MYODBC_STRSERIES " Unicode Driver\" \\ \n"
-"|              -t \"DRIVER=myodbc5w.dll;SETUP=myodbc5S.dll\"\n"
+"|              -t \"DRIVER=myodbc8w.dll;SETUP=myodbc8S.dll\"\n"
 "|                                                                      \n"
 "|      Note                                                            \n"
-"|         * driver_name is myodbc5a.dll for the ANSI version and       \n"
-"|           myodbc5w.dll for the Unicode version of MySQL ODBC Driver  \n"
+"|         * driver_name is myodbc8a.dll for the ANSI version and       \n"
+"|           myodbc8w.dll for the Unicode version of MySQL ODBC Driver  \n"
 "|                                                                      \n"
 #endif
 "|    Add a new system data source name for Unicode driver              \n"
@@ -506,7 +506,7 @@ int list_datasource_details(DataSource *ds)
   if (ds->sslmode   ) printf("SSL Mode:             %s\n", ds_get_utf8attr(ds->sslmode, &ds->sslmode8));
   if (ds->sslverify) printf("Verify SSL cert      yes\n");
   if (ds->rsakey)    printf("RSA public key       %s\n", ds_get_utf8attr(ds->rsakey, &ds->rsakey8));
-  if (ds->port       ) printf("Port:                %d\n", ds->port);
+  if (ds->port && ds->has_port) printf("Port:                %d\n", ds->port);
   if (ds->plugin_dir  ) printf("Plugin directory:    %s\n", ds_get_utf8attr(ds->plugin_dir, &ds->plugin_dir8));
   if (ds->default_auth) printf("Default Authentication Library: %s\n", ds_get_utf8attr(ds->default_auth, &ds->default_auth8));
   printf("Options:\n");
@@ -514,7 +514,6 @@ int list_datasource_details(DataSource *ds)
   if (ds->allow_big_results) printf("\tBIG_PACKETS\n");
   if (ds->dont_prompt_upon_connect) printf("\tNO_PROMPT\n");
   if (ds->dynamic_cursor) printf("\tDYNAMIC_CURSOR\n");
-  if (ds->ignore_N_in_name_table) printf("\tNO_SCHEMA\n");
   if (ds->user_manager_cursor) printf("\tNO_DEFAULT_CURSOR\n");
   if (ds->dont_use_set_locale) printf("\tNO_LOCALE\n");
   if (ds->pad_char_to_full_length) printf("\tPAD_SPACE\n");
@@ -552,6 +551,8 @@ int list_datasource_details(DataSource *ds)
   if (ds->can_handle_exp_pwd) printf("\tCAN_HANDLE_EXP_PWD\n");
   if (ds->enable_cleartext_plugin) printf("\tENABLE_CLEARTEXT_PLUGIN\n");
   if (ds->get_server_public_key) printf("\tGET_SERVER_PUBLIC_KEY\n");
+  if (ds->enable_dns_srv) printf("\tENABLE_DNS_SRV\n");
+  if (ds->multi_host) printf("\tMULTI_HOST\n");
 
   return 0;
 }
