@@ -56,7 +56,7 @@
 Summary:	An ODBC @CONNECTOR_BASE_VERSION@ driver for MySQL - driver package
 Name:		mysql-connector-odbc%{?product_suffix}
 Version:	@CONNECTOR_NODASH_VERSION@
-Release:	1%{?dist}
+Release:	1%{?commercial:.1}%{?dist}
 License:	Copyright (c) 2000, @CURRENT_YEAR@, %{mysql_vendor}. All rights reserved.  Under %{license_type} license as shown in the Description field.
 Source0:	http://cdn.mysql.com/Downloads/Connector-ODBC/@CONNECTOR_BASE_VERSION@/%{name}-@CONNECTOR_VERSION@-src.tar.gz
 URL:		http://www.mysql.com/
@@ -65,6 +65,11 @@ Vendor:		%{mysql_vendor}
 Packager:	%{mysql_vendor} Product Engineering Team <mysql-build@oss.oracle.com>
 BuildRequires:	cmake 
 %{?shared_mysqlclient:BuildRequires: mysql-community-devel}
+%if 0%{?commercial}
+Requires: mysql-commercial-client-plugins = %{version}
+%else
+Requires: mysql-community-client-plugins = %{version}
+%endif
 BuildRequires:	unixODBC-devel
 BuildRoot:	%(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)
 
@@ -96,10 +101,12 @@ mysql-connector-odbc @CONNECTOR_BASE_VERSION@ is an enhanced version of mysql-co
 The driver comes in 2 flavours - ANSI and Unicode and commonly referred to as
 'MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver' or 'MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver' respectively.
 
-License information can be found in the %{license_files} file. 
-This distribution may include materials developed by third parties.
-For license and attribution notices for these materials, please refer
-to the %{license_files} file.
+The MySQL software has Dual Licensing, which means you can use the
+MySQL software free of charge under the GNU General Public License
+(http://www.gnu.org/licenses/). You can also purchase commercial MySQL
+licenses from %{mysql_vendor} if you do not wish to be bound by the
+terms of the GPL. See the chapter "Licensing and Support" in the
+manual for further info.
 
 The MySQL web site (http://www.mysql.com/) provides the latest news
 and information about the MySQL software. Also please see the
@@ -170,7 +177,7 @@ rm -rf %{buildroot}
 %install
 pushd release
 make DESTDIR=%{buildroot} install VERBOSE=1
-rm -vf  %{buildroot}%{_prefix}/{ChangeLog,README*,LICENSE*.*}
+rm -vf  %{buildroot}%{_prefix}/{ChangeLog,README*,LICENSE*.*,INFO_SRC,INFO_BIN}
 mkdir -p %{buildroot}%{_libdir}/mysql-connector-odbc
 mv %{buildroot}%{_prefix}/test %{buildroot}%{_libdir}/mysql-connector-odbc/
 mv bin/dltest                  %{buildroot}%{_libdir}/mysql-connector-odbc/
@@ -184,16 +191,16 @@ popd
 
 %post 
 if [ -x /usr/bin/myodbc-installer ]; then
-    /usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc5w.so"
-    /usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc5a.so"
+    /usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc8w.so"
+    /usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc8a.so"
 fi
 
 %if 0%{?odbc_gui}
 %post setup
 /usr/bin/myodbc-installer -r -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver"
 /usr/bin/myodbc-installer -r -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"
-/usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc5w.so;SETUP=%{_libdir}/libmyodbc5S.so"
-/usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc5a.so;SETUP=%{_libdir}/libmyodbc5S.so"
+/usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc8w.so;SETUP=%{_libdir}/libmyodbc8S.so"
+/usr/bin/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc8a.so;SETUP=%{_libdir}/libmyodbc8S.so"
 %endif
 
 # ----------------------------------------------------------------------
@@ -212,8 +219,8 @@ if [ "$1" = 0 ]; then
     if [ -x %{_bindir}/myodbc-installer ]; then
         %{_bindir}/myodbc-installer -r -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" > /dev/null 2>&1 || :
         %{_bindir}/myodbc-installer -r -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    > /dev/null 2>&1 || :
-        %{_bindir}/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc5w.so" > /dev/null 2>&1 || :
-        %{_bindir}/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc5a.so" > /dev/null 2>&1 || :
+        %{_bindir}/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ Unicode Driver" -t "DRIVER=%{_libdir}/libmyodbc8w.so" > /dev/null 2>&1 || :
+        %{_bindir}/myodbc-installer -a -d -n "MySQL ODBC @CONNECTOR_BASE_VERSION@ ANSI Driver"    -t "DRIVER=%{_libdir}/libmyodbc8a.so" > /dev/null 2>&1 || :
     fi
 fi
 %endif
@@ -227,21 +234,23 @@ fi
 %files
 %defattr(-, root, root, -)
 %{_bindir}/myodbc-installer
-%{_libdir}/libmyodbc5w.so
-%{_libdir}/libmyodbc5a.so
+%{_libdir}/libmyodbc8w.so
+%{_libdir}/libmyodbc8a.so
 %doc %{license_files}
-%doc ChangeLog
+%doc ChangeLog INFO_SRC INFO_BIN
 
 %if 0%{?odbc_gui}
 %files setup
 %defattr(-, root, root, -)
 %doc %{license_files}
-%{_libdir}/libmyodbc5S.so
+%doc INFO_SRC INFO_BIN
+%{_libdir}/libmyodbc8S.so
 %endif
 
 %files test
 %defattr(-, root, root, -)
 %doc %{license_files}
+%doc INFO_SRC INFO_BIN
 %attr(-, root, root)   %{_libdir}/mysql-connector-odbc/test
 %attr(755, root, root) %{_libdir}/mysql-connector-odbc/dltest
 
