@@ -1,30 +1,30 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Connector/ODBC, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of MySQL Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
  @file  odbcdialogparams.cpp
@@ -388,7 +388,7 @@ void btnDetails_Click (HWND hwnd)
 					          &FormMain_DlgProc,          // address of main windows proc
 					          NULL,                       // address of size function
 					          TRUE);                      // stretch tab page to fit tab ctrl
-		flag = true;		
+		flag = true;
 
 
     HWND ssl_tab = TabCtrl_1.hTabPages[4];
@@ -636,6 +636,8 @@ void FormMain_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
       chooseFile(hwnd, IDC_EDIT_rsakey); break;
     case IDC_CHOOSER_plugin_dir:
       choosePath(hwnd, IDC_EDIT_plugin_dir); break;
+    case IDC_CHOOSER_load_data_local_dir:
+      choosePath(hwnd, IDC_EDIT_load_data_local_dir); break;
     case IDC_RADIO_tcp:
     case IDC_RADIO_pipe:
       SwitchTcpOrPipe(hwnd, !!Button_GetCheck(GetDlgItem(hwnd, IDC_RADIO_pipe)));
@@ -649,7 +651,7 @@ void FormMain_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
 
         EnableWindow(prefetch, !!Button_GetCheck(GetDlgItem(cursorTab,
                                             IDC_CHECK_cursor_prefetch_active)));
-                  
+
         if (Edit_GetTextLength(prefetch) == 0)
         {
           setUnsignedFieldData(cursorTab, default_cursor_prefetch,
@@ -700,27 +702,27 @@ void FormMain_OnScroll(HWND hwnd, HWND hCtrl, UINT code, int pos)
   int yNewPos;    // new position
   switch (code)
   {
-      // User clicked the scroll bar shaft above the scroll box. 
+      // User clicked the scroll bar shaft above the scroll box.
     case SB_PAGEUP:
       yNewPos = yCurrentScroll - 50;
       break;
 
-      // User clicked the scroll bar shaft below the scroll box. 
+      // User clicked the scroll bar shaft below the scroll box.
     case SB_PAGEDOWN:
       yNewPos = yCurrentScroll + 50;
       break;
 
-      // User clicked the top arrow. 
+      // User clicked the top arrow.
     case SB_LINEUP:
       yNewPos = yCurrentScroll - 5;
       break;
 
-      // User clicked the bottom arrow. 
+      // User clicked the bottom arrow.
     case SB_LINEDOWN:
       yNewPos = yCurrentScroll + 5;
       break;
 
-      // User dragged the scroll box. 
+      // User dragged the scroll box.
     case SB_THUMBPOSITION:
       yNewPos = pos;
       break;
@@ -728,7 +730,7 @@ void FormMain_OnScroll(HWND hwnd, HWND hCtrl, UINT code, int pos)
     default:
       yNewPos = yCurrentScroll;
   }
-  
+
   si.nPos = yNewPos;
 
   ScrollWindowEx(hwnd, 0, -50,
