@@ -1,32 +1,31 @@
 /////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2006, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Connector/ODBC, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
-
+// Copyright (c) 2006, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of MySQL Connector/ODBC, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 //{{NO_DEPENDENCIES}}
 // Microsoft Visual C++ generated include file.
 // Used by odbcdialogparams.rc
@@ -94,7 +93,6 @@
 #define IDC_CHECK_auto_increment_null_search 10014
 #define IDC_CHECK_dont_prompt_upon_connect 10015
 #define IDC_CHECK_dynamic_cursor        10016
-#define IDC_CHECK_ignore_N_in_name_table 10017
 #define IDC_CHECK_user_manager_cursor   10018
 #define IDC_CHECK_dont_use_set_locale   10019
 #define IDC_CHECK_pad_char_to_full_length 10020
@@ -133,14 +131,17 @@
 #define IDC_EDIT_plugin_dir             10053
 #define IDC_EDIT_default_auth           10054
 #define IDC_CHOOSER_plugin_dir          10055
-#define IDC_CHECK_disable_ssl_default   10056
 #define IDC_CHECK_no_date_overflow      10057
 #define IDC_CHECK_no_tls_1              10058
 #define IDC_CHECK_no_tls_1_1            10059
 #define IDC_CHECK_no_tls_1_2            10060
-#define IDC_EDIT_sslmode               10061
+#define IDC_EDIT_sslmode                10061
 #define IDC_CHECK_get_server_public_key 10062
 #define IDC_CHECK_enable_local_infile   10063
+#define IDC_CHECK_enable_dns_srv        10064
+#define IDC_CHECK_multi_host            10065
+#define IDC_EDIT_load_data_local_dir    10066
+#define IDC_CHOOSER_load_data_local_dir 10067
 #define IDC_BUTTON_TEST                 11014
 #define IDC_BUTTON_HELP                 11015
 #define IDC_STATIC                      -1
