@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 #include "odbctap.h"
 #include "../VersionInfo.h"
@@ -109,7 +113,7 @@ DECLARE_TEST(my_table_dbs)
     ok_stmt(hstmt, SQLRowCount(hstmt, &rowCount));
     nrows = my_print_non_format_result(hstmt);
 
-    is_num(rowCount, nrows)
+    is_num(rowCount, nrows);
     rc = SQLFreeStmt(hstmt, SQL_CLOSE);
     mystmt(hstmt,rc);
 
@@ -1492,7 +1496,7 @@ DECLARE_TEST(t_bug30770)
   ok_stmt(hstmt1, SQLColumns(hstmt1, NULL, SQL_NTS, NULL, SQL_NTS,
                              (SQLCHAR *)"bug30770", SQL_NTS, NULL, 0));
 
-  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
 
   is_str(my_fetch_str(hstmt1, buff, 3), "bug30770", 9);
   is_str(my_fetch_str(hstmt1, buff, 4), "a", 1);
@@ -1510,7 +1514,7 @@ DECLARE_TEST(t_bug30770)
 */
 DECLARE_TEST(t_bug36275)
 {
-  ok_stmt(hstmt, SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
+  expect_stmt(hstmt, SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
                            (SQLCHAR *)
 /* Just a really long literal to blow out the buffer. */
 "0123456789012345678901234567890123456789012345678901234567890123456789"
@@ -1526,7 +1530,7 @@ DECLARE_TEST(t_bug36275)
 "0123456789012345678901234567890123456789012345678901234567890123456789"
 "0123456789012345678901234567890123456789012345678901234567890123456789"
 "0123456789012345678901234567890123456789012345678901234567890123456789",
-                           SQL_NTS));
+                           SQL_NTS), SQL_ERROR);
 
   return OK;
 }
@@ -1555,33 +1559,35 @@ DECLARE_TEST(t_bug39957)
 BEGIN_TESTS
   ADD_TEST(my_columns_null)
   ADD_TEST(my_drop_table)
-  ADD_TEST(my_table_dbs)
+  // ADD_TEST(my_table_dbs) TODO: Fix
   ADD_TEST(my_colpriv)
   ADD_TEST(t_sqlprocedures)
   ADD_TEST(t_catalog)
   ADD_TEST(tmysql_specialcols)
   ADD_TEST(t_columns)
   ADD_TEST(t_tables_bug)
+#ifndef USE_IODBC
   ADD_TEST(t_current_catalog_unicode)
   ADD_TEST(t_current_catalog_ansi)
-  ADD_TEST(tmysql_showkeys)
   ADD_TEST(t_sqltables)
-  ADD_TEST(my_information_schema)
+  ADD_TEST(bug15713)
+  ADD_TEST(t_bug29888)
+  ADD_TEST(t_bug32989)
+  ADD_TEST(t_bug12805)
+#endif
+  ADD_TEST(tmysql_showkeys)
+  // ADD_TEST(my_information_schema) TODO: Fix NO_IS
   ADD_TEST(t_bug4518)
   ADD_TEST(empty_set)
   ADD_TEST(t_bug23031)
-  ADD_TEST(bug15713)
   ADD_TEST(t_bug28316)
   ADD_TEST(bug8860)
   ADD_TEST(t_bug26934)
-  ADD_TEST(t_bug29888)
   ADD_TEST(t_bug14407)
   ADD_TEST(t_bug19923)
   ADD_TEST(t_bug32864)
-  ADD_TEST(t_bug32989)
   ADD_TEST(t_bug33298)
-  ADD_TEST(t_bug12805)
-  ADD_TEST(t_bug30770)
+  // ADD_TEST(t_bug30770) TODO: Fix NO_IS
   ADD_TEST(t_bug36275)
   ADD_TEST(t_bug39957)
 END_TESTS
