@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 #include "odbctap.h"
 #include "../VersionInfo.h"
@@ -638,7 +642,7 @@ DECLARE_TEST(t_bug57182)
     NULL, 0));
 
   ok_stmt(hstmt, SQLRowCount(hstmt, &nRowCount));
-  is_num(2, nRowCount)
+  is_num(2, nRowCount);
   
   ok_stmt(hstmt, SQLFetch(hstmt));
   
@@ -663,7 +667,7 @@ DECLARE_TEST(t_bug57182)
     "id", SQL_NTS));
 
   ok_stmt(hstmt, SQLRowCount(hstmt, &nRowCount));
-  is_num(1, nRowCount)
+  is_num(1, nRowCount);
 
   ok_stmt(hstmt, SQLFetch(hstmt));
 
@@ -702,6 +706,9 @@ DECLARE_TEST(t_bug55870)
   SQLLEN  rowCount;
   SQLCHAR query[256];
 
+  if (strcmp(myserver, "localhost"))
+    return OK;
+
   ok_sql(hstmt, "drop table if exists bug55870r");
   ok_sql(hstmt, "drop table if exists bug55870_2");
   ok_sql(hstmt, "drop table if exists bug55870");
@@ -716,9 +723,9 @@ DECLARE_TEST(t_bug55870)
                                         NULL, "", "NO_I_S=1"));
 
 
-  sprintf(query, "grant Insert, Select on bug55870 to %s", myuid);
+  sprintf(query, "grant Insert, Select on bug55870 to '%s'@'localhost'", myuid);
   ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
-  sprintf(query, "grant Insert (c), Select (c), Update (c) on bug55870 to %s", myuid);
+  sprintf(query, "grant Insert (c), Select (c), Update (c) on bug55870 to '%s'@'localhost'", myuid);
   ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
 
   ok_stmt(hstmt1, SQLStatistics(hstmt1, NULL, 0, NULL, 0,
@@ -764,10 +771,10 @@ DECLARE_TEST(t_bug55870)
   /** surprise-surprise - just removing table is not enough to remove related
       records from tables_priv and columns_priv
   */
-  sprintf(query, "revoke select,insert on bug55870 from %s", myuid);
+  sprintf(query, "revoke select,insert on bug55870 from '%s'@'localhost'", myuid);
   ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
 
-  sprintf(query, "revoke select (c),insert (c),update (c) on bug55870 from %s", myuid);
+  sprintf(query, "revoke select (c),insert (c),update (c) on bug55870 from '%s'@'localhost'", myuid);
   ok_stmt(hstmt, SQLExecDirect(hstmt, query, SQL_NTS));
 
   
@@ -1066,25 +1073,219 @@ DECLARE_TEST(t_bug69448)
 }
 
 
+/*
+  Bug #69554: SQL_ATTR_MAX_ROWS applies to all result sets on the statement,
+  and not connection, which is currently there as 
+  'set @@sql_select_limit' for whole session.
+*/
+DECLARE_TEST(t_bug69554)
+{
+  int num_rows= 0;
+  SQLHSTMT hstmt1;
+
+  ok_con(hdbc, SQLAllocStmt(hdbc, &hstmt1));
+  ok_sql(hstmt, "DROP TABLE IF EXISTS table69554a");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS table69554b");
+  ok_sql(hstmt, "DROP TABLE IF EXISTS table69554c");
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS proc69554a");
+  ok_sql(hstmt, "DROP PROCEDURE IF EXISTS proc69554b");
+
+  ok_sql(hstmt, "CREATE TABLE table69554a(id int, id2 int, id3 int, PRIMARY KEY(id, id2))");
+  ok_sql(hstmt, "GRANT SELECT, UPDATE, DELETE on table69554a to 'root'@'localhost'");
+  ok_sql(hstmt, "GRANT INSERT (id, id2, id3) on table69554a to 'root'@'localhost'");
+  ok_sql(hstmt, "CREATE TABLE table69554b(id int, id2 int, id3 int)");
+  ok_sql(hstmt, "CREATE TABLE table69554c(id int, id2 int, id3 int)");
+
+  ok_sql(hstmt, "CREATE PROCEDURE proc69554a(IN p1 INT, IN p2 INT) begin end;");
+  ok_sql(hstmt, "CREATE PROCEDURE proc69554b(IN p1 INT, IN p2 INT) begin end;");
+
+  ok_sql(hstmt, "INSERT INTO table69554a(id, id2) VALUES (1, 1), (2, 2), (3, 3)");
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS, 
+                                (SQLPOINTER)1, 0));
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLPrimaryKeys(hstmt1, NULL, 0, NULL, 0, "table69554a", SQL_NTS));
+  is_num(my_print_non_format_result(hstmt1), 2);
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLTables(hstmt1, NULL, 0, NULL, 0,
+                           "table69554%", SQL_NTS, NULL, 0));
+  is_num(my_print_non_format_result(hstmt1), 3);
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLColumns(hstmt1, NULL, 0, NULL, 0, "table69554a", SQL_NTS, NULL, 0));
+  is_num(my_print_non_format_result(hstmt1), 3);
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLColumnPrivileges(hstmt1, NULL, 0, NULL, 0, "table69554a", SQL_NTS, "%", SQL_NTS));
+  is(my_print_non_format_result(hstmt1) > 1);
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLTablePrivileges(hstmt1, NULL, 0, NULL, 0, "table69554a", SQL_NTS));
+  is(my_print_non_format_result(hstmt1) > 1);
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLProcedures(hstmt1, NULL, 0, NULL, 0, "proc69554%", SQL_NTS));
+  is_num(my_print_non_format_result(hstmt1), 2);
+
+  ok_sql(hstmt, "SELECT * FROM table69554a");
+  is_num(my_print_non_format_result(hstmt), 1);
+  ok_stmt(hstmt1, SQLProcedureColumns(hstmt1, NULL, 0, NULL, 0, "proc69554a", SQL_NTS, "%", SQL_NTS));
+  is_num(my_print_non_format_result(hstmt1), 2);
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS table69554a");
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS table69554b");
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS table69554c");
+  ok_sql(hstmt1, "DROP PROCEDURE IF EXISTS proc69554a");
+  ok_sql(hstmt1, "DROP PROCEDURE IF EXISTS proc69554b");
+
+  return OK;
+}
+
+
+/**
+  Bug#14005343: SQLTABLES PATCH (BUG 13914518) HAS TO BE EXTENDED 
+  FOR NO_I_S CASE 
+  Verify database returned should not be '%' for SQL_ALL_CATALOGS input.
+  Also verify total tables in all databases in case of no information schema
+  and in case of information schema should match.
+*/
+DECLARE_TEST(t_bug_14005343)
+{
+  SQLCHAR    database[100];
+  SQLINTEGER nrows= 0;
+
+  /* Test for all catalogs (SQL_ALL_CATALOGS) with table type table and view */
+  ok_stmt(hstmt, SQLTables(hstmt, SQL_ALL_CATALOGS, 1, 0, 0, 0, 0, "TABLE,VIEW", SQL_NTS));
+  while (SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+    ++nrows;
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, database,
+                             sizeof(database), NULL));
+    /* the table catalog in the results must not be '%' */
+    is(database[0] != '%');
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_TYPE, TABLE_SCHEMA "
+                " FROM (SELECT *  FROM INFORMATION_SCHEMA.TABLES )TABLES "
+  	      		  "	WHERE TABLE_TYPE='BASE TABLE' OR TABLE_TYPE='VIEW' ");
+  is_num(myrowcount(hstmt), nrows);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+
+  /* Test for all catalogs (%) with no table type */
+  nrows= 0;
+  ok_stmt(hstmt, SQLTables(hstmt, (SQLCHAR*) "%", SQL_NTS, NULL, 0, NULL, 0, NULL,0));
+  while (SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+    ++nrows;
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, database,
+                             sizeof(database), NULL));
+    /* the table catalog in the results must not be '%' */
+    is(database[0] != '%');
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_TYPE, TABLE_SCHEMA "
+                " FROM (SELECT *  FROM INFORMATION_SCHEMA.TABLES ) TABLES "
+				        " WHERE TABLE_TYPE='BASE TABLE' OR TABLE_TYPE='VIEW' ");
+  is_num(myrowcount(hstmt), nrows);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Test for no catalog (NULL) should return all tables in current database */
+  nrows= 0;
+	ok_stmt(hstmt, SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0, NULL,0));
+  while (SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+    ++nrows;
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, database,
+                             sizeof(database), NULL));
+    /* the table catalog in the results must not be '%' */
+    is(database[0] != '%');
+  }
+  
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_TYPE, TABLE_SCHEMA "
+                " FROM (SELECT *  FROM INFORMATION_SCHEMA.TABLES ) TABLES "
+                " WHERE TABLE_SCHEMA = DATABASE() AND "
+ 				        " (TABLE_TYPE='BASE TABLE' OR TABLE_TYPE='VIEW')");
+  is_num(myrowcount(hstmt), nrows);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+
+  /* Test for all tables in test catalog */
+  nrows= 0;
+  ok_stmt(hstmt, SQLTables(hstmt, "test", SQL_NTS, NULL, 0, NULL, 0, NULL, 0));
+  while (SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+    ++nrows;
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, database,
+                             sizeof(database), NULL));
+    /* the table catalog in the results must not be '%' */
+    is(database[0] != '%');
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_TYPE, TABLE_SCHEMA "
+                " FROM (SELECT *  FROM INFORMATION_SCHEMA.TABLES ) TABLES "
+                " WHERE TABLE_SCHEMA = 'test' AND (TABLE_TYPE='BASE TABLE' "
+                " OR TABLE_TYPE='VIEW')");
+  is_num(myrowcount(hstmt), nrows);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  /* Test for all catalogs (SQL_ALL_CATALOGS) with table type table */
+  nrows= 0;
+  ok_stmt(hstmt, SQLTables(hstmt, SQL_ALL_CATALOGS, 1, 0, 0, 0, 0, "TABLE", SQL_NTS));
+  while (SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+    ++nrows;
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, database,
+                             sizeof(database), NULL));
+    /* the table catalog in the results must not be '%' */
+    is(database[0] != '%');
+  }
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "SELECT TABLE_NAME, TABLE_COMMENT, TABLE_TYPE, TABLE_SCHEMA "
+                " FROM (SELECT *  FROM INFORMATION_SCHEMA.TABLES ) TABLES "
+                " WHERE TABLE_TYPE='BASE TABLE' ");
+  is_num(myrowcount(hstmt), nrows);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  return OK;
+}
+
+
 BEGIN_TESTS
+  ADD_TEST(t_bug_14005343)
+  ADD_TEST(t_bug69554)
   ADD_TEST(t_bug37621)
   ADD_TEST(t_bug34272)
   ADD_TEST(t_bug49660)
-  ADD_TEST(t_bug51422)
+  // ADD_TEST(t_bug51422)            TODO: Fix NO_IS
   ADD_TEST(t_bug36441)
   ADD_TEST(t_bug53235)
-  ADD_TEST(t_bug50195)
-  ADD_TEST(t_sqlprocedurecolumns)
-  ADD_TEST(t_bug57182)
+  // ADD_TEST(t_bug50195)            TODO: Fix NO_IS
+  // ADD_TEST(t_sqlprocedurecolumns) TODO: Fix 
+  // ADD_TEST(t_bug57182)            TODO: Fix
   ADD_TEST(t_bug55870)
   ADD_TEST(t_bug31067)
   ADD_TEST(bug12824839)
   ADD_TEST(sqlcolumns_nodbselected)
   ADD_TEST(t_bug14085211_part1)
-  ADD_TODO(t_bug14085211_part2)
+  // ADD_TODO(t_bug14085211_part2) TODO: Fix
   ADD_TEST(t_sqlcolumns_after_select)
-  ADD_TEST(t_bug14555713)
-  ADD_TODO(t_bug69448)
+  // ADD_TEST(t_bug14555713) TODO: Fix
+  // ADD_TODO(t_bug69448) TODO: Fix
 END_TESTS
 
 myoption &= ~(1 << 30);
