@@ -1,26 +1,31 @@
-/*
-  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
 
 #include <time.h>
 #include "odbctap.h"
@@ -82,11 +87,11 @@ DECLARE_TEST(my_ts)
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 1));
 
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szTs, sizeof(szTs), &len));
-  
-  is_str(szTs, is_fraction_capable ? "2002-01-07 10:20:49.060000" : 
+
+  is_str(szTs, is_fraction_capable ? "2002-01-07 10:20:49.060000" :
                                      "2002-01-07 10:20:49", len);
 
-  printf("# row1 using SQL_C_CHAR: %s (%ld)\n", szTs, len);
+  printf("# row1 using SQL_C_CHAR: %s (%ld)\n", szTs, (long)len);
 
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 1));
 
@@ -101,14 +106,14 @@ DECLARE_TEST(my_ts)
 
   printf("# row1 using SQL_C_TIMESTAMP: %d-%d-%d %d:%d:%d.%d (%ld)\n",
          ts.year, ts.month,ts.day, ts.hour, ts.minute, ts.second, ts.fraction,
-         len);
+         (long)len);
 
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 2));
 
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szTs, sizeof(szTs), &len));
   is_str(szTs, is_fraction_capable ? "2002-01-07 19:47:59.123456" :
                                       "2002-01-07 19:47:59", len);
-  printf("# row2 using SQL_C_CHAR: %s(%ld)\n", szTs, len);
+  printf("# row2 using SQL_C_CHAR: %s(%ld)\n", szTs, (long)len);
 
   ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 2));
 
@@ -123,7 +128,7 @@ DECLARE_TEST(my_ts)
 
   printf("# row2 using SQL_C_TIMESTAMP: %d-%d-%d %d:%d:%d.%d (%ld)\n",
          ts.year, ts.month,ts.day, ts.hour, ts.minute, ts.second, ts.fraction,
-         len);
+         (long)len);
 
 
   expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_ABSOLUTE, 3),
@@ -135,7 +140,7 @@ DECLARE_TEST(my_ts)
   ok_sql(hstmt, "DROP TABLE IF EXISTS my_ts");
 
   /* Test of 2-digits year(YYMMDD) format */
-  ok_sql(hstmt, "select \"910825\""); 
+  ok_sql(hstmt, "select \"910825\"");
   ok_stmt(hstmt, SQLFetch(hstmt));
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TIMESTAMP, &ts, sizeof(ts), &len));
 
@@ -149,7 +154,6 @@ DECLARE_TEST(my_ts)
 
 DECLARE_TEST(t_tstotime)
 {
-  SQLRETURN rc;
   SQL_TIMESTAMP_STRUCT ts, ts1, ts2;
   int is_fraction_capable = mysql_min_version(hdbc, "5.6.", 4);
 
@@ -159,20 +163,20 @@ DECLARE_TEST(t_tstotime)
   ts.hour   = 18;
   ts.minute = 20;
   ts.second = 45;
-  ts.fraction = is_fraction_capable ? 555000 : 0;   
+  ts.fraction = is_fraction_capable ? 555000 : 0;
 
   memcpy(&ts1, (void*) &ts, sizeof(SQL_TIMESTAMP_STRUCT));
 
   memcpy(&ts2, (void*) &ts1, sizeof(SQL_TIMESTAMP_STRUCT));
 
-  /* 
+  /*
     SQL_TIME cannot have the fractional part
     http://msdn.microsoft.com/en-us/library/ms709385%28v=vs.85%29.aspx
   */
   ts1.fraction= 0;
 
   /* Same for SQL_DATE - time is truncated -> error */
-  ts2.hour= ts2.minute= ts2.second= ts2.fraction= 0;
+  ts2.fraction= ts2.hour= ts2.minute= ts2.second= 0;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_tstotime");
 
@@ -1039,7 +1043,7 @@ DECLARE_TEST(t_bug60646)
         " ,'1000-01-01 12:00:00.000000001'"                         /*4*/
         " ,time('2011-12-31 23:59:59.999999')"                      /*5*/
         " ,ADDTIME('9999-12-31 23:59:59.999999', '1 1:1:1.000002')" /*6*/
-        ); 
+        );
   ok_stmt(hstmt, SQLFetch(hstmt));
 
   /* Fields 1-4 checking conversions from date as a string
@@ -1218,23 +1222,223 @@ DECLARE_TEST(t_b13975271)
 }
 
 
+DECLARE_TEST(t_17613161)
+{
+  SQL_TIME_STRUCT ts, result;
+  SQL_INTERVAL_STRUCT h2s, interval;
+
+  h2s.intval.day_second.hour=   ts.hour  = 100;
+  h2s.intval.day_second.minute= ts.minute= 20;
+  h2s.intval.day_second.second= ts.second= 45;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161");
+
+  ok_sql(hstmt, "CREATE TABLE t_17613161(col1 time)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"INSERT INTO t_17613161 "
+                            "(col1) VALUES (?)",SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
+                                  SQL_TIME, 0, 0, &ts, sizeof(ts), NULL));
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  /* Such conversion is not supported */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
+                                  SQL_INTERVAL_HOUR_TO_SECOND, 0, 0, &ts, sizeof(ts), NULL));
+ expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+ is_num(check_sqlstate(hstmt, "07006"), OK);
+
+  /* For interval types big hours should work fine */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_INTERVAL_HOUR_TO_SECOND,
+                                  SQL_INTERVAL_HOUR_TO_SECOND, 0, 0, &h2s, sizeof(h2s), NULL));
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_sql(hstmt,"SELECT * FROM t_17613161");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TYPE_TIME, &result, 0,
+                            NULL), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22007"), OK);
+
+  expect_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_INTERVAL_HOUR_TO_MINUTE, &interval, 0,
+                        NULL), SQL_SUCCESS_WITH_INFO);
+  is_num(check_sqlstate(hstmt, "01S07"), OK);
+
+  is_num(interval.intval.day_second.second, 0);
+  is_num(interval.intval.day_second.minute, 20);
+  is_num(interval.intval.day_second.hour, 100);
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_INTERVAL_HOUR_TO_SECOND, &interval, 0,
+                        NULL));
+
+  is_num(interval.intval.day_second.second, 45);
+  is_num(interval.intval.day_second.minute, 20);
+  is_num(interval.intval.day_second.hour, 100);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_17613161_bookmark)
+{
+  SQLCHAR bData[2][10];
+  SQL_TIME_STRUCT tm[2]= {0};
+  SQL_INTERVAL_STRUCT h2s[2];
+
+  h2s[0].intval.day_second.hour=   tm[0].hour   = 11;
+  h2s[0].intval.day_second.minute= tm[0].minute = 02;
+  h2s[0].intval.day_second.second= tm[0].second = 19;
+
+  h2s[1].intval.day_second.hour=   tm[1].hour  = 100;
+  h2s[1].intval.day_second.minute= tm[1].minute= 20;
+  h2s[1].intval.day_second.second= tm[1].second= 45;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161_bookmark");
+
+  ok_sql(hstmt,"CREATE TABLE t_17613161_bookmark(col1 time)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)2, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CONCURRENCY,
+                                (SQLPOINTER)SQL_CONCUR_ROWVER, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_USE_BOOKMARKS,
+                                (SQLPOINTER) SQL_UB_VARIABLE, 0));
+
+  ok_sql(hstmt, "SELECT * FROM t_17613161_bookmark");
+
+  expect_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_NEXT, 0),
+              SQL_NO_DATA_FOUND);
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 0, SQL_C_VARBOOKMARK, bData,
+                            sizeof(bData[0]), NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_TYPE_TIME, tm,
+                            sizeof(tm[0]), NULL));
+
+  expect_stmt(hstmt, SQLBulkOperations(hstmt, SQL_ADD), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_INTERVAL_HOUR_TO_SECOND, h2s,
+                            sizeof(h2s[0]), NULL));
+  ok_stmt(hstmt, SQLBulkOperations(hstmt, SQL_ADD));
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_UNBIND));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)1, 0));
+
+  ok_sql(hstmt, "SELECT * FROM t_17613161_bookmark");
+  is_num(myrowcount(hstmt), 2);
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0));
+
+  is_num(atol(bData[0]), 1);
+  is_num(tm[0].hour, 11);
+  is_num(tm[0].minute, 02);
+  is_num(tm[0].second, 19);
+
+  is_num(atol(bData[1]), 2);
+  is_num(tm[1].hour, 100);
+  is_num(tm[1].minute, 20);
+  is_num(tm[1].second, 45);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_17613161_bookmark");
+
+  return OK;
+}
+
+
+/**
+ Bug #25386024: Error "Date Overflow"
+*/
+DECLARE_TEST(t_date_overflow)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+           NULL, NULL, NULL, "NO_DATE_OVERFLOW=1"));
+
+  char *query_prep = "INSERT INTO t_date_overflow (a) VALUES (?)";
+  SQL_TIMESTAMP_STRUCT ts;
+
+  ts.day = 14;
+  ts.month = 2;
+  ts.year = 2017;
+  ts.hour = 16;
+  ts.minute = 11;
+  ts.second = 45;
+  ts.fraction = 0;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_date_overflow");
+  ok_sql(hstmt, "CREATE TABLE t_date_overflow (a DATE)");
+
+  /* By default the INSERT should end with the error */
+  ok_stmt(hstmt, SQLPrepare(hstmt, query_prep, SQL_NTS));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIMESTAMP,
+                                  SQL_DATE, 0, 0, &ts, sizeof(ts), NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+
+  /* Empty result is expected */
+  ok_sql(hstmt, "SELECT * FROM t_date_overflow");
+  is_num(my_print_non_format_result(hstmt), 0);
+
+  /* The INSERT with NO_DATE_OVERFLOW=1 should be successful */
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, query_prep, SQL_NTS));
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT, SQL_C_TIMESTAMP,
+                                  SQL_DATE, 0, 0, &ts, sizeof(ts), NULL));
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  /* One row is expected */
+  ok_sql(hstmt, "SELECT * FROM t_date_overflow");
+  is_num(my_print_non_format_result(hstmt), 1);
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_date_overflow");
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+
+
+
 BEGIN_TESTS
+  ADD_TEST(t_date_overflow)
   ADD_TEST(my_ts)
+#ifndef USE_IODBC
   ADD_TEST(t_tstotime)
   ADD_TEST(t_tstotime1)
-  ADD_TEST(t_bug25846)
-  ADD_TEST(t_time)
-  ADD_TEST(t_time1)
-  ADD_TEST(t_bug12520)
-  ADD_TEST(t_bug15773)
-  ADD_TEST(t_bug9927)
-  ADD_TEST(t_bug30081)
   ADD_TEST(t_datecolumns)
-  ADD_TEST(t_bug14414)
-  ADD_TEST(t_bug30939)
   ADD_TEST(t_bug31009)
-  ADD_TEST(t_bug37342)
-  ADD_TEST(t_bug60646)
+  ADD_TEST(t_17613161)
+  ADD_TEST(t_17613161_bookmark)
+#endif
+  ADD_TEST(t_bug25846)
+  // ADD_TEST(t_time) TODO: Fix
+  ADD_TEST(t_time1)
+  // ADD_TEST(t_bug12520) TODO: Fix
+  // ADD_TEST(t_bug15773) TODO: Fix
+  // ADD_TEST(t_bug9927)  TODO: Fix
+  // ADD_TEST(t_bug30081) TODO: Fix
+  // ADD_TEST(t_bug14414) TODO: Fix
+  ADD_TEST(t_bug30939)
+  // ADD_TEST(t_bug37342) TODO: Fix
+  // ADD_TEST(t_bug60646) TODO: Fix
   ADD_TEST(t_bug60648)
   ADD_TEST(t_b13975271)
 END_TESTS
