@@ -28,6 +28,257 @@
 
 #include "odbctap.h"
 
+typedef struct {
+  long            length;
+  unsigned char data[4096];
+} T_LOB;
+
+DECLARE_TEST(my_param_data)
+{
+  int   i, rcnt, c1 = 1, c2 = 4096;
+  T_LOB c3, *pt;
+  c3.length = c2;
+  SQLRETURN rc = 0;
+  SQLLEN c1_len_or_ind = 0, c2_len_or_ind = 0, c3_len_or_ind = 0;
+
+  unsigned char *org_data[10];
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  /* Connect with SSPS enabled */
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
+    NULL, "NO_SSPS=1");
+
+  SQLExecDirect(hstmt, "DROP TABLE IF EXISTS test_param_data", SQL_NTS);
+
+  ok_sql(hstmt, "CREATE TABLE test_param_data"\
+                "(c1 int, c2 int, c3 longblob)");
+
+  for (c1 = 1; c1 < 11; ++c1)
+  {
+    c3.length = (c1 % 2) ? 944 : 3312;
+    org_data[c1 - 1] = calloc(c3.length + 100, 1);
+
+    SQLLEN indic1 = sizeof(long);
+    SQLLEN indic2 = sizeof(long);
+    SQLLEN indic3 = SQL_LEN_DATA_AT_EXEC(c3.length);
+
+    for (i = 0; i < c3.length; i++)
+    {
+      org_data[c1 - 1][i] = c3.data[i] = i % 256;
+    }
+
+    ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT,
+            SQL_C_LONG, SQL_INTEGER, sizeof(long), 0, &c1,
+            sizeof(long), &indic1));
+
+    ok_stmt(hstmt1, SQLBindParameter(hstmt1, 2, SQL_PARAM_INPUT,
+            SQL_C_LONG, SQL_INTEGER, sizeof(long), 0, &c2,
+            sizeof(long), &indic2));
+
+    ok_stmt(hstmt1, SQLBindParameter(hstmt1, 3, SQL_PARAM_INPUT,
+            SQL_C_BINARY, SQL_LONGVARBINARY, sizeof(T_LOB), 0, &c3,
+            sizeof(T_LOB), &indic3));
+
+    expect_stmt(hstmt1, SQLExecDirect(hstmt1, "INSERT INTO test_param_data(c1, c2, c3)"\
+                "VALUES (?, ?, ?)", SQL_NTS),
+                SQL_NEED_DATA);
+
+
+    expect_stmt(hstmt1, SQLParamData(hstmt1, (SQLPOINTER)&pt),
+                SQL_NEED_DATA);
+
+    ok_stmt(hstmt1, SQLPutData(hstmt1, pt->data, pt->length));
+
+    ok_stmt(hstmt1, SQLParamData(hstmt1, (SQLPOINTER)&pt));
+
+  }
+
+  ok_sql(hstmt1, "SELECT * FROM test_param_data ORDER BY c1");
+
+  ok_stmt(hstmt1, SQLBindCol(hstmt1, 1, SQL_C_ULONG, &c1, sizeof(c1), &c1_len_or_ind));
+  ok_stmt(hstmt1, SQLBindCol(hstmt1, 2, SQL_C_ULONG, &c2, sizeof(c2), &c2_len_or_ind));
+  ok_stmt(hstmt1, SQLBindCol(hstmt1, 3, SQL_C_BINARY, c3.data, 4000, &c3_len_or_ind));
+
+  rcnt = 1;
+  while (1)
+  {
+    int expected_length = (rcnt % 2) ? 944 : 3312;
+    memset(c3.data, 0, 4000);
+    c1 = 0;
+    c2 = 0;
+    if (SQLFetch(hstmt1) != SQL_SUCCESS)
+      break;
+    is_num(c1, rcnt);
+    is_num(c2, 4096);
+    is_num(c3_len_or_ind, expected_length);
+    is(memcmp(c3.data, org_data[rcnt-1], expected_length) == 0);
+    ++rcnt;
+  }
+
+  is_num(rcnt, 11);
+
+  for(i = 0; i < 10; ++i)
+    free(org_data[i]);
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+DECLARE_TEST(t_bug31373948)
+{
+  char *buf;
+  char foo[16] = "foo ";
+  char *ins_query = "INSERT INTO t_bug31373948 (name, largedata) VALUES (?,?)";
+  char *sel_query = "SELECT id, largedata FROM t_bug31373948 WHERE name LIKE ? "\
+                    "ORDER BY id";
+  SQLUINTEGER id = 0;
+  unsigned long rnum = 0;
+  SQLLEN idLenOrInd = 0;
+  SQLLEN largeLenOrInd = 0;
+  SQLRETURN rcc = 0;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug31373948");
+  ok_sql(hstmt, "CREATE TABLE t_bug31373948 (id int primary key auto_increment,"\
+                "name VARCHAR(45), largedata VARCHAR(16000))");
+  buf = malloc(100*1000);
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, ins_query, SQL_NTS));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 0,
+                                  0, foo, 5, NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 0,
+                                  0, buf, 16000, NULL));
+
+  for (char i = 'A'; i < 'Q'; ++i)
+    memset(buf + (i - 'A') * 1000, i, 1000);
+
+  for (char c = '0'; c <= '9'; ++c)
+  {
+    buf[0] = c;
+    foo[3] = c;
+    ok_stmt(hstmt, SQLExecute(hstmt));
+  }
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, sel_query, SQL_NTS));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 0,
+    0, foo, 5, NULL));
+  foo[3] = '%';
+  ok_stmt(hstmt, SQLExecute(hstmt));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_ULONG, &id, sizeof(id), &idLenOrInd));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, buf, 17000, &largeLenOrInd));
+
+  memset(buf, 0, 16000);
+
+  while (SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+
+    printMessage("id: %d, Len 1: %ld, Len 2: %ld", id, idLenOrInd, largeLenOrInd);
+    is(id == (rnum + 1));
+
+    idLenOrInd = 0;
+    is(largeLenOrInd == 16000);
+    largeLenOrInd = 0;
+
+    is(buf[0] == '0' + rnum);
+
+    for (unsigned i = 1; i < 16000; ++i)
+    {
+      is (buf[i] == 'A' + i / 1000);
+    }
+
+    ++rnum;
+    memset(buf, 0, 16000);
+  }
+  is(rnum == 10);
+  free(buf);
+  return OK;
+}
+
+DECLARE_TEST(t_bug30428851)
+{
+  SQLRETURN rc = 0;
+#define NROW 2
+  SQLINTEGER par = NROW;
+  int i = 0;
+  SQLLEN    len = 0, offset = 0;
+  typedef struct data_fetch
+  {
+    SQLINTEGER id;
+    SQLCHAR strVal[32];
+    SQLINTEGER intVal;
+    SQLLEN     len1;
+    SQLLEN     len2;
+    SQLLEN     len3;
+  }df;
+
+  df data;
+  df src[NROW] = {{1, "Test1", 10, 4, 5, 4},
+                  {2, "Test2", 20, 4, 5, 4}};
+
+  memset(&data, 0, sizeof(data));
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug30428851");
+
+  ok_sql(hstmt, "CREATE TABLE bug30428851( " \
+    "`id` int(11) NOT NULL AUTO_INCREMENT, " \
+    "`strVal` varchar(45) DEFAULT NULL, " \
+    "`intVal` int(11) DEFAULT NULL, " \
+    "`txtVal` longtext, " \
+    " PRIMARY KEY(`id`))");
+
+  for (i = 0; i < NROW; ++i)
+  {
+    char buf[512] = {0};
+    snprintf(buf, 512, "INSERT INTO bug30428851 (id, strVal, intVal) VALUES (%d, '%s', %d)",
+    src[i].id, src[i].strVal, src[i].intVal);
+    ok_stmt(hstmt, SQLExecDirect(hstmt, buf, SQL_NTS));
+  }
+
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+    "SELECT * FROM `bug30428851` WHERE id = ? LIMIT 1;", SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
+                                  SQL_C_SLONG, SQL_INTEGER, 10, 0, &par,
+                                  0, &len));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
+                                (SQLPOINTER)1, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_BIND_TYPE,
+                                (SQLPOINTER)sizeof(df), 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_BIND_OFFSET_PTR,
+                                (SQLPOINTER)&offset, 0));
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG,
+                            (SQLPOINTER)&data.id,
+                            sizeof(SQLINTEGER),
+                            (SQLLEN*)&data.len1));
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR,
+                            (SQLPOINTER)data.strVal,
+                            sizeof(data.strVal),
+                            (SQLLEN*)&data.len2));
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 3, SQL_C_LONG,
+                            (SQLPOINTER)&data.intVal,
+                            sizeof(SQLINTEGER),
+                            (SQLLEN*)&data.len3));
+
+  while(SQLFetch(hstmt) == SQL_SUCCESS);
+
+  printf("ROW # %i: %d %s %d \n", i, data.id, data.strVal, data.intVal);
+  is_num(src[NROW-1].id, data.id);
+  is_str(src[NROW-1].strVal, data.strVal, src[NROW-1].len2);
+  is_num(src[NROW-1].intVal, data.intVal);
+
+  is_num(src[NROW-1].len1, data.len1);
+  is_num(src[NROW-1].len2, data.len2);
+  is_num(src[NROW-1].len3, data.len3);
+
+  return OK;
+}
+
+
 /********************************************************
 * initialize tables                                     *
 *********************************************************/
@@ -745,6 +996,86 @@ DECLARE_TEST(paramarray_select)
 #undef STMTS_TO_EXEC
 }
 
+/*
+  Bug #31678876/100329 ODBC Driver overwriting memory causes process dead
+  when parameterized query on
+*/
+DECLARE_TEST(t_bug31678876)
+{
+  int i = 0;
+  char buf2[128] = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e'};
+  char buf3[128] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E',
+                    'F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T'};
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug31678876");
+  ok_sql(hstmt, "CREATE TABLE bug31678876 (id int primary key auto_increment,"\
+                "col2 VARCHAR(255), col3 VARCHAR(255)) "\
+                "DEFAULT CHARACTER SET = utf8;");
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, "INSERT INTO bug31678876 "\
+                                   "(col2, col3) VALUES (?,?)", SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
+                                  SQL_C_CHAR, SQL_CHAR, 0, 0, &buf2,
+                                  15, NULL));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT,
+                                  SQL_C_CHAR, SQL_CHAR, 0, 0, &buf3,
+                                  32, NULL));
+
+  for (i = 0; i < 100; ++i)
+    ok_stmt(hstmt, SQLExecute(hstmt));
+
+  wchar_t sel[] = L"select distinct `pa11`.`id` `id`,"\
+  "CONCAT(`a12`.`col3`, `a12`.`col2`) `CustCol_12`,"\
+  "CONCAT((Case when `pa11`.`MYCOL` = ? then ? else ? end), `pa11`.`MYCOL`) `MYCOL`"\
+  "from (select `a11`.`id` `id`,"\
+  " max(CONCAT(`a11`.`col3`, `a11`.`col2`)) `MYCOL`"\
+  " from `bug31678876` `a11`"\
+  " group by `a11`.`id`"\
+  " ) `pa11`"\
+  " join `bug31678876` `a12`"\
+  " on (`pa11`.`id` = `a12`.`id`)";
+
+  SQLWCHAR p1[] = {0x014e, 0xe452, 0xc56b};
+  SQLWCHAR p2[] = {0x1f77};
+  SQLWCHAR p3[] = {0x4750};
+
+  SQLULEN len1 = sizeof(p1);
+  SQLULEN len2 = sizeof(p2);
+  SQLULEN len3 = sizeof(p3);
+
+  ok_stmt(hstmt, SQLPrepareW(hstmt, W(sel), SQL_NTS));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
+                                  SQL_C_WCHAR, SQL_WCHAR, 0, 0, &p1,
+                                  sizeof(p1), &len1));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT,
+                                  SQL_C_WCHAR, SQL_WCHAR, 0, 0, &p2,
+                                  sizeof(p2), &len2));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT,
+                                  SQL_C_WCHAR, SQL_WCHAR, 0, 0, &p3,
+                                  sizeof(p3), &len3));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+  int rnum = 0;
+
+  while(SQLFetch(hstmt) == SQL_SUCCESS)
+  {
+    int col1;
+    SQLULEN col1_len = 0, col2_len = 0, col3_len = 0;
+    SQLWCHAR col2[128];
+    SQLWCHAR col3[128];
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_LONG, &col1, sizeof(int), &col1_len));
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, &col2, sizeof(col2), &col2_len));
+    ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_WCHAR, &col3, sizeof(col3), &col3_len));
+    ++rnum;
+    // Crash happens when the local buffers go out of scope
+  }
+  printf("Fetched %d rows\n", rnum);
+  return OK;
+}
+
 
 /*
   Bug #49029 - Server with sql mode NO_BACKSLASHES_ESCAPE obviously
@@ -765,13 +1096,33 @@ DECLARE_TEST(t_bug49029)
 
   ok_stmt(hstmt, SQLFetch(hstmt));
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_BINARY, (SQLPOINTER)buff, 6, &len));
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   is(memcmp((const void*) buff, (const void*)bData, 5)==0);
 
+  SQLCHAR *bData128K = (SQLCHAR*)calloc(128000, 1);
+  SQLCHAR *rData128K = (SQLCHAR*)calloc(128000, 1);
+  SQLULEN b_len= 1020, r_len = 0;
+
+  memcpy(bData128K, "\x01\x80\x00\x80\x01", 5);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_BINARY, SQL_BINARY,
+    0, 0, (SQLPOINTER)bData128K, 0, &b_len));
+
+  ok_stmt(hstmt, SQLExecDirect(hstmt, "select ?", SQL_NTS));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_BINARY, (SQLPOINTER)rData128K, 80000, &r_len));
+
+  is(memcmp((const void*)bData128K, (const void*)rData128K, b_len)==0);
+  free(bData128K);
+  free(rData128K);
   return OK;
 }
 
 
+
 /*
   Bug #56804 - Server with sql mode NO_BACKSLASHES_ESCAPE obviously
   can work incorrectly (at least) with binary parameters
@@ -918,6 +1269,8 @@ DECLARE_TEST(t_bug59772)
     if (rc != SQL_ERROR)
       overall_result= FAIL;
 
+    is (paramsProcessed == ROWS_TO_INSERT);
+
     for (i= 0; i < paramsProcessed; ++i)
 
       /* We expect error statuses for all parameters */
@@ -1724,6 +2077,7 @@ DECLARE_TEST(t_bug28175772)
           " BEGIN"\
           " SET param1 := CONCAT(param1, 'bar'); "\
           " SET param2 := CONCAT(param2, 'bar'); "\
+          " SELECT 1; "\
           " END; ");
   sprintf(sql_drop_proc, "%s", "DROP PROCEDURE if exists inoutproc");
   sprintf(sql_select, "%s", "call inoutproc(?, ?)");
@@ -1759,10 +2113,145 @@ DECLARE_TEST(t_bug28175772)
 }
 
 
+DECLARE_TEST(t_sp_return)
+{
+  char sql_select[100] = { 0 };
+  SQLLEN iSize = SQL_NTS, iSize1 = SQL_NTS;
+  char val1[200] = { 0 }, val2[200] = { 0 };
+  SQLCHAR       SqlState[6], Msg[SQL_MAX_MESSAGE_LENGTH];
+  SQLINTEGER    NativeError;
+  SQLLEN len = 0;
+  SQLSMALLINT   i, MsgLen;
+  char strVal[255];
+
+
+  // connection strings
+  strcpy(val1, "test data 1");
+  strcpy(val2, "test data 2");
+
+  ok_sql(hstmt, "DROP PROCEDURE if exists simpleproc");
+  ok_sql(hstmt, "CREATE PROCEDURE simpleproc (param1 VARCHAR(100), " \
+          " param2 VARCHAR(100))"\
+          " BEGIN"\
+          " SELECT param1 UNION SELECT param2 UNION SELECT 'ABC'; "\
+          " END; ");
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR*)"call simpleproc(?, ?)", SQL_NTS));
+  //bind the call statement 1
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_CHAR, 0, 0, val1, 200, &iSize));
+  //bind the call statement 1
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT_OUTPUT, SQL_C_CHAR,
+                                  SQL_CHAR, 0, 0, val2, 200, &iSize1));
+  // execute
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR,
+    (SQLPOINTER)strVal, sizeof(strVal), &len));
+
+  memset(strVal, 0, sizeof(strVal));
+  i = 0;
+  while (SQL_SUCCESS == SQLFetch(hstmt))
+  {
+    switch (i)
+    {
+      case 0:
+        is_str("test data 1", strVal, 11); break;
+      case 1:
+        is_str("test data 2", strVal, 11); break;
+      case 2:
+        is_str("ABC", strVal, 3); break;
+    }
+    ++i;
+  }
+  is_num(3, i);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  return OK;
+}
+
+/*
+  Bug 30591722
+  Only a single value is being inserted instead of the array with
+  SQLParamOptions
+*/
+DECLARE_TEST(t_bug30591722)
+{
+  #undef RCNT
+  #define RCNT 100
+
+  int cnt = 0;
+
+  SQLUINTEGER lval[RCNT] = { 0 };
+  SQLUINTEGER uintval1[RCNT] = { 0 };
+  SQLUINTEGER uintval2[RCNT] = { 0 };
+
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  /* Connect with SSPS enabled */
+  printf("\nCreating a new connection NO_SSPS=1\n");
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL,
+    "test", "NO_SSPS=1");
+
+  printf("Creating a table\n");
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS bug30591722");
+  ok_sql(hstmt1, "CREATE TABLE bug30591722(id INT, id2 INT)");
+
+  ok_stmt(hstmt1, SQLParamOptions(hstmt1, RCNT, (SQLULEN*)&lval));
+
+  for (int j = 0; j < RCNT; j++)
+  {
+    uintval1[j] = j;
+    uintval2[j] = 100 + j;
+  }
+
+  printf("SQLPrepare\n");
+  ok_stmt(hstmt1, SQLPrepare(hstmt1,
+    "INSERT INTO bug30591722 VALUES (?+1, ?+100)", SQL_NTS));
+
+  printf("Bind Parameters\n");
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 1,
+    SQL_PARAM_INPUT, SQL_C_ULONG, SQL_NUMERIC, 4, 0, &uintval1, 0, NULL));
+
+  ok_stmt(hstmt1, SQLBindParameter(hstmt1, 2,
+    SQL_PARAM_INPUT, SQL_C_ULONG, SQL_NUMERIC, 4, 0, &uintval2, 0, NULL));
+
+  printf("SQLExecute\n");
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  printf("Checking the result using SELECT\n");
+  ok_stmt(hstmt1, SQLExecDirect(hstmt1,
+    (SQLCHAR*)"select * from bug30591722 order by id", SQL_NTS));
+
+  while (SQLFetch(hstmt1) == SQL_SUCCESS)
+  {
+    SQLUINTEGER id1 = 0, id2 = 0;
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_ULONG, &id1, 0, NULL));
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_ULONG, &id2, 0, NULL));
+    is_num(cnt + 1, id1);
+    is_num(cnt + 200, id2);
+    ++cnt;
+  }
+  is_num(100, cnt);
+  printf("Freeing handles\n");
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  printf("Handles are freed\n");
+  return OK;
+}
+
+
 BEGIN_TESTS
-  // ADD_TEST(t_bug28175772)
+  ADD_TEST(my_param_data)
+  ADD_TEST(t_bug31373948)
+  ADD_TEST(t_bug30591722)
+  ADD_TEST(t_sp_return)
+  // ADD_TEST(t_bug28175772) the libmysqlclient SERVER_PS_OUT_PARAMS flag
+  //                         is not working the test should be re-enabled
+  //                         after bug 30566136 is fixed
+  ADD_TEST(t_bug30428851)
   ADD_TEST(my_init_table)
 #ifndef USE_IODBC
+  ADD_TEST(t_bug59772)
   ADD_TEST(my_param_insert)
   ADD_TEST(my_param_update)
   ADD_TEST(my_param_delete)
@@ -1772,10 +2261,10 @@ BEGIN_TESTS
   ADD_TEST(paramarray_ignore_paramset)
   ADD_TEST(paramarray_select)
   ADD_TEST(t_bug56804)
+  ADD_TEST(t_bug31678876)
 #endif
   ADD_TEST(t_param_offset)
   ADD_TEST(t_bug49029)
-  // ADD_TEST(t_bug59772) TODO: Fix
   // ADD_TEST(t_odbcoutparams) TODO: Fix
   // ADD_TEST(t_bug14501952) TODO: Fix
   // ADD_TEST(t_bug14563386) TODO: Fix
