@@ -1,30 +1,30 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
@@ -1255,6 +1255,33 @@ DECLARE_TEST(t_bug67920)
   return OK;
 }
 
+/*
+  Bug #31667091: ODBC prepared statements broken by changes
+  in behavior of @@sql_select_limit
+*/
+DECLARE_TEST(t_bug31667091)
+{
+  int id = 10;
+  ok_sql(hstmt,"DROP TABLE IF EXISTS bug31667091");
+  ok_sql(hstmt,"CREATE TABLE bug31667091 (id int)");
+  ok_sql(hstmt,"INSERT INTO bug31667091 VALUES (1), (2), (3)");
+  // Set row number limit to 1
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS, (SQLPOINTER)1, 0));
+  ok_sql(hstmt, "SELECT * FROM bug31667091");
+  // Make sure the limit is applied
+  is_num(my_print_non_format_result(hstmt), 1);
+
+  ok_stmt(hstmt, SQLPrepare(hstmt, "SELECT * FROM bug31667091 WHERE id < ?",
+                            SQL_NTS));
+  // Remove row number limit
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_MAX_ROWS, (SQLPOINTER)0, 0));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_LONG,
+                                  SQL_INTEGER, 0, 0, &id, 0, NULL));
+  ok_stmt(hstmt, SQLExecute(hstmt));
+  is_num(my_print_non_format_result(hstmt), 3);
+  ok_sql(hstmt,"DROP TABLE bug31667091");
+  return OK;
+}
 
 BEGIN_TESTS
   ADD_TEST(t_prep_basic)
@@ -1277,6 +1304,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug67702)
   ADD_TEST(t_bug68243)
   ADD_TEST(t_bug67920)
+  // ADD_TODO(t_bug31667091) Re-enable when server fixes sql_select_limit
 END_TESTS
 
 
