@@ -1,30 +1,30 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /*
  * Installer wrapper implementations.
@@ -120,8 +120,6 @@ static SQLWCHAR W_NO_PROMPT[]=
   {'N','O','_','P','R','O','M','P','T',0};
 static SQLWCHAR W_DYNAMIC_CURSOR[]=
   {'D','Y','N','A','M','I','C','_','C','U','R','S','O','R',0};
-static SQLWCHAR W_NO_SCHEMA[]=
-  {'N','O','_','S','C','H','E','M','A',0};
 static SQLWCHAR W_NO_DEFAULT_CURSOR[]=
   {'N','O','_','D','E','F','A','U','L','T','_','C','U','R','S','O','R',0};
 static SQLWCHAR W_NO_LOCALE[]=
@@ -154,6 +152,10 @@ static SQLWCHAR W_FORWARD_CURSOR[]=
   {'F','O','R','W','A','R','D','_','C','U','R','S','O','R',0};
 static SQLWCHAR W_AUTO_RECONNECT[]=
   {'A','U','T','O','_','R','E','C','O','N','N','E','C','T',0};
+static SQLWCHAR W_ENABLE_DNS_SRV[]=
+  {'E','N','A','B','L','E','_','D','N','S','_','S','R','V',0};
+static SQLWCHAR W_MULTI_HOST[]=
+  {'M','U','L','T','I','_','H','O','S','T',0};
 static SQLWCHAR W_AUTO_IS_NULL[]=
   {'A','U','T','O','_','I','S','_','N','U','L','L',0};
 static SQLWCHAR W_ZERO_DATE_TO_MIN[]=
@@ -184,10 +186,6 @@ static SQLWCHAR W_PLUGIN_DIR[]=
   {'P','L','U','G','I','N','_','D','I','R',0};
 static SQLWCHAR W_DEFAULT_AUTH[]=
   {'D','E','F','A','U','L','T','_','A','U','T', 'H',0};
-static SQLWCHAR W_DISABLE_SSL_DEFAULT[] =
-{ 'D', 'I', 'S', 'A', 'B', 'L', 'E', '_', 'S', 'S', 'L', '_', 'D', 'E', 'F', 'A', 'U', 'L', 'T', 0 };
-static SQLWCHAR W_SSL_ENFORCE[] =
-{ 'S', 'S', 'L', '_', 'E', 'N', 'F', 'O', 'R', 'C', 'E', 0 };
 static SQLWCHAR W_NO_TLS_1[] =
 { 'N', 'O', '_', 'T', 'L', 'S', '_', '1', '_', '0', 0 };
 static SQLWCHAR W_NO_TLS_1_1[] =
@@ -200,6 +198,8 @@ static SQLWCHAR W_NO_DATE_OVERFLOW[] =
 { 'N', 'O', '_', 'D', 'A', 'T', 'E', '_', 'O', 'V', 'E', 'R', 'F', 'L', 'O', 'W', 0 };
 static SQLWCHAR W_ENABLE_LOCAL_INFILE[] =
 { 'E', 'N', 'A', 'B', 'L', 'E', '_', 'L', 'O', 'C', 'A', 'L', '_', 'I', 'N', 'F', 'I', 'L', 'E', 0 };
+static SQLWCHAR W_LOAD_DATA_LOCAL_DIR[] =
+{ 'L', 'O', 'A', 'D', '_', 'D', 'A', 'T', 'A', '_', 'L', 'O', 'C', 'A', 'L', '_', 'D', 'I', 'R', 0 };
 
 /* DS_PARAM */
 /* externally used strings */
@@ -219,7 +219,7 @@ SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
                         W_SSLCERT, W_SSLCA, W_SSLCAPATH, W_SSLCIPHER,
                         W_SSLVERIFY, W_READTIMEOUT, W_WRITETIMEOUT,
                         W_FOUND_ROWS, W_BIG_PACKETS, W_NO_PROMPT,
-                        W_DYNAMIC_CURSOR, W_NO_SCHEMA, W_NO_DEFAULT_CURSOR,
+                        W_DYNAMIC_CURSOR, W_NO_DEFAULT_CURSOR,
                         W_NO_LOCALE, W_PAD_SPACE, W_FULL_COLUMN_NAMES,
                         W_COMPRESSED_PROTO, W_IGNORE_SPACE, W_NAMED_PIPE,
                         W_NO_BIGINT, W_NO_CATALOG, W_USE_MYCNF, W_SAFE,
@@ -230,11 +230,10 @@ SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
                         W_NO_BINARY_RESULT, W_DFLT_BIGINT_BIND_STR,
                         W_CLIENT_INTERACTIVE, W_NO_I_S, W_PREFETCH, W_NO_SSPS,
                         W_CAN_HANDLE_EXP_PWD, W_ENABLE_CLEARTEXT_PLUGIN,
-                        W_GET_SERVER_PUBLIC_KEY,
+                        W_GET_SERVER_PUBLIC_KEY, W_ENABLE_DNS_SRV, W_MULTI_HOST,
                         W_SAVEFILE, W_RSAKEY, W_PLUGIN_DIR, W_DEFAULT_AUTH,
-                        W_DISABLE_SSL_DEFAULT, W_SSL_ENFORCE,
                         W_NO_TLS_1, W_NO_TLS_1_1, W_NO_TLS_1_2,
-                        W_SSLMODE, W_NO_DATE_OVERFLOW};
+                        W_SSLMODE, W_NO_DATE_OVERFLOW, W_LOAD_DATA_LOCAL_DIR};
 static const
 int dsnparamcnt= sizeof(dsnparams) / sizeof(SQLWCHAR *);
 /* DS_PARAM */
@@ -655,7 +654,8 @@ DataSource *ds_new()
   memset(ds, 0, sizeof(DataSource));
 
   /* non-zero DataSource defaults here */
-  ds->port=                   3306;
+  ds->port = 3306;
+  ds->has_port = false;
   /* DS_PARAM */
 
   return ds;
@@ -687,6 +687,7 @@ void ds_delete(DataSource *ds)
   x_free(ds->savefile);
   x_free(ds->plugin_dir);
   x_free(ds->default_auth);
+  x_free(ds->load_data_local_dir);
 
   x_free(ds->name8);
   x_free(ds->driver8);
@@ -708,6 +709,7 @@ void ds_delete(DataSource *ds)
   x_free(ds->savefile8);
   x_free(ds->plugin_dir8);
   x_free(ds->default_auth8);
+  x_free(ds->load_data_local_dir8);
 
   x_free(ds);
 }
@@ -809,9 +811,11 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *strdest= &ds->savefile;
   else if (!sqlwcharcasecmp(W_RSAKEY, param))
     *strdest= &ds->rsakey;
-
   else if (!sqlwcharcasecmp(W_PORT, param))
+  {
+    ds->has_port = true;
     *intdest= &ds->port;
+  }
   else if (!sqlwcharcasecmp(W_SSLVERIFY, param))
     *intdest= &ds->sslverify;
   else if (!sqlwcharcasecmp(W_READTIMEOUT, param))
@@ -830,8 +834,6 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest= &ds->dont_prompt_upon_connect;
   else if (!sqlwcharcasecmp(W_DYNAMIC_CURSOR, param))
     *booldest= &ds->dynamic_cursor;
-  else if (!sqlwcharcasecmp(W_NO_SCHEMA, param))
-    *booldest= &ds->ignore_N_in_name_table;
   else if (!sqlwcharcasecmp(W_NO_DEFAULT_CURSOR, param))
     *booldest= &ds->user_manager_cursor;
   else if (!sqlwcharcasecmp(W_NO_LOCALE, param))
@@ -888,14 +890,14 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest= &ds->enable_cleartext_plugin;
   else if (!sqlwcharcasecmp(W_GET_SERVER_PUBLIC_KEY, param))
     *booldest = &ds->get_server_public_key;
+  else if (!sqlwcharcasecmp(W_ENABLE_DNS_SRV, param))
+    *booldest = &ds->enable_dns_srv;
+  else if (!sqlwcharcasecmp(W_MULTI_HOST, param))
+    *booldest = &ds->multi_host;
   else if (!sqlwcharcasecmp(W_PLUGIN_DIR, param))
     *strdest= &ds->plugin_dir;
   else if (!sqlwcharcasecmp(W_DEFAULT_AUTH, param))
     *strdest= &ds->default_auth;
-  else if (!sqlwcharcasecmp(W_DISABLE_SSL_DEFAULT, param))
-    *booldest = &ds->disable_ssl_default;
-  else if (!sqlwcharcasecmp(W_SSL_ENFORCE, param))
-    *booldest = &ds->ssl_enforce;
   else if (!sqlwcharcasecmp(W_NO_TLS_1, param))
     *booldest = &ds->no_tls_1;
   else if (!sqlwcharcasecmp(W_NO_TLS_1_1, param))
@@ -906,6 +908,8 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest = &ds->no_date_overflow;
   else if (!sqlwcharcasecmp(W_ENABLE_LOCAL_INFILE, param))
     *booldest = &ds->enable_local_infile;
+  else if (!sqlwcharcasecmp(W_LOAD_DATA_LOCAL_DIR, param))
+    *strdest= &ds->load_data_local_dir;
 
   /* DS_PARAM */
 }
@@ -921,7 +925,8 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
  */
 int ds_lookup(DataSource *ds)
 {
-  SQLWCHAR buf[8192];
+#define DS_BUF_LEN 8192
+  SQLWCHAR buf[DS_BUF_LEN];
   SQLWCHAR *entries= buf;
   SQLWCHAR **dest;
   SQLWCHAR val[256];
@@ -932,14 +937,11 @@ int ds_lookup(DataSource *ds)
   BOOL *booldest;
   /* No need for SAVE_MODE() because we always call config_get() above. */
 
-#ifdef _WIN32
-  /* We must do this to detect the WinXP bug mentioned below */
   memset(buf, 0xff, sizeof(buf));
-#endif
 
   /* get entries and check if data source exists */
   if ((size= SQLGetPrivateProfileStringW(ds->name, NULL, W_EMPTY,
-                                         buf, 8192, W_ODBC_INI)) < 1)
+                                         buf, DS_BUF_LEN, W_ODBC_INI)) < 1)
   {
     rc= -1;
     goto end;
@@ -982,7 +984,7 @@ int ds_lookup(DataSource *ds)
     /* revert to system mode and try again */
     config_set(ODBC_SYSTEM_DSN);
     if ((size= SQLGetPrivateProfileStringW(ds->name, NULL, W_EMPTY,
-                                           buf, 8192, W_ODBC_INI)) < 1)
+                                           buf, DS_BUF_LEN, W_ODBC_INI)) < 1)
     {
       rc= -1;
       goto end;
@@ -990,8 +992,10 @@ int ds_lookup(DataSource *ds)
   }
 #endif
 
-  for (used= 0; used < size; used += sqlwcharlen(entries) + 1,
-                             entries += sqlwcharlen(entries) + 1)
+  for (used= 0;
+       used < DS_BUF_LEN && entries[0];
+       used += sqlwcharlen(entries) + 1,
+       entries += sqlwcharlen(entries) + 1)
   {
     int valsize;
     ds_map_param(ds, entries, &dest, &intdest, &booldest);
@@ -1354,7 +1358,8 @@ int ds_add(DataSource *ds)
   if (ds_add_strprop(ds->name, W_SAVEFILE   , ds->savefile   )) goto error;
 
   if (ds_add_intprop(ds->name, W_SSLVERIFY  , ds->sslverify  )) goto error;
-  if (ds_add_intprop(ds->name, W_PORT       , ds->port       )) goto error;
+  if(ds->has_port)
+    if (ds_add_intprop(ds->name, W_PORT       , ds->port       )) goto error;
   if (ds_add_intprop(ds->name, W_READTIMEOUT, ds->readtimeout)) goto error;
   if (ds_add_intprop(ds->name, W_WRITETIMEOUT, ds->writetimeout)) goto error;
   if (ds_add_intprop(ds->name, W_CLIENT_INTERACTIVE, ds->clientinteractive)) goto error;
@@ -1364,7 +1369,6 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_BIG_PACKETS, ds->allow_big_results)) goto error;
   if (ds_add_intprop(ds->name, W_NO_PROMPT, ds->dont_prompt_upon_connect)) goto error;
   if (ds_add_intprop(ds->name, W_DYNAMIC_CURSOR, ds->dynamic_cursor)) goto error;
-  if (ds_add_intprop(ds->name, W_NO_SCHEMA, ds->ignore_N_in_name_table)) goto error;
   if (ds_add_intprop(ds->name, W_NO_DEFAULT_CURSOR, ds->user_manager_cursor)) goto error;
   if (ds_add_intprop(ds->name, W_NO_LOCALE, ds->dont_use_set_locale)) goto error;
   if (ds_add_intprop(ds->name, W_PAD_SPACE, ds->pad_char_to_full_length)) goto error;
@@ -1393,16 +1397,17 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_CAN_HANDLE_EXP_PWD, ds->can_handle_exp_pwd)) goto error;
   if (ds_add_intprop(ds->name, W_ENABLE_CLEARTEXT_PLUGIN, ds->enable_cleartext_plugin)) goto error;
   if (ds_add_intprop(ds->name, W_GET_SERVER_PUBLIC_KEY, ds->get_server_public_key)) goto error;
+  if (ds_add_intprop(ds->name, W_ENABLE_DNS_SRV, ds->enable_dns_srv)) goto error;
+  if (ds_add_intprop(ds->name, W_MULTI_HOST, ds->multi_host)) goto error;
   if (ds_add_strprop(ds->name, W_PLUGIN_DIR  , ds->plugin_dir  )) goto error;
   if (ds_add_strprop(ds->name, W_DEFAULT_AUTH, ds->default_auth)) goto error;
-  if (ds_add_intprop(ds->name, W_DISABLE_SSL_DEFAULT, ds->disable_ssl_default)) goto error;
-  if (ds_add_intprop(ds->name, W_SSL_ENFORCE, ds->ssl_enforce)) goto error;
   if (ds_add_intprop(ds->name, W_NO_TLS_1, ds->no_tls_1)) goto error;
   if (ds_add_intprop(ds->name, W_NO_TLS_1_1, ds->no_tls_1_1)) goto error;
   if (ds_add_intprop(ds->name, W_NO_TLS_1_2, ds->no_tls_1_2)) goto error;
   if (ds_add_intprop(ds->name, W_NO_DATE_OVERFLOW, ds->no_date_overflow)) goto error;
   if (ds_add_intprop(ds->name, W_ENABLE_LOCAL_INFILE, ds->enable_local_infile)) goto error;
-  /* DS_PARAM */
+  if (ds_add_strprop(ds->name, W_LOAD_DATA_LOCAL_DIR, ds->load_data_local_dir)) goto error;
+ /* DS_PARAM */
 
   rc= 0;
 
@@ -1470,7 +1475,6 @@ void ds_set_options(DataSource *ds, ulong options)
   ds->allow_big_results=                    (options & FLAG_BIG_PACKETS) > 0;
   ds->dont_prompt_upon_connect=             (options & FLAG_NO_PROMPT) > 0;
   ds->dynamic_cursor=                       (options & FLAG_DYNAMIC_CURSOR) > 0;
-  ds->ignore_N_in_name_table=               (options & FLAG_NO_SCHEMA) > 0;
   ds->user_manager_cursor=                  (options & FLAG_NO_DEFAULT_CURSOR) > 0;
   ds->dont_use_set_locale=                  (options & FLAG_NO_LOCALE) > 0;
   ds->pad_char_to_full_length=              (options & FLAG_PAD_SPACE) > 0;
@@ -1513,8 +1517,6 @@ ulong ds_get_options(DataSource *ds)
     options|= FLAG_NO_PROMPT;
   if (ds->dynamic_cursor)
     options|= FLAG_DYNAMIC_CURSOR;
-  if (ds->ignore_N_in_name_table)
-    options|= FLAG_NO_SCHEMA;
   if (ds->user_manager_cursor)
     options|= FLAG_NO_DEFAULT_CURSOR;
   if (ds->dont_use_set_locale)
