@@ -1,30 +1,30 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 /**
   @file  stringutil.c
@@ -34,7 +34,7 @@
 
 
 #include "stringutil.h"
-#include <ctype.h>
+
 
 CHARSET_INFO *utf8_charset_info= NULL;
 
@@ -926,7 +926,7 @@ static const MY_CSET_OS_NAME charsets[]=
   {NULL,             NULL,       (my_cs_match_type)0}
 };
 
-#if(!MYSQL8)
+#if(!MYSQLCLIENT_STATIC_LINKING)
 const char *
 my_os_charset_to_mysql_charset(const char *csname)
 {
@@ -1044,7 +1044,7 @@ char *myodbc_ll2str(longlong val, char *dst, int radix)
   return dst - 1;
 }
 
-#if(MYSQLCLIENT_STATIC_LINKING)
+#if(MYSQL8)
 /* We need to use qsort with 2 different compare functions */
 #ifdef QSORT_EXTRA_CMP_ARGUMENT
 #define CMP(A,B) ((*cmp)(cmp_argument,(A),(B)))
@@ -1218,4 +1218,150 @@ void myodbc_qsort(void *base_ptr, size_t count, size_t size, qsort_cmp cmp)
   } while (stack_ptr > stack);
   return;
 }
+
+/*
+  Converts integer to its string representation in decimal notation.
+
+  SYNOPSIS
+    myodbc_int10_to_str()
+      val     - value to convert
+      dst     - points to buffer where string representation should be stored
+      radix   - flag that shows whenever val should be taken as signed or not
+
+  DESCRIPTION
+    This is version of int2str() function which is optimized for normal case
+    of radix 10/-10. It takes only sign of radix parameter into account and
+    not its absolute value.
+
+  RETURN VALUE
+    Pointer to ending NUL character.
+*/
+
+char *myodbc_int10_to_str(long int val, char *dst, int radix) {
+  char buffer[65];
+  char *p;
+  long int new_val;
+  unsigned long int uval = (unsigned long int)val;
+
+  if (radix < 0) /* -10 */
+  {
+    if (val < 0) {
+      *dst++ = '-';
+      /* Avoid integer overflow in (-val) for LLONG_MIN (BUG#31799). */
+      uval = (unsigned long int)0 - uval;
+    }
+  }
+
+  p = &buffer[sizeof(buffer) - 1];
+  *p = '\0';
+  new_val = (long)(uval / 10);
+  *--p = '0' + (char)(uval - (unsigned long)new_val * 10);
+  val = new_val;
+
+  while (val != 0) {
+    new_val = val / 10;
+    *--p = '0' + (char)(val - new_val * 10);
+    val = new_val;
+  }
+  while ((*dst++ = *p++) != 0)
+    ;
+  return dst - 1;
+}
+
+
+bool myodbc_init_dynamic_string(DYNAMIC_STRING *str, const char *init_str,
+                         size_t init_alloc, size_t alloc_increment) {
+  size_t length;
+  DBUG_TRACE;
+
+  if (!alloc_increment) alloc_increment = 128;
+  length = 1;
+  if (init_str && (length = strlen(init_str) + 1) < init_alloc)
+    init_alloc =
+        ((length + alloc_increment - 1) / alloc_increment) * alloc_increment;
+  if (!init_alloc) init_alloc = alloc_increment;
+
+  if (!(str->str = (char *)my_malloc(PSI_NOT_INSTRUMENTED, init_alloc,
+                                     MYF(MY_WME))))
+    return true;
+  str->length = length - 1;
+  if (init_str) memcpy(str->str, init_str, length);
+  str->max_length = init_alloc;
+  str->alloc_increment = alloc_increment;
+  return false;
+}
+
+bool myodbc_append_mem(DYNAMIC_STRING *str, const char *append, size_t length) {
+  char *new_ptr;
+  if (str->length + length >= str->max_length) {
+    size_t new_length =
+        (str->length + length + str->alloc_increment) / str->alloc_increment;
+    new_length *= str->alloc_increment;
+    if (!(new_ptr = (char *)my_realloc(PSI_NOT_INSTRUMENTED, str->str,
+                                       new_length, MYF(MY_WME))))
+      return true;
+    str->str = new_ptr;
+    str->max_length = new_length;
+  }
+  if (length > 0) memcpy(str->str + str->length, append, length);
+  str->length += length;
+  str->str[str->length] = 0; /* Safety for C programs */
+  return false;
+}
+
+bool myodbc_append_os_quoted(DYNAMIC_STRING *str, const char *append, ...) {
+#ifdef _WIN32
+  const char *quote_str = "\"";
+  const uint quote_len = 1;
+#else
+  const char *quote_str = "\'";
+  const uint quote_len = 1;
+#endif /* _WIN32 */
+  bool ret = true;
+  va_list dirty_text;
+
+  ret &= myodbc_append_mem(str, quote_str, quote_len); /* Leading quote */
+  va_start(dirty_text, append);
+  while (append != NullS) {
+    const char *cur_pos = append;
+    const char *next_pos = cur_pos;
+
+    /* Search for quote in each string and replace with escaped quote */
+    while (*(next_pos = strcend(cur_pos, quote_str[0])) != '\0') {
+      ret &= myodbc_append_mem(str, cur_pos, (uint)(next_pos - cur_pos));
+      ret &= myodbc_append_mem(str, "\\", 1);
+      ret &= myodbc_append_mem(str, quote_str, quote_len);
+      cur_pos = next_pos + 1;
+    }
+    ret &= myodbc_append_mem(str, cur_pos, (uint)(next_pos - cur_pos));
+    append = va_arg(dirty_text, char *);
+  }
+  va_end(dirty_text);
+  ret &= myodbc_append_mem(str, quote_str, quote_len); /* Trailing quote */
+
+  return ret;
+}
+
+void myodbc_dynstr_free(DYNAMIC_STRING *str) {
+  my_free(str->str);
+  str->str = NULL;
+}
+
+bool myodbc_dynstr_realloc(DYNAMIC_STRING *str, size_t additional_size) {
+  DBUG_TRACE;
+
+  if (!additional_size) return false;
+  if (str->length + additional_size > str->max_length) {
+    str->max_length =
+        ((str->length + additional_size + str->alloc_increment - 1) /
+         str->alloc_increment) *
+        str->alloc_increment;
+    if (!(str->str = (char *)my_realloc(PSI_NOT_INSTRUMENTED, str->str,
+                                        str->max_length, MYF(MY_WME))))
+      return true;
+  }
+  return false;
+}
+
+
 #endif
\ No newline at end of file
